package pps.core.common.cache;

import lombok.Data;
import xstartup.base.XContext;
import xstartup.cache.XCacheLife;
import xstartup.cache.XCacheObject;
import xstartup.cache.XSingleCache;

import java.util.Date;

/**
 * 第三方Token查询缓存
 *
 * @author ZWT
 * @date 2023/12/12
 */
@Data
public class ThirdPartyConfigCache implements XCacheObject, XCacheLife {
    private String codeKey;
    private String code;
    private Integer validity;
    private Date currentDate;

    /**
     * 检查缓存是否存在指定的code
     *
     * @param context 上下文
     * @param code    密码
     * @return {@link ThirdPartyConfigCache}
     */
    public static ThirdPartyConfigCache exist(XContext context, String code) {
        return XSingleCache.get(Tool.class).find(context, code, ThirdPartyConfigCache.class);
    }

    /**
     * 删去
     *
     * @param context   上下文
     * @param loginName 登录名
     */
    public static void delete(XContext context, String loginName) {
        XSingleCache.get(Tool.class).delete(context, loginName, ThirdPartyConfigCache.class);
    }

    /**
     * 设置缓存
     *
     * @param context 上下文
     * @param cache   隐藏物
     */
    public static void set(XContext context, ThirdPartyConfigCache cache) {
        XSingleCache.get(Tool.class).set(context, cache);
    }

    @Override
    public Integer getDuration() {
        return validity * 60;
    }

    @Override
    public String getCacheKey() {
        return this.codeKey;
    }

    static class Tool extends XSingleCache<ThirdPartyConfigCache> {
        @Override
        protected ThirdPartyConfigCache restore(XContext context, String cacheKey) {
            return null;
        }
    }
}