package pps.core.common.excel.util;

import cn.hutool.core.collection.CollUtil;
import jakarta.validation.ConstraintViolation;
import jakarta.validation.Validation;
import jakarta.validation.Validator;
import jakarta.validation.ValidatorFactory;
import xstartup.base.exception.XServiceException;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

/**
 * 验证util
 *
 * @author ZWT
 * @date 2024/05/10
 */
public class ValidationUtil {

    static Validator validator;

    static {
        ValidatorFactory validatorFactory = Validation.buildDefaultValidatorFactory();
        validator = validatorFactory.getValidator();
    }

    public static Validator getValidator() {
        return validator;
    }

    /**
     * 进行验证
     *
     * @param var1 var1
     * @param var2 var2
     */
    public static <T> void doValidate(List<T> var1, Class<?>... var2) {
        StringBuilder stringBuilder = new StringBuilder();
        int rowNum = 1;
        for (T var0 : var1) {
            rowNum++; // 跳过标题行, 此为excel的行号
            List<String> errMsg = new ArrayList<>();
            Set<ConstraintViolation<T>> validateRstSet = validator.validate(var0, var2);
            for (ConstraintViolation<T> constraintViolation : validateRstSet) {
                errMsg.add(constraintViolation.getMessage());
            }
            if (CollUtil.isNotEmpty(errMsg)) {
                stringBuilder.append("第").append(rowNum).append("行,").append(errMsg).append(';');
            }
        }
        if (stringBuilder.length() > 0) {
            // 组装异常数据
            throw new XServiceException(stringBuilder.toString());
        }
    }
}