package pps.core.common.provider.impl;

import cn.hutool.core.util.ObjectUtil;
import xstartup.annotation.XImplement;
import xstartup.base.XContext;
import xstartup.base.util.XJsonUtils;
import xstartup.core.base.cache.SessionCache;
import xstartup.core.base.provider.spec.XSessionStorageProvider;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;

/**
 * @author lixueyan
 * @date 2022/9/21 0021 18:11
 */

@XImplement
public class PpsSessionStorageProviderImpl implements XSessionStorageProvider {
    public PpsSessionStorageProviderImpl() {
    }

    public <T> XServiceResult saveSession(XContext context, Long userId, T data, Integer expires) {
        SessionCache cache = new SessionCache(userId, XJsonUtils.toMiniJson(data), expires);
        SessionCache.set(context, cache);
        return XServiceResult.OK;
    }

    public <T> XSingleResult<T> loadSession(XContext context, Long userId, Class<T> tClass) {
        SessionCache cache = SessionCache.find(context, userId);
        if (ObjectUtil.isNull(cache)) {
            return XSingleResult.error(context, XError.NotAuth);
        } else {
            T sessionData = XJsonUtils.toObject(cache.getContent(), tClass);
            return XSingleResult.success(sessionData);
        }
    }

    public XServiceResult deleteSession(XContext context, Long userId) {
        SessionCache.delete(context, userId);
        return XServiceResult.OK;
    }

    @Override
    public <T> XServiceResult refreshSession(XContext context, Long userId, Integer expires, Class<T> tClass) {
        SessionCache.tryRefresh(context, userId, expires);
        return XServiceResult.OK;
    }
}