package pps.core.common.utils;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.text.CharSequenceUtil;
import com.google.common.collect.ImmutableList;

import java.util.ArrayList;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.atomic.AtomicLong;

/*
 * 计数构建器
 * */
public class CounterBuilder {
    public static final CounterBuilder globalCounterBuilder = new CounterBuilder();
    public final String counterId = UUID.randomUUID().toString();
    private final ConcurrentHashMap<String, String> fieldValueMap = new ConcurrentHashMap<>();
    private final ConcurrentHashMap<String, AtomicLong> counterMap = new ConcurrentHashMap<>();
    private final ConcurrentHashMap<String, CounterBuildItemInfoType> fileInfoTypeMap = new ConcurrentHashMap<>();
    private String groupName;

    public CounterBuilder() {
        this.setFieldValue("__COUNTERID__", counterId);
        this.setFieldValue("__CREATETIME__", DateUtil.date().toString());
    }

    public CounterBuilder addCounter(String counterName) {
        return this.addCounter(counterName, 1, detectInfoTypeByCounterName(counterName));
    }

    public CounterBuilder addCounter(String counterName, long v, CounterBuildItemInfoType infoType) {
        if (CharSequenceUtil.isBlank(counterName)) {
            return this;
        }
        if (CharSequenceUtil.isNotBlank(this.groupName)) {
            counterName = String.format("%s-%s", this.groupName, counterName);
        }
        if (this.counterMap.containsKey(counterName)) {
            this.counterMap.get(counterName).getAndAdd(v);
            return this;
        }
        this.counterMap.put(counterName, new AtomicLong(v));
        this.fileInfoTypeMap.put(counterName, infoType);
        return this;
    }

    public void setFieldValue(String fieldName, String fieldValue) {
        setFieldValue(fieldName, fieldValue, detectInfoTypeByCounterName(fieldName));
    }

    public void setFieldValue(String fieldName, String fieldValue, CounterBuildItemInfoType infoType) {
        if (CharSequenceUtil.isBlank(fieldName)) {
            return;
        }
        if (CharSequenceUtil.isNotBlank(this.groupName)) {
            fieldName = String.format("%s-%s", this.groupName, fieldName);
        }
        this.fieldValueMap.put(fieldName, fieldValue);
        this.fileInfoTypeMap.put(fieldName, infoType);
    }

    private CounterBuildItemInfoType detectInfoTypeByCounterName(String counterName) {
        if (CharSequenceUtil.containsAny(counterName, "Error", "error", "ERROR", "Exception")) {
            return CounterBuildItemInfoType.ERROR;
        }
        return CounterBuildItemInfoType.NORNAL;
    }

    @Override
    public String toString() {
        ArrayList<String> lines = new ArrayList<>(this.fieldValueMap.size() + this.counterMap.size());
        StringBuilder stringBuilder = new StringBuilder();
        ImmutableList<Map.Entry<String, String>> fieldsList = ImmutableList.copyOf(this.fieldValueMap.entrySet());
        fieldsList.forEach(item -> {
            lines.add(String.format("%s : %s", item.getKey(), item.getValue()));
        });
        ImmutableList<Map.Entry<String, AtomicLong>> fieldsList2 = ImmutableList.copyOf(this.counterMap.entrySet());
        fieldsList2.forEach(item -> {
            lines.add(String.format("%s : %d", item.getKey(), item.getValue().get()));
        });
        for (String line : lines) {
            stringBuilder.append(line).append(System.getProperty("line.separator"));
        }
        return stringBuilder.toString();
    }

    public enum CounterBuildItemInfoType {
        NORNAL,
        ERROR;
    }
}