package pps.core.system.cache;

import xstartup.base.XContext;
import xstartup.cache.XCacheLife;
import xstartup.cache.XCacheObject;
import xstartup.cache.XSingleCache;

/**
 * 登录验证码缓存
 *
 * @author ZWT
 * @date 2024/08/30
 */
public class LoginVerCodeCache implements XCacheObject, XCacheLife {
    private String codeKey;
    private String code;

    /**
     * 检查缓存是否存在指定的code
     *
     * @param context  上下文
     * @param codeIden 代码id
     * @return {@link LoginVerCodeCache }
     */
    public static LoginVerCodeCache exist(XContext context, String codeIden) {
        return Tool.get(Tool.class).find(context, codeIden, LoginVerCodeCache.class);
    }

    /**
     * 设置缓存
     *
     * @param context 上下文
     * @param cache   隐藏物
     */
    public static void set(XContext context, LoginVerCodeCache cache) {
        LoginVerCodeCache.Tool.get(LoginVerCodeCache.Tool.class).set(context, cache);
    }

    /**
     * 删除缓存
     *
     * @param context  上下文
     * @param codeIden 代码id
     */
    public static void delete(XContext context, String codeIden) {
        LoginVerCodeCache.Tool.get(LoginVerCodeCache.Tool.class).delete(context, codeIden, LoginVerCodeCache.class);
    }

    public String getCode() {
        return code;
    }

    public void setCode(String code) {
        this.code = code;
    }

    public String getCodeKey() {
        return codeKey;
    }

    public void setCodeKey(String codeKey) {
        this.codeKey = codeKey;
    }

    @Override
    public Integer getDuration() {
        return 3 * 60;
    }

    @Override
    public String getCacheKey() {
        return this.codeKey;
    }

    static class Tool extends XSingleCache<LoginVerCodeCache> {
        @Override
        protected LoginVerCodeCache restore(XContext context, String cacheKey) {
            return null;
        }
    }
}
