package pps.core.system.service;


import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import pps.cloud.base.service.IConfigOilFieldCloudService;
import pps.cloud.base.service.data.config_oil_field.GetConfigOilFieldOutput;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.VerificationCode;
import pps.core.system.cache.LoginVerCodeCache;
import pps.core.system.entity.SysUserEnt;
import pps.core.system.entity.SysUserLoginLogEnt;
import pps.core.system.error.LoginError;
import pps.core.system.mapper.SysUserLoginLogMapper;
import pps.core.system.mapper.SysUserMapper;
import pps.core.system.service.data.sys_login.*;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.XSsoAuth;
import xstartup.base.XUser;
import xstartup.base.conf.XConfManager;
import xstartup.base.exception.XServiceException;
import xstartup.base.util.XDateUtils;
import xstartup.base.util.XUuidUtils;
import xstartup.core.base.provider.data.XToken;
import xstartup.data.XFileResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.api.conf.XApiCookieConf;
import xstartup.helper.XTokenHelper;

import java.awt.image.BufferedImage;
import java.util.Date;
import java.util.Map;
import java.util.UUID;

@XService
public class LoginService {

    /**
     * 账号密码登录
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult }<{@link LoginByAccountOutput }>
     */
    @XText("账号密码登录")
    @XApiPost(anonymous = true)
    public XSingleResult<LoginByAccountOutput> loginByAccount(XContext context, LoginByAccountInput input) {
        Long count = getErrorTimes(context, input.getLoginName(), null);
        if (count >= 5) {
            return XSingleResult.error(context, LoginError.ISLOCKED);
        }
        checkCode(context, input.getCode(), input.getCodeIden());
        XSingleResult<SysUserEnt> sysUserResult = checkAccount(context, input.getLoginName(), input.getPassword());
        SysUserEnt sysUserEnt = sysUserResult.getResult();
        if (ObjectUtil.isNotNull(sysUserEnt.getStatus()) && sysUserEnt.getStatus().equals(1)) {
            return XSingleResult.error(context, LoginError.ISLOCKED);
        }
        if (ObjectUtil.isNotNull(sysUserEnt.getIsOuter()) && !sysUserEnt.getIsOuter().equals(1)) {
            return XSingleResult.error(context, LoginError.ISOUT);
        }
        XApiCookieConf apiCookieConf = XConfManager.getConf(XApiCookieConf.class);
        Integer expiresIn = apiCookieConf.getExpires();
        XSingleResult<XToken> saveUserResult = XTokenHelper.saveUser(context,
                XUser.createUser(XUser.ENTRY_API,
                        1L,
                        new Long[]{1L, 1L},
                        IdWorker.getId(),
                        sysUserEnt.getId(),
                        sysUserEnt.getLoginName(),
                        sysUserEnt.getUserName(),
                        new XSsoAuth(XUuidUtils.randomUUID(), null, null)),
                expiresIn);
        saveUserResult.throwIfFail();
        LoginByAccountOutput output = new LoginByAccountOutput();
        output.setExpiresIn(expiresIn);
        output.setToken(saveUserResult.getResult().getToken());
        output.setSession(context.getSession(PpsUserSession.class));
        //登录成功后，删除失败记录
        deleteError(context, sysUserEnt);
        return XSingleResult.success(output);
    }

    @XText("检测token是否有效")
    @XApiPost
    public XServiceResult testToken(XContext context) {
        try {
            context.getSession(PpsUserSession.class);
        } catch (XServiceException e) {
            return XServiceResult.error(context, XError.NotAuth);
        }
        return XServiceResult.OK;
    }

    /**
     * 通过模拟登录
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link LoginBySimulateOutput}>
     */
    @XText("模拟用户登陆")
    @XApiGet(anonymous = true)
    public XSingleResult<LoginBySimulateOutput> thirdToken(XContext context, LoginBySimulateInput input) {
        GetConfigOilFieldOutput currentConfig = this.getCurrentConfig(context);
        if (ObjectUtil.isNull(currentConfig) || !CharSequenceUtil.isAllNotBlank(currentConfig.getAuthKey(), currentConfig.getSimulateUser())
                || !CharSequenceUtil.equals(input.getAuthKey(), currentConfig.getAuthKey())) {
            return XSingleResult.error(context, LoginError.Trade_IpWhiteError);
        }
        SysUserMapper mapper = context.getBean(SysUserMapper.class);
        SysUserEnt sysUserEnt = mapper.selectOne(
                new LambdaQueryWrapper<SysUserEnt>()
                        .eq(SysUserEnt::getLoginName, currentConfig.getSimulateUser())
        );
        if (ObjectUtil.isNull(sysUserEnt)) {
            return XSingleResult.error(context, LoginError.NotExistUserName);
        }
        Integer expiresIn = XConfManager.getConf(XApiCookieConf.class).getExpires();
        XSingleResult<XToken> saveUserResult = XTokenHelper.saveUser(context,
                XUser.createUser(XUser.ENTRY_API,
                        BusinessConstant.LOGIN_FLAG_SIMULATE,
                        new Long[]{1L, 1L},
                        IdWorker.getId(),
                        sysUserEnt.getId(),
                        sysUserEnt.getUserName(),
                        sysUserEnt.getUserName(),
                        new XSsoAuth(XUuidUtils.randomUUID(), null, null))
                , expiresIn);
        saveUserResult.throwIfFail();
        return XSingleResult.success(LoginBySimulateOutput.builder()
                .expiresIn(expiresIn)
                .token(saveUserResult.getResult().getToken())
                .build());
    }

    @XText("退出")
    @XApiPost
    public XServiceResult logout(XContext context) {
        XUser user = context.getUser();
        XTokenHelper.deleteUser(context, user);
        context.updateToken(context.getWebRequest().getToken(), 0);
        return XServiceResult.OK;
    }

    @XText("验证码")
    @XApiGet(anonymous = true)
    public XFileResult getVerificationCode(XContext context, VerificationCodeInput input) {
        Map<String, Object> codeMap = VerificationCode.generateCodeAndPic();
        // 将四位数字的验证码保存到Session中。
        LoginVerCodeCache cache = new LoginVerCodeCache();
        cache.setCodeKey(input.getCodeIden());
        cache.setCode(codeMap.get("code").toString());
        LoginVerCodeCache.set(context, cache);
        return XFileResult.createWithPng(input.getCodeIden() + ".png", (BufferedImage) codeMap.get("codePic"));
    }

    /*------------------------------- private -------------------------------*/

    private XSingleResult<SysUserEnt> checkAccount(XContext context, String loginName, String password) {
        SysUserMapper mapper = context.getBean(SysUserMapper.class);
        SysUserEnt sysUserEnt = mapper.selectOne(new LambdaQueryWrapper<SysUserEnt>()
                .eq(SysUserEnt::getLoginName, loginName)
        );
        if (ObjectUtil.isNull(sysUserEnt)) {
            insertError(context, loginName);
            Long count = getErrorTimes(context, loginName, sysUserEnt);
            throw new XServiceException(1000, "用户名或密码错误，还剩" + (5 - count) + "次机会");
        }
        if (CharSequenceUtil.isNotBlank(password)) {
            String decrypt = sysUserEnt.getPassword();
            if (!CharSequenceUtil.equals(password, decrypt)) {
                insertError(context, loginName);
                Long count = getErrorTimes(context, loginName, sysUserEnt);
                throw new XServiceException(1000, "用户名或密码错误，还剩" + (5 - count) + "次机会");
            }
        }
        return XSingleResult.success(sysUserEnt);
    }

    private void checkCode(XContext context, String code, String codeIden) {
        if (CharSequenceUtil.isBlank(code)) {
            throw new XServiceException(context, LoginError.NotExistCode);
        }
        LoginVerCodeCache isUse = LoginVerCodeCache.exist(context, codeIden);
        if (ObjectUtil.isNull(isUse)) {
            throw new XServiceException(context, LoginError.NotUsedCode);
        } else {
            //清除验证码
            LoginVerCodeCache.delete(context, codeIden);
        }
        if (!CharSequenceUtil.equalsAnyIgnoreCase(isUse.getCode(), code)) {
            throw new XServiceException(context, LoginError.NotCompareCode);
        }
    }

    private void deleteError(XContext context, SysUserEnt sysUserEnt) {
        //删除登录日志错误信息
        SysUserLoginLogMapper userLoginLogMapper = context.getBean(SysUserLoginLogMapper.class);
        QueryWrapper<SysUserLoginLogEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysUserLoginLogEnt::getLoginName, sysUserEnt.getLoginName());
        userLoginLogMapper.delete(queryWrapper);
        SysUserMapper mapper = context.getBean(SysUserMapper.class);
        sysUserEnt.setStatus(0);
        sysUserEnt.setModifyTime(new Date());
        mapper.updateById(sysUserEnt);
    }

    private void insertError(XContext context, String loginName) {
        //记录登录日志错误信息
        SysUserLoginLogMapper userLoginLogMapper = context.getBean(SysUserLoginLogMapper.class);
        SysUserLoginLogEnt logEnt = new SysUserLoginLogEnt();
        logEnt.setId(UUID.randomUUID().toString());
        logEnt.setErrorMsg("账户、密码错误");
        logEnt.setLoginName(loginName);
        logEnt.setCreateTime(new Date());
        userLoginLogMapper.insert(logEnt);
    }

    private Long getErrorTimes(XContext context, String loginName, SysUserEnt sysUserEnt) {
        SysUserLoginLogMapper userLoginLogMapper = context.getBean(SysUserLoginLogMapper.class);
        QueryWrapper<SysUserLoginLogEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysUserLoginLogEnt::getLoginName, loginName);
        queryWrapper.lambda().ge(SysUserLoginLogEnt::getCreateTime, XDateUtils.getDayBeginTime(new Date()));
        Long count = userLoginLogMapper.selectCount(queryWrapper);
        if (count >= 5 && ObjectUtil.isNotNull(sysUserEnt)) {
            SysUserMapper mapper = context.getBean(SysUserMapper.class);
            sysUserEnt.setStatus(2);
            sysUserEnt.setModifyTime(new Date());
            mapper.updateById(sysUserEnt);
        }
        return count;
    }

    /**
     * 获取当前配置
     *
     * @param context 上下文
     * @return {@link GetConfigOilFieldOutput }
     */
    private GetConfigOilFieldOutput getCurrentConfig(XContext context) {
        IConfigOilFieldCloudService bean = context.getBean(IConfigOilFieldCloudService.class);
        XSingleResult<GetConfigOilFieldOutput> currentConfig = bean.getCurrentConfig(context);
        currentConfig.throwIfFail();
        return currentConfig.getResult();
    }
}