package pps.core.system.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import pps.core.common.constant.BusinessConstant;
import pps.core.system.cache.SysAreaCache;
import pps.core.system.entity.SysAreaEnt;
import pps.core.system.entity.SysAreaView;
import pps.core.system.mapper.SysAreaMapper;
import pps.core.system.mapper.SysAreaViewMapper;
import pps.core.system.service.data.sys_area.*;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.data.CustomQueryInput;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;

import java.util.ArrayList;
import java.util.List;

@XService
@XText("地区管理")
public class SysAreaService {

    @XText("查询地区树")
    @XApiPost
    public XListResult<GetSysAreaViewOutputTree> getSysAreaTree(XContext context, GetSysAreaInput input) {
        //先从缓存中获取
        List<SysAreaCache> sysAreaCaches = SysAreaCache.list(context, BusinessConstant.SYS_AREA_CACHE);
        if (ObjectUtil.isNotNull(input.getLev()) && input.getLev() == 1) {
            List<SysAreaCache> items = new ArrayList<>();
            for (SysAreaCache ent : sysAreaCaches) {
                if (CollUtil.isNotEmpty(ent.getChildren())) {
                    items.addAll(ent.getChildren());
                }
            }
            sysAreaCaches = items;
        }
        //缓存不存在从数据库获取
        if (CollUtil.isEmpty(sysAreaCaches)) {
            List<SysAreaCache> caches = selectSysAreaTree(context, input);
            List<GetSysAreaViewOutputTree> resultTree = XCopyUtils.copyNewList(caches, GetSysAreaViewOutputTree.class);
            if (ObjectUtil.isNotNull(input.getLev()) && input.getLev() == 1) {
                List<GetSysAreaViewOutputTree> items = new ArrayList<>();
                for (GetSysAreaViewOutputTree ent : resultTree) {
                    if (CollUtil.isNotEmpty(ent.getChildren())) {
                        items.addAll(ent.getChildren());
                    }
                }
                resultTree = items;
            }
            return XListResult.success(resultTree);
        }
        List<GetSysAreaViewOutputTree> resultTree = XCopyUtils.copyNewList(sysAreaCaches, GetSysAreaViewOutputTree.class);
        return XListResult.success(resultTree);
    }

    private List<SysAreaCache> selectSysAreaTree(XContext context, GetSysAreaInput input) {
        SysAreaViewMapper mapper = context.getBean(SysAreaViewMapper.class);
        SysAreaView record = new SysAreaView();
        XCopyUtils.copyObject(input, record);
        List<SysAreaView> areaViews = mapper.selectListAll(record);
        List<GetSysAreaViewOutputTree> list = XCopyUtils.copyNewList(areaViews, GetSysAreaViewOutputTree.class);
        List<GetSysAreaViewOutputTree> trees = new ArrayList<>(list.size());
        for (GetSysAreaViewOutputTree tree : list) {
            if (tree.getLev().equals(0)) {
                trees.add(tree);
            }
            //子集
            for (GetSysAreaViewOutputTree it : list) {
                if (tree.getId().equals(it.getParentId())) {
                    if (CollUtil.isEmpty(tree.getChildren())) {
                        tree.setChildren(new ArrayList<>());
                    }
                    tree.getChildren().add(it);
                }
            }
        }
        List<SysAreaCache> caches = XCopyUtils.copyNewList(trees, SysAreaCache.class);
        //放入redis缓存中
        SysAreaCache.set(context, BusinessConstant.SYS_AREA_CACHE, caches);
        return caches;
    }

    @XApiPost
    public XServiceResult createSysArea(XContext context, CreateSysAreaInput input) {
        SysAreaMapper mapper = context.getBean(SysAreaMapper.class);
        SysAreaEnt entity = new SysAreaEnt();
        XCopyUtils.copyObject(input, entity);
        mapper.insert(entity);
        return XServiceResult.OK;
    }

    @XApiPost
    public XServiceResult updateSysArea(XContext context, UpdateSysAreaInput input) {
        SysAreaMapper mapper = context.getBean(SysAreaMapper.class);
        SysAreaEnt entity = mapper.selectOne(new QueryWrapper<SysAreaEnt>()
                .lambda()
                .eq(SysAreaEnt::getId, input.getId()));
        if (ObjectUtil.isNull(entity)) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XApiPost
    public XServiceResult deleteSysArea(XContext context, DeleteSysAreaInput input) {
        SysAreaMapper mapper = context.getBean(SysAreaMapper.class);
        SysAreaEnt entity = mapper.selectOne(new QueryWrapper<SysAreaEnt>()
                .lambda()
                .eq(SysAreaEnt::getId, input.getId()));
        if (ObjectUtil.isNull(entity)) {
            return XServiceResult.error(context, XError.NotFound);
        }
        mapper.deleteById(entity);
        return XServiceResult.OK;
    }

    @XApiGet
    public XSingleResult<GetSysAreaOutput> getSysArea(XContext context, GetSysAreaInput input) {
        SysAreaMapper mapper = context.getBean(SysAreaMapper.class);
        SysAreaEnt entity = mapper.selectOne(new QueryWrapper<SysAreaEnt>()
                .lambda()
                .eq(SysAreaEnt::getId, input.getId()));
        if (ObjectUtil.isNull(entity)) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetSysAreaOutput output = new GetSysAreaOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XApiGet
    public XPageResult<QuerySysAreaOutput> querySysArea(XContext context, QuerySysAreaInput input) {
        SysAreaMapper mapper = context.getBean(SysAreaMapper.class);
        QueryWrapper<SysAreaEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysAreaEnt::getId, input.getId())
                .like(SysAreaEnt::getName, input.getName())
                .eq(SysAreaEnt::getParentId, input.getParentId())
                .eq(SysAreaEnt::getLev, input.getLev());
        return XMapperHelper.query(mapper, input, queryWrapper, QuerySysAreaOutput.class);
    }

    @XApiGet
    public XPageResult<QuerySysAreaOutput> dynamicQuerySysArea(XContext context, CustomQueryInput input) {
        SysAreaMapper mapper = context.getBean(SysAreaMapper.class);
        return XMapperHelper.query(mapper, input, SysAreaEnt.class, QuerySysAreaOutput.class);
    }

    @XApiGet
    public XSingleResult<GetSysAreaViewOutput> getSysAreaView(XContext context, GetSysAreaViewInput input) {
        SysAreaViewMapper mapper = context.getBean(SysAreaViewMapper.class);
        SysAreaView record = new SysAreaView();
        XCopyUtils.copyObject(input, record);
        SysAreaView view = mapper.selectOne(record);
        if (ObjectUtil.isNull(view)) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetSysAreaViewOutput output = new GetSysAreaViewOutput();
        XCopyUtils.copyObject(view, output);
        return XSingleResult.success(output);
    }

    @XApiGet
    public XPageResult<QuerySysAreaViewOutput> querySysAreaView(XContext context, QuerySysAreaViewInput input) {
        SysAreaViewMapper mapper = context.getBean(SysAreaViewMapper.class);
        SysAreaView record = new SysAreaView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<SysAreaView> list = mapper.selectList(record);
        PageInfo<SysAreaView> pageInfo = new PageInfo<>(list);
        List<QuerySysAreaViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySysAreaViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }
}