package pps.core.system.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import pps.core.common.session.PpsUserSession;
import pps.core.system.entity.SysAuthOuScopeEnt;
import pps.core.system.entity.SysAuthOuScopeView;
import pps.core.system.mapper.SysAuthOuScopeMapper;
import pps.core.system.mapper.SysAuthOuScopeViewMapper;
import pps.core.system.service.data.sys_auth_ou_scope.BatchDeleteSysAuthOuScopeInput;
import pps.core.system.service.data.sys_auth_ou_scope.CreateSysAuthOuScopeInput;
import pps.core.system.service.data.sys_auth_ou_scope.QuerySysAuthOuScopeViewInput;
import pps.core.system.service.data.sys_auth_ou_scope.QuerySysAuthOuScopeViewOutput;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.exception.XServiceException;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.helper.XTransactionHelper;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

@XText("用户或用户组授权管理")
@XService
public class SysAuthOuScopeService {

    @XText("创建用户和菜单、组织机构关系表")
    @XApiPost
    public XServiceResult createSysAuthOuScope(XContext context, CreateSysAuthOuScopeInput input) throws XServiceException {
        return XTransactionHelper.begin(context, () -> {
            SysAuthOuScopeMapper sysAuthOuScopeMapper = context.getBean(SysAuthOuScopeMapper.class);
            SysAuthOuScopeViewMapper sysAuthOuScopeViewMapper = context.getBean(SysAuthOuScopeViewMapper.class);
            //删除原关系
            List<SysAuthOuScopeEnt> sysAuthOuScopeEnts = sysAuthOuScopeMapper.selectList(new QueryWrapper<SysAuthOuScopeEnt>()
                    .lambda()
                    .eq(SysAuthOuScopeEnt::getUserId, input.getUserId()));
            List<SysAuthOuScopeEnt> deleteList = new ArrayList<>();
            //查询已存在的组织机构和功能
            if (CollUtil.isNotEmpty(sysAuthOuScopeEnts)) {
                for (SysAuthOuScopeEnt scopeEnt : sysAuthOuScopeEnts) {
                    List<String> ouIds = input.getOuIds();
                    for (String ouId : ouIds) {
                        List<String> menuIds = input.getMenuIds();
                        for (String menuId : menuIds) {
                            if (ObjectUtil.isNotNull(scopeEnt)) {
                                if (CharSequenceUtil.equals(ouId, scopeEnt.getOuId()) && CharSequenceUtil.equals(menuId, scopeEnt.getMenuId())) {
                                    deleteList.add(scopeEnt);
                                }
                            }
                        }
                    }
                }
            }
            if (CollUtil.isNotEmpty(deleteList)) {
                List<String> collect = deleteList.stream().map(SysAuthOuScopeEnt::getId).collect(Collectors.toList());
                sysAuthOuScopeMapper.deleteBatchIds(collect);
            }
            //创建新关系
            PpsUserSession session = context.getSession(PpsUserSession.class);
            List<SysAuthOuScopeView> viewList = new ArrayList<>();
            List<String> ouIds = input.getOuIds();
            for (String ouId : ouIds) {
                List<String> menuIds = input.getMenuIds();
                for (String menuId : menuIds) {
                    SysAuthOuScopeView entity = new SysAuthOuScopeView();
                    entity.setMenuId(menuId);
                    entity.setOuId(ouId);
                    entity.setUserId(input.getUserId());
                    entity.setId(UUID.randomUUID().toString());
                    entity.setCreateById(session.getId());
                    entity.setCreateByName(session.getUserName());
                    entity.setCreateTime(new Date());
                    viewList.add(entity);
                }
            }
            if (CollUtil.isNotEmpty(viewList)) {
                sysAuthOuScopeViewMapper.batchInsert(viewList);
            }
            //返回执行成功的结果，事务提交
            return XServiceResult.OK;
        });
    }

    @XText("批量删除")
    @XApiPost
    public XServiceResult batchDeleteSysAuthOuScope(XContext context, BatchDeleteSysAuthOuScopeInput input) {
        List<String> idList = input.getIdList();
        if (ObjectUtil.isNotNull(idList)) {
            idList = idList.stream()
                    .filter(CharSequenceUtil::isNotBlank)
                    .collect(Collectors.toList());
        }
        if (CollUtil.isEmpty(idList)) {
            return XServiceResult.error(context, XError.NotFound);
        }
        SysAuthOuScopeMapper mapper = context.getBean(SysAuthOuScopeMapper.class);
        mapper.deleteBatchIds(idList);
        return XServiceResult.OK;
    }

    @XText("查询管理授权")
    @XApiPost
    public XPageResult<QuerySysAuthOuScopeViewOutput> querySysAuthOuScopeView(XContext context, QuerySysAuthOuScopeViewInput input) {
        SysAuthOuScopeViewMapper mapper = context.getBean(SysAuthOuScopeViewMapper.class);
        SysAuthOuScopeView record = new SysAuthOuScopeView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<SysAuthOuScopeView> list = mapper.selectList(record);
        PageInfo<SysAuthOuScopeView> pageInfo = new PageInfo<>(list);
        List<QuerySysAuthOuScopeViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySysAuthOuScopeViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    @XText("根据用户组id查询")
    @XApiPost
    public XPageResult<QuerySysAuthOuScopeViewOutput> querySysAuthOuScopeViewByUserGroupId(XContext context, QuerySysAuthOuScopeViewInput input) {
        SysAuthOuScopeViewMapper mapper = context.getBean(SysAuthOuScopeViewMapper.class);
        SysAuthOuScopeView record = new SysAuthOuScopeView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<SysAuthOuScopeView> list = mapper.selectListByUserGroupId(record);
        PageInfo<SysAuthOuScopeView> pageInfo = new PageInfo<>(list);
        List<QuerySysAuthOuScopeViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySysAuthOuScopeViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }
}