package pps.core.system.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import pps.cloud.system.service.SystemDictionaryService;
import pps.cloud.system.service.data.sys_dictionary.*;
import pps.core.system.cache.SysDictionaryCache;
import pps.core.system.entity.SysDictionaryEnt;
import pps.core.system.entity.SysDictionaryView;
import pps.core.system.entity.SysOrganizationDictionaryEnt;
import pps.core.system.entity.SysOrganizationRelEnt;
import pps.core.system.mapper.SysDictionaryMapper;
import pps.core.system.mapper.SysDictionaryViewMapper;
import pps.core.system.mapper.SysOrganizationDictionaryMapper;
import pps.core.system.mapper.SysOrganizationRelMapper;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@XService
public class SysDictionaryServiceImpl implements SystemDictionaryService {

    @Override
    public XListResult<QuerySysDictionaryViewOutput> querySysDictionarysByAlias(XContext context, CreateSysDictionaryInput input) {
        SysDictionaryMapper mapper = context.getBean(SysDictionaryMapper.class);
        SysOrganizationDictionaryMapper orgDicmapper = context.getBean(SysOrganizationDictionaryMapper.class);
        SysOrganizationRelMapper orgRelMapper = context.getBean(SysOrganizationRelMapper.class);
        List<SysDictionaryCache> list = SysDictionaryCache.list(context, input.getAlias());
        SysDictionaryEnt sysDictionaryEnt = mapper.selectOne(new QueryWrapper<SysDictionaryEnt>()
                .lambda()
                .eq(SysDictionaryEnt::getAlias, input.getAlias()));
        list = list.stream().filter((SysDictionaryCache dic) -> CharSequenceUtil.equals(dic.getPid(), sysDictionaryEnt.getId()) && !CharSequenceUtil.equals(dic.getAlias(), input.getAlias()))
                .collect(Collectors.toList());
        if (CollUtil.isEmpty(list)) {
            list = new ArrayList<>();
            SysDictionaryCache.set(context, input.getAlias(), list);
            //查询alias字典
            SysDictionaryEnt sysDictionary = mapper.selectOne(new QueryWrapper<SysDictionaryEnt>()
                    .lambda()
                    .eq(SysDictionaryEnt::getAlias, input.getAlias())
                    .gt(SysDictionaryEnt::getEndTime, new Date()));
            if (ObjectUtil.isNotNull(sysDictionary)) {
                SysDictionaryCache sysDictionaryCache = new SysDictionaryCache();
                XCopyUtils.copyObject(sysDictionary, sysDictionaryCache);
                list.add(sysDictionaryCache);
                //查询alias字典下级字典项
                QueryWrapper<SysDictionaryEnt> queryItems = new QueryWrapper<>();
                if (ObjectUtil.isNotNull(sysDictionary.getIsOrg()) && sysDictionary.getIsOrg() == 1) {
                    //查询当前登录人的组织机构层级路径
                    SysOrganizationRelEnt sysOrganizationRelEnt = orgRelMapper.selectOne(new QueryWrapper<SysOrganizationRelEnt>()
                            .lambda()
                            .eq(SysOrganizationRelEnt::getOuId, input.getOuId())
                            .gt(SysOrganizationRelEnt::getEndTime, new Date()));
                    //根据字典id查询
                    List<SysOrganizationDictionaryEnt> sysOrganizationDictionaryEnts = orgDicmapper.selectList(new QueryWrapper<SysOrganizationDictionaryEnt>()
                            .lambda()
                            .eq(SysOrganizationDictionaryEnt::getParentDicCode, sysDictionary.getId())
                            .gt(SysOrganizationDictionaryEnt::getEndTime, new Date()));
                    String ouIdPath = sysOrganizationRelEnt.getOuIdPath();
                    String ouId = sysOrganizationRelEnt.getOuId();
                    if (CharSequenceUtil.isNotBlank(ouIdPath)) {
                        List<String> dicIds = sysOrganizationDictionaryEnts.stream().map(p -> {
                            if (CharSequenceUtil.contains(ouIdPath, p.getOuId()) && p.getUseLowerLevel() == 1) {
                                return p.getDicCode();
                            } else if (CharSequenceUtil.contains(ouId, p.getOuId())) {
                                return p.getDicCode();
                            } else {
                                return "1";
                            }
                        }).collect(Collectors.toList());
                        queryItems.lambda().in(SysDictionaryEnt::getId, dicIds);
                    }
                }
                queryItems.lambda().eq(SysDictionaryEnt::getPid, sysDictionaryCache.getId()).gt(SysDictionaryEnt::getEndTime, new Date());
                List<SysDictionaryEnt> sysDictionaryList = mapper.selectList(queryItems);
                if (CollUtil.isNotEmpty(sysDictionaryList)) {
                    if (CollUtil.isNotEmpty(list)) {
                        for (SysDictionaryEnt ent : sysDictionaryList) {
                            SysDictionaryCache cache = new SysDictionaryCache();
                            XCopyUtils.copyObject(ent, cache);
                            list.add(cache);
                        }
                    }
                    sysDictionaryList.sort((o1, o2) -> {
                        if (ObjectUtil.isNotNull(o1.getSort()) && ObjectUtil.isNotNull(o2.getSort())) {
                            return o1.getSort() - o2.getSort();
                        } else {
                            return 0;
                        }
                    });
                    List<QuerySysDictionaryViewOutput> outputs = XCopyUtils.copyNewList(sysDictionaryList, QuerySysDictionaryViewOutput.class);
                    return XListResult.success(outputs);
                }
            }
        }
        if (ObjectUtil.isNotNull(sysDictionaryEnt) && ObjectUtil.isNotNull(sysDictionaryEnt.getIsOrg()) && sysDictionaryEnt.getIsOrg() == 1) {
            //查询当前登录人的组织机构层级路径
            SysOrganizationRelEnt sysOrganizationRelEnt = orgRelMapper.selectOne(new QueryWrapper<SysOrganizationRelEnt>()
                    .lambda()
                    .eq(SysOrganizationRelEnt::getOuId, input.getOuId())
                    .gt(SysOrganizationRelEnt::getEndTime, new Date()));
            //根据字典id查询
            List<SysOrganizationDictionaryEnt> sysOrganizationDictionaryEnts = orgDicmapper.selectList(new QueryWrapper<SysOrganizationDictionaryEnt>()
                    .lambda()
                    .eq(SysOrganizationDictionaryEnt::getParentDicCode, sysDictionaryEnt.getId())
                    .gt(SysOrganizationDictionaryEnt::getEndTime, new Date()));
            String ouIdPath = sysOrganizationRelEnt.getOuIdPath();
            String ouId = sysOrganizationRelEnt.getOuId();
            if (CharSequenceUtil.isNotBlank(ouIdPath)) {
                List<String> dicIds = sysOrganizationDictionaryEnts.stream().map(p -> {
                    if (CharSequenceUtil.contains(ouIdPath, p.getOuId()) && p.getUseLowerLevel() == 1) {
                        return p.getDicCode();
                    } else if (CharSequenceUtil.contains(ouId, p.getOuId())) {
                        return p.getDicCode();
                    } else {
                        return "1";
                    }
                }).collect(Collectors.toList());
                list = list.stream().filter(i -> CollUtil.contains(dicIds, i.getId())).collect(Collectors.toList());
            }
        }
        list.sort((o1, o2) -> {
            if (ObjectUtil.isNotNull(o1.getSort()) && ObjectUtil.isNotNull(o2.getSort())) {
                return o1.getSort() - o2.getSort();
            } else {
                return 0;
            }
        });
        List<QuerySysDictionaryViewOutput> outputs = XCopyUtils.copyNewList(list, QuerySysDictionaryViewOutput.class);
        return XListResult.success(outputs);
    }

    @Override
    public XListResult<QuerySysDictionaryViewOutput> queryChildSysDictionarysByParentAlias(XContext context, QuerySysDictionaryViewInput input) {
        List<QuerySysDictionaryViewOutput> resultList = new ArrayList<>();
        SysDictionaryMapper sysDictionaryMapper = context.getBean(SysDictionaryMapper.class);
        List<SysDictionaryEnt> list = sysDictionaryMapper.selectList(new QueryWrapper<SysDictionaryEnt>().eq("alias", input.getAlias()));
        if (CollUtil.isEmpty(list)) {
            return XListResult.success(resultList);
        }
        SysDictionaryEnt parentDictionaryEnt = list.get(0);
        list = sysDictionaryMapper.selectList(new QueryWrapper<SysDictionaryEnt>().eq("pid", parentDictionaryEnt.getId()));
        resultList = XCopyUtils.copyNewList(list, QuerySysDictionaryViewOutput.class);
        return XListResult.success(resultList);
    }

    @Override
    public XListResult<GetSysDictionaryViewOutput> querySysDictionaryByParam(XContext context, GetSysDictionaryViewInput input) {
        SysDictionaryViewMapper mapper = context.getBean(SysDictionaryViewMapper.class);
        List<SysDictionaryView> list = mapper.selectListByParam(SysDictionaryView.builder()
                .pid(input.getPid())
                .alias(input.getAlias())
                .build());
        return XListResult.success(XCopyUtils.copyNewList(list, GetSysDictionaryViewOutput.class));
    }
}