package pps.core.system.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import pps.cloud.system.service.SystemDictionaryService;
import pps.cloud.system.service.data.sys_dictionary.CreateSysDictionaryInput;
import pps.cloud.system.service.data.sys_dictionary.QuerySysDictionaryViewOutput;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.TreeUtils;
import pps.core.system.entity.SysMenuEnt;
import pps.core.system.entity.SysMenuView;
import pps.core.system.error.MenuError;
import pps.core.system.mapper.SysMenuMapper;
import pps.core.system.mapper.SysMenuViewMapper;
import pps.core.system.service.data.sys_menu.*;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.data.CustomQueryInput;
import xstartup.base.exception.XServiceException;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XStringUtils;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;
import xstartup.helper.XTransactionHelper;

import java.util.*;
import java.util.stream.Collectors;

@XText("菜单管理")
@XService
public class SysMenuService {

    @XText("角色列表使用的菜单树")
    @XApiPost
    public XListResult<QuerySysMenuOutputTree> getSysMenuTreeRole(XContext context, GetSysMenuInput input) {
        SysMenuViewMapper mapper = context.getBean(SysMenuViewMapper.class);
        SystemDictionaryService dictService = context.getBean(SystemDictionaryService.class);
        SysMenuView record = new SysMenuView();
        XCopyUtils.copyObject(input, record);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        Integer isSuperAdmin = session.getIsSuperAdmin();
        List<SysMenuView> viewList;
        //普通角色且用户所属ou和查看的角色所属ou一样时返回当前用户可操作的菜单
        if (BusinessConstant.ZERO.equals(isSuperAdmin) && CharSequenceUtil.equals(session.getOuId(), input.getOuId())) {
            record.setUserId(session.getId());
            viewList = mapper.selectSysMenuListByUserId(record);
        } else {
            viewList = mapper.selectSysMenuList(record);
        }
        List<QuerySysMenuOutputTree> list = XCopyUtils.copyNewList(viewList, QuerySysMenuOutputTree.class);
        //介质类型
        CreateSysDictionaryInput mediaTypesInput = new CreateSysDictionaryInput();
        mediaTypesInput.setAlias("MEDIUM_TYPE");
        XListResult<QuerySysDictionaryViewOutput> mediaTypes = dictService.querySysDictionarysByAlias(context, mediaTypesInput);
        for (QuerySysMenuOutputTree tree : list) {
            //介质类型
            if (CharSequenceUtil.isNotBlank(tree.getMediaType())) {
                if (CollUtil.isNotEmpty(mediaTypes.getResult())) {
                    StringBuilder buffer = new StringBuilder();
                    for (String mediaType : CharSequenceUtil.splitToArray(tree.getMediaType(), ',')) {
                        for (QuerySysDictionaryViewOutput item : mediaTypes.getResult()) {
                            if (CharSequenceUtil.equals(mediaType, item.getDicKey())) {
                                buffer.append(",").append(item.getDicName());
                            }
                        }
                    }
                    if (buffer.length() > 0) {
                        tree.setMediaTypeName(buffer.substring(1));
                    }
                }
                if (CharSequenceUtil.isBlank(tree.getMediaTypeName())) {
                    tree.setMediaTypeName(tree.getMediaType());
                }
            }
        }
        List<QuerySysMenuOutputTree> trees = new ArrayList<>(list.size());
        for (QuerySysMenuOutputTree tree : list) {
            if (CharSequenceUtil.isBlank(tree.getParentMenuId())) {
                trees.add(tree);
            }
            //子集
            for (QuerySysMenuOutputTree it : list) {
                if (CharSequenceUtil.equals(tree.getId(), it.getParentMenuId())) {
                    if (CollUtil.isEmpty(tree.getChildren())) {
                        tree.setChildren(new ArrayList<>());
                    }
                    tree.getChildren().add(it);
                }
            }
            if (CollUtil.isNotEmpty(tree.getChildren())) {
                List<QuerySysMenuOutputTree> children = tree.getChildren();
                children.sort(Comparator.comparing(QuerySysMenuOutputTree::getSort));
                tree.setChildren(children);
            }
        }
        trees.sort(Comparator.comparing(QuerySysMenuOutputTree::getSort));
        return XListResult.success(trees);
    }

    @XText("根据当前用户查询菜单列表树")
    @XApiPost
    public XListResult<QuerySysMenuOutputTree> getSysMenuTreeByUserId(XContext context, GetSysMenuInput input) {
        SysMenuViewMapper mapper = context.getBean(SysMenuViewMapper.class);
        SystemDictionaryService dictService = context.getBean(SystemDictionaryService.class);
        SysMenuView record = new SysMenuView();
        XCopyUtils.copyObject(input, record);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        record.setUserId(session.getId());
        record.setMediaType(input.getOilGasType());
        record.setApplyScene(input.getApplyScene());
        record.setOuId(session.getOuId());
        List<SysMenuView> viewList = mapper.selectSysMenuListByUserIdAndMediaType(record);
        List<QuerySysMenuOutputTree> list = XCopyUtils.copyNewList(viewList, QuerySysMenuOutputTree.class);
        //介质类型
        CreateSysDictionaryInput mediaTypesInput = new CreateSysDictionaryInput();
        mediaTypesInput.setAlias("MEDIUM_TYPE");
        XListResult<QuerySysDictionaryViewOutput> mediaTypes = dictService.querySysDictionarysByAlias(context, mediaTypesInput);
        for (QuerySysMenuOutputTree tree : list) {
            if (CharSequenceUtil.isNotBlank(tree.getMenuType())) {
                if (CharSequenceUtil.isBlank(tree.getBusinessTypeName())) {
                    tree.setBusinessTypeName(tree.getMenuType());
                }
                //介质类型
                if (CollUtil.isNotEmpty(mediaTypes.getResult())) {
                    StringBuilder buffer = new StringBuilder();
                    for (String mediaType : CharSequenceUtil.splitToArray(tree.getMediaType(), ',')) {
                        for (QuerySysDictionaryViewOutput item : mediaTypes.getResult()) {
                            if (CharSequenceUtil.equals(mediaType, item.getDicKey())) {
                                buffer.append(",").append(item.getDicName());
                            }
                        }
                    }
                    if (buffer.length() > 0) {
                        tree.setMediaTypeName(buffer.substring(1));
                    }
                }
                if (CharSequenceUtil.isBlank(tree.getMediaTypeName())) {
                    tree.setMediaTypeName(tree.getMediaType());
                }
            }
        }
        List<QuerySysMenuOutputTree> trees = new ArrayList<>(list.size());
        for (QuerySysMenuOutputTree tree : list) {
            if (CharSequenceUtil.isBlank(tree.getParentMenuId())) {
                trees.add(tree);
            }
            //子集
            for (QuerySysMenuOutputTree it : list) {
                if (CharSequenceUtil.equals(tree.getId(), it.getParentMenuId())) {
                    if (CollUtil.isEmpty(tree.getChildren())) {
                        //初始化集合
                        tree.setChildren(new ArrayList<>());
                    }
                    tree.getChildren().add(it);
                }
            }
            if (CollUtil.isNotEmpty(tree.getChildren())) {
                List<QuerySysMenuOutputTree> children = tree.getChildren();
                children.sort(Comparator.comparing(QuerySysMenuOutputTree::getSort));
                tree.setChildren(children);
            }
        }
        trees.sort(Comparator.comparing(QuerySysMenuOutputTree::getSort));
        return XListResult.success(trees);
    }

    @XText("查询菜单列表树")
    @XApiPost
    public XListResult<QuerySysMenuOutputTree> getSysMenuTree(XContext context, GetSysMenuInput input) {
        SysMenuViewMapper mapper = context.getBean(SysMenuViewMapper.class);
        SystemDictionaryService dictService = context.getBean(SystemDictionaryService.class);
        SysMenuView record = new SysMenuView();
        XCopyUtils.copyObject(input, record);
        List<SysMenuView> viewList;
        if (CharSequenceUtil.isNotBlank(record.getMenuName())) {
            viewList = mapper.selectSysMenuListByMenuName(record);
        } else {
            viewList = mapper.selectSysMenuList(record);
        }
        List<QuerySysMenuOutputTree> list = XCopyUtils.copyNewList(viewList, QuerySysMenuOutputTree.class);
        //介质类型
        CreateSysDictionaryInput mediaTypesInput = new CreateSysDictionaryInput();
        mediaTypesInput.setAlias("MEDIUM_TYPE");
        XListResult<QuerySysDictionaryViewOutput> mediaTypes = dictService.querySysDictionarysByAlias(context, mediaTypesInput);
        for (QuerySysMenuOutputTree tree : list) {
            //介质类型
            if (CharSequenceUtil.isNotBlank(tree.getMediaType())) {
                if (CollUtil.isNotEmpty(mediaTypes.getResult())) {
                    StringBuilder buffer = new StringBuilder();
                    for (String mediaType : CharSequenceUtil.splitToArray(tree.getMediaType(), ',')) {
                        for (QuerySysDictionaryViewOutput item : mediaTypes.getResult()) {
                            if (mediaType.equals(item.getDicKey())) {
                                buffer.append(",").append(item.getDicName());
                            }
                        }
                    }
                    if (buffer.length() > 0) {
                        tree.setMediaTypeName(buffer.substring(1));
                    }
                }
                if (CharSequenceUtil.isBlank(tree.getMediaTypeName())) {
                    tree.setMediaTypeName(tree.getMediaType());
                }
            }
        }
        List<QuerySysMenuOutputTree> trees = new ArrayList<>();
        for (QuerySysMenuOutputTree tree : list) {
            if (CharSequenceUtil.isBlank(tree.getParentMenuId())) {
                trees.add(tree);
            }
            //子集
            for (QuerySysMenuOutputTree it : list) {
                if (CharSequenceUtil.equals(tree.getId(), it.getParentMenuId())) {
                    if (CollUtil.isEmpty(tree.getChildren())) {
                        tree.setChildren(new ArrayList<>());
                    }
                    tree.getChildren().add(it);
                }
            }
            if (CollUtil.isNotEmpty(tree.getChildren())) {
                List<QuerySysMenuOutputTree> children = tree.getChildren();
                children.sort(Comparator.comparing(QuerySysMenuOutputTree::getSort));
                tree.setChildren(children);
            }
        }
        trees.sort(Comparator.comparing(QuerySysMenuOutputTree::getSort));
        return XListResult.success(trees);
    }

    @XText("查询菜单列表树-异步")
    @XApiPost
    public XListResult<QuerySysMenuOutputTree> getSysMenuTreeAsyn(XContext context, GetSysMenuInput input) {
        if (XStringUtils.isNotEmpty(input.getMenuName()))
            return getSysMenuNameTree(context, input);
        else
            return getSysMenuListTree(context, input);
    }

    public XListResult<QuerySysMenuOutputTree> getSysMenuListTree(XContext context, GetSysMenuInput input) {
        SysMenuMapper mapper = context.getBean(SysMenuMapper.class);
        QueryWrapper<SysMenuEnt> queryWrapper = new QueryWrapper<>();
        if (CharSequenceUtil.isNotBlank(input.getParentMenuId())) {
            queryWrapper.lambda().eq(SysMenuEnt::getParentMenuId, input.getParentMenuId());
        } else {
            queryWrapper.lambda().and(wrapper -> wrapper.isNull(SysMenuEnt::getParentMenuId).or().eq(SysMenuEnt::getParentMenuId, ""));
        }
        List<SysMenuEnt> viewList = mapper.selectList(queryWrapper);
        SystemDictionaryService dictService = context.getBean(SystemDictionaryService.class);
        List<QuerySysMenuOutputTree> list = XCopyUtils.copyNewList(viewList, QuerySysMenuOutputTree.class);
        //介质类型
        CreateSysDictionaryInput mediaTypesInput = new CreateSysDictionaryInput();
        mediaTypesInput.setAlias("MEDIUM_TYPE");
        XListResult<QuerySysDictionaryViewOutput> mediaTypes = dictService.querySysDictionarysByAlias(context, mediaTypesInput);
        for (QuerySysMenuOutputTree tree : list) {
            //介质类型
            if (CharSequenceUtil.isNotBlank(tree.getMediaType())) {
                if (CollUtil.isNotEmpty(mediaTypes.getResult())) {
                    StringBuilder buffer = new StringBuilder();
                    for (String mediaType : CharSequenceUtil.splitToArray(tree.getMediaType(), ',')) {
                        for (QuerySysDictionaryViewOutput item : mediaTypes.getResult()) {
                            if (CharSequenceUtil.equals(mediaType, item.getDicKey())) {
                                buffer.append(",").append(item.getDicName());
                            }
                        }
                    }
                    if (buffer.length() > 0) {
                        tree.setMediaTypeName(buffer.substring(1));
                    }
                }
                if (CharSequenceUtil.isBlank(tree.getMediaTypeName())) {
                    tree.setMediaTypeName(tree.getMediaType());
                }
            }
        }
        //判断是否有子集
        List<String> parentIds = list.stream().map(QuerySysMenuOutputTree::getId).collect(Collectors.toList());
        SysMenuViewMapper viewMapper = context.getBean(SysMenuViewMapper.class);
        List<SysMenuView> childList = viewMapper.selectCountByParentMenuIds(parentIds);
        Map<String, List<SysMenuView>> groupChildList = childList.stream().collect(Collectors.groupingBy(SysMenuView::getParentMenuId));
        list.forEach(item -> {
            if (!groupChildList.containsKey(item.getId())) {
                item.setChildren(null);
                item.setIsHasChildren(0);
            }
        });
        list.sort(Comparator.comparing(QuerySysMenuOutputTree::getSort));
        return XListResult.success(list);
    }

    public XListResult<QuerySysMenuOutputTree> getSysMenuNameTree(XContext context, GetSysMenuInput input) {
        SysMenuViewMapper mapper = context.getBean(SysMenuViewMapper.class);
        SystemDictionaryService dictService = context.getBean(SystemDictionaryService.class);
        SysMenuView record = new SysMenuView();
        XCopyUtils.copyObject(input, record);
        List<SysMenuView> viewList = mapper.selectSysMenuListByMenuName(record);
        List<QuerySysMenuOutputTree> list = XCopyUtils.copyNewList(viewList, QuerySysMenuOutputTree.class);
        //介质类型
        CreateSysDictionaryInput mediaTypesInput = new CreateSysDictionaryInput();
        mediaTypesInput.setAlias("MEDIUM_TYPE");
        XListResult<QuerySysDictionaryViewOutput> mediaTypes = dictService.querySysDictionarysByAlias(context, mediaTypesInput);
        for (QuerySysMenuOutputTree tree : list) {
            if (CharSequenceUtil.isNotBlank(tree.getMenuType())) {
                if (CharSequenceUtil.isBlank(tree.getBusinessTypeName())) {
                    tree.setBusinessTypeName(tree.getMenuType());
                }
                //介质类型
                if (CollUtil.isNotEmpty(mediaTypes.getResult())) {
                    StringBuilder buffer = new StringBuilder();
                    for (String mediaType : CharSequenceUtil.splitToArray(tree.getMediaType(), ',')) {
                        for (QuerySysDictionaryViewOutput item : mediaTypes.getResult()) {
                            if (mediaType.equals(item.getDicKey())) {
                                buffer.append(",").append(item.getDicName());
                            }
                        }
                    }
                    if (buffer.length() > 0) {
                        tree.setMediaTypeName(buffer.substring(1));
                    }
                }
                if (CharSequenceUtil.isBlank(tree.getMediaTypeName())) {
                    tree.setMediaTypeName(tree.getMediaType());
                }
            }
        }
        List<QuerySysMenuOutputTree> trees = new ArrayList<>(list.size());
        for (QuerySysMenuOutputTree tree : list) {
            if (CharSequenceUtil.isBlank(tree.getParentMenuId())) {
                trees.add(tree);
            }
            //子集
            for (QuerySysMenuOutputTree it : list) {
                if (CharSequenceUtil.equals(tree.getId(), it.getParentMenuId())) {
                    if (CollUtil.isEmpty(tree.getChildren())) {
                        tree.setChildren(new ArrayList<>());
                    }
                    tree.getChildren().add(it);
                }
            }
            if (CollUtil.isNotEmpty(tree.getChildren())) {
                List<QuerySysMenuOutputTree> children = tree.getChildren();
                children.sort(Comparator.comparing(QuerySysMenuOutputTree::getSort));
                tree.setChildren(children);
            }
        }
        trees.sort(Comparator.comparing(QuerySysMenuOutputTree::getSort));
        return XListResult.success(trees);
    }

    @XText("菜单排序")
    @XApiPost
    public XServiceResult updateSysMenuSort(XContext context, UpdateSysMenuInput input) throws XServiceException {
        return XTransactionHelper.begin(context, () -> {
            SysMenuMapper mapper = context.getBean(SysMenuMapper.class);
            SysMenuEnt entity = mapper.selectOne(new QueryWrapper<SysMenuEnt>()
                    .lambda()
                    .eq(SysMenuEnt::getId, input.getId()));
            if (ObjectUtil.isNull(entity)) {
                return XServiceResult.error(context, XError.NotFound);
            }
            Integer entitySort = entity.getSort();
            QueryWrapper<SysMenuEnt> neighborQueryWrapper = new QueryWrapper<>();
            neighborQueryWrapper.lambda().eq(SysMenuEnt::getParentMenuId, entity.getParentMenuId());
            //判断是上移还是下移
            if (CharSequenceUtil.equals(input.getSortFlag(), "up")) {
                //sort-=1;
                neighborQueryWrapper.lambda().le(SysMenuEnt::getSort, entitySort);
                neighborQueryWrapper.lambda().orderByDesc(SysMenuEnt::getSort);
            } else {
                //sort+=1;
                neighborQueryWrapper.lambda().ge(SysMenuEnt::getSort, entitySort);
                neighborQueryWrapper.lambda().orderByAsc(SysMenuEnt::getSort);
            }
            List<SysMenuEnt> sysMenuEnts = mapper.selectList(neighborQueryWrapper);
            PpsUserSession session = context.getSession(PpsUserSession.class);
            if (CollUtil.isNotEmpty(sysMenuEnts)) {
                SysMenuEnt neighborEnt = sysMenuEnts.get(1);
                Integer neighborEntSort = neighborEnt.getSort();
                if (CharSequenceUtil.equals(input.getSortFlag(), "up")) {
                    if (ObjectUtil.isNotNull(neighborEntSort) && neighborEntSort.equals(entitySort)) {
                        neighborEntSort -= 1;
                    }
                } else {
                    if (ObjectUtil.isNotNull(neighborEntSort) && neighborEntSort.equals(entitySort)) {
                        neighborEntSort += 1;
                    }
                }
                //上方或下方有数据进行交换序号
                neighborEnt.setSort(entitySort);
                neighborEnt.setModifyTime(new Date());
                neighborEnt.setModifyById(session.getId());
                neighborEnt.setModifyByName(session.getUserName());
                mapper.updateById(neighborEnt);
                entity.setSort(neighborEntSort);
            }
            entity.setModifyTime(new Date());
            entity.setModifyById(session.getId());
            entity.setModifyByName(session.getUserName());
            mapper.updateById(entity);
            SysMenuViewMapper viewMapper = context.getBean(SysMenuViewMapper.class);
            SysMenuView menuView = new SysMenuView();
            menuView.setShowChildren(false);
            viewMapper.updateShowChildren(menuView);
            menuView.setId(input.getId());
            List<SysMenuView> menuViews = viewMapper.selectParentMenus(menuView);
            menuViews.forEach(sysMenuView -> sysMenuView.setShowChildren(true));
            List<SysMenuEnt> menuEnts = XCopyUtils.copyNewList(menuViews, SysMenuEnt.class);
            menuEnts.forEach(mapper::updateById);
            //返回执行成功的结果，事务提交
            return XServiceResult.OK;
        });
    }

    @XText("新增菜单")
    @XApiPost
    public XServiceResult createSysMenu(XContext context, CreateSysMenuInput input) throws XServiceException {
        return XTransactionHelper.begin(context, () -> {
            SysMenuMapper mapper = context.getBean(SysMenuMapper.class);
            SysMenuViewMapper viewMapper = context.getBean(SysMenuViewMapper.class);
            SysMenuEnt entity = new SysMenuEnt();
            XCopyUtils.copyObject(input, entity);
            if (ObjectUtil.isNull(entity.getSort())) {
                SysMenuView relView = viewMapper.selectMaxSort(entity.getParentMenuId());
                if (ObjectUtil.isNotNull(relView)) {
                    entity.setSort(relView.getSort() + 1);
                } else {
                    entity.setSort(1);
                }
            } else {
                List<SysMenuEnt> sysMenuEnts = mapper.selectList(new QueryWrapper<SysMenuEnt>()
                        .lambda()
                        .eq(SysMenuEnt::getParentMenuId, entity.getParentMenuId())
                        .ge(SysMenuEnt::getSort, entity.getSort()));
                if (CollUtil.isNotEmpty(sysMenuEnts)) {
                    sysMenuEnts.forEach(sysMenuEnt -> {
                        sysMenuEnt.setSort(sysMenuEnt.getSort() + 1);
                        mapper.updateById(sysMenuEnt);
                    });
                }
            }
            if (CharSequenceUtil.isBlank(entity.getParentMenuId())) {
                entity.setLevel("1");
            }
            entity.setCreateTime(new Date());
            PpsUserSession session = context.getSession(PpsUserSession.class);
            if (ObjectUtil.isNotNull(session)) {
                entity.setCreateById(session.getId());
                entity.setCreateByName(session.getUserName());
            }
            entity.setMediaType("ALL");
            entity.setShowChildren(false);
            mapper.insert(entity);
            //返回执行成功的结果，事务提交
            return XServiceResult.OK;
        });
    }

    @XText("获取当前父级下最大的序号")
    @XApiPost
    public XSingleResult<QuerySysMenuInput> selectMaxSort(XContext context, GetSysMenuInput input) {
        SysMenuViewMapper viewMapper = context.getBean(SysMenuViewMapper.class);
        SysMenuView relView = viewMapper.selectMaxSort(input.getParentMenuId());
        QuerySysMenuInput output = new QuerySysMenuInput();
        if (ObjectUtil.isNotNull(relView)) {
            output.setSort(relView.getSort() + 1);
        } else {
            output.setSort(1);
        }
        return XSingleResult.success(output);
    }

    @XText("修改菜单信息")
    @XApiPost
    public XServiceResult updateSysMenu(XContext context, UpdateSysMenuInput input) throws XServiceException {
        return XTransactionHelper.begin(context, () -> {
            SysMenuMapper mapper = context.getBean(SysMenuMapper.class);
            SysMenuViewMapper viewMapper = context.getBean(SysMenuViewMapper.class);
            SysMenuEnt entity = mapper.selectOne(new QueryWrapper<SysMenuEnt>()
                    .lambda()
                    .eq(SysMenuEnt::getId, input.getId()));
            if (ObjectUtil.isNull(entity)) {
                return XServiceResult.error(context, XError.NotFound);
            }
            XCopyUtils.copyObject(input, entity);
            entity.setModifyTime(new Date());
            PpsUserSession session = context.getSession(PpsUserSession.class);
            if (ObjectUtil.isNotNull(session)) {
                entity.setModifyById(session.getId());
                entity.setModifyByName(session.getUserName());
            }
            mapper.updateById(entity);
            SysMenuView menuView = new SysMenuView();
            menuView.setShowChildren(false);
            viewMapper.updateShowChildren(menuView);
            menuView.setId(input.getId());
            List<SysMenuView> menuViews = viewMapper.selectParentMenus(menuView);
            menuViews.forEach(sysMenuView -> sysMenuView.setShowChildren(true));
            List<SysMenuEnt> sysMenuEnts = XCopyUtils.copyNewList(menuViews, SysMenuEnt.class);
            sysMenuEnts.forEach(mapper::updateById);
            //返回执行成功的结果，事务提交
            return XServiceResult.OK;
        });
    }

    @XText("删除菜单")
    @XApiPost
    public XServiceResult deleteSysMenu(XContext context, DeleteSysMenuInput input) throws XServiceException {
        return XTransactionHelper.begin(context, () -> {
            SysMenuMapper mapper = context.getBean(SysMenuMapper.class);
            SysMenuEnt entity = mapper.selectOne(new QueryWrapper<SysMenuEnt>()
                    .lambda()
                    .eq(SysMenuEnt::getId, input.getId()));
            if (ObjectUtil.isNull(entity)) {
                return XServiceResult.error(context, XError.NotFound);
            }
            List<SysMenuEnt> sysMenuEnts = mapper.selectList(new QueryWrapper<SysMenuEnt>()
                    .lambda()
                    .eq(SysMenuEnt::getParentMenuId, entity.getId()));
            if (CollUtil.isEmpty(sysMenuEnts)) {
                mapper.deleteById(entity);
            } else {
                return XServiceResult.error(context, MenuError.NotDelete);
            }
            //返回执行成功的结果，事务提交
            return XServiceResult.OK;
        });
    }

    @XApiGet
    public XSingleResult<GetSysMenuOutput> getSysMenu(XContext context, GetSysMenuInput input) {
        SysMenuMapper mapper = context.getBean(SysMenuMapper.class);
        SysMenuEnt entity = mapper.selectOne(new QueryWrapper<SysMenuEnt>()
                .lambda()
                .eq(SysMenuEnt::getId, input.getId()));
        if (ObjectUtil.isNull(entity)) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetSysMenuOutput output = new GetSysMenuOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XApiGet
    public XPageResult<QuerySysMenuOutput> querySysMenu(XContext context, QuerySysMenuInput input) {
        SysMenuMapper mapper = context.getBean(SysMenuMapper.class);
        QueryWrapper<SysMenuEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().like(SysMenuEnt::getId, input.getId())
                .like(SysMenuEnt::getMenuName, input.getMenuName())
                .like(SysMenuEnt::getParentMenuId, input.getParentMenuId())
                .eq(SysMenuEnt::getIsVisiable, input.getIsVisiable())
                .eq(SysMenuEnt::getIsHasScope, input.getIsHasScope())
                .eq(SysMenuEnt::getSort, input.getSort())
                .eq(SysMenuEnt::getIsHasChildren, input.getIsHasChildren())
                .like(SysMenuEnt::getUrl, input.getUrl())
                .eq(SysMenuEnt::getIsExternal, input.getIsExternal())
                .like(SysMenuEnt::getCreateById, input.getCreateById())
                .like(SysMenuEnt::getCreateByName, input.getCreateByName())
                .eq(SysMenuEnt::getCreateTime, input.getCreateTime())
                .like(SysMenuEnt::getModifyById, input.getModifyById())
                .like(SysMenuEnt::getModifyByName, input.getModifyByName())
                .eq(SysMenuEnt::getModifyTime, input.getModifyTime());
        return XMapperHelper.query(mapper, input, queryWrapper, QuerySysMenuOutput.class);
    }

    @XApiGet
    public XPageResult<QuerySysMenuOutput> dynamicQuerySysMenu(XContext context, CustomQueryInput input) {
        SysMenuMapper mapper = context.getBean(SysMenuMapper.class);
        return XMapperHelper.query(mapper, input, SysMenuEnt.class, QuerySysMenuOutput.class);
    }

    @XApiGet
    public XSingleResult<GetSysMenuViewOutput> getSysMenuView(XContext context, GetSysMenuViewInput input) {
        SysMenuViewMapper mapper = context.getBean(SysMenuViewMapper.class);
        SysMenuView record = new SysMenuView();
        XCopyUtils.copyObject(input, record);
        SysMenuView view = mapper.selectOne(record);
        if (ObjectUtil.isNull(view)) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetSysMenuViewOutput output = new GetSysMenuViewOutput();
        XCopyUtils.copyObject(view, output);
        return XSingleResult.success(output);
    }

    @XApiGet
    public XPageResult<QuerySysMenuViewOutput> querySysMenuView(XContext context, QuerySysMenuViewInput input) {
        SysMenuViewMapper mapper = context.getBean(SysMenuViewMapper.class);
        SysMenuView record = new SysMenuView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<SysMenuView> list = mapper.selectList(record);
        PageInfo<SysMenuView> pageInfo = new PageInfo<>(list);
        List<QuerySysMenuViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySysMenuViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    @XText("获取用户菜单")
    @XApiGet
    public XListResult<QuerySysUserMenuViewOutput> querySysUserMenuView(XContext context) {
        PpsUserSession session = context.getSession(PpsUserSession.class);
        SysMenuViewMapper mapper = context.getBean(SysMenuViewMapper.class);
        SysMenuView menuView = new SysMenuView();
        menuView.setUserId(session.getId());
        List<SysMenuView> list = mapper.selectUserMenuList(menuView);
        List<QuerySysUserMenuViewOutput> outputs = XCopyUtils.copyNewList(list, QuerySysUserMenuViewOutput.class);
        return XListResult.success(TreeUtils.build(outputs));
    }

    @XText("获取三级菜单")
    @XApiGet
    public XListResult<QuerySysUserMenuViewOutput> queryMenuForThreeLevelView(XContext context) {
        SysMenuViewMapper mapper = context.getBean(SysMenuViewMapper.class);
        List<SysMenuView> list = mapper.selectMenuForThreeLevelList(new SysMenuView());
        List<QuerySysUserMenuViewOutput> outputs = XCopyUtils.copyNewList(list.stream().filter(item -> !item.getUrl().contains("#")).collect(Collectors.toList()), QuerySysUserMenuViewOutput.class);
        return XListResult.success(outputs);
    }
}