package pps.core.system.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.text.CharSequenceUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.session.PpsUserSession;
import pps.core.system.entity.SysOrganizationView;
import pps.core.system.entity.SysUserRoleRelEnt;
import pps.core.system.entity.SysUserRoleRelView;
import pps.core.system.mapper.SysOrganizationViewMapper;
import pps.core.system.mapper.SysUserRoleRelMapper;
import pps.core.system.mapper.SysUserRoleRelViewMapper;
import pps.core.system.service.data.sys_user_role_rel.*;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.exception.XServiceException;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.helper.XTransactionHelper;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

@XText("用户角色管理")
@XService
public class SysUserRoleRelService {

    @XText("用户绑定多角色关系")
    @XApiPost
    public XServiceResult createSysUserRoleRel(XContext context, CreateSysUserRoleRelInput input) throws XServiceException {
        return XTransactionHelper.begin(context, () -> {
            //删除原关系
            SysUserRoleRelMapper mapper = context.getBean(SysUserRoleRelMapper.class);
            List<SysUserRoleRelEnt> sysUserRoleRelEnts = mapper.selectList(new QueryWrapper<SysUserRoleRelEnt>()
                    .lambda()
                    .eq(SysUserRoleRelEnt::getUserId, input.getUserId()));
            if (CollUtil.isNotEmpty(sysUserRoleRelEnts)) {
                mapper.deleteBatchIds(sysUserRoleRelEnts.stream().map(SysUserRoleRelEnt::getId).collect(Collectors.toList()));
            }
            //创建新关系
            SysUserRoleRelViewMapper viewMapper = context.getBean(SysUserRoleRelViewMapper.class);
            List<String> roleIdList = input.getRoleIdList();
            Date now = new Date();
            List<SysUserRoleRelView> viewList = new ArrayList<>();
            PpsUserSession session = context.getSession(PpsUserSession.class);
            for (String roleId : roleIdList) {
                SysUserRoleRelView entity = new SysUserRoleRelView();
                entity.setId(UUID.randomUUID().toString());
                entity.setUserId(input.getUserId());
                entity.setRoleId(roleId);
                entity.setCreateById(session.getId());
                entity.setCreateByName(session.getUserName());
                entity.setCreateTime(now);
                viewList.add(entity);
            }
            if (CollUtil.isNotEmpty(viewList)) {
                viewMapper.batchInsert(viewList);
            }
            //返回执行成功的结果，事务提交
            return XServiceResult.OK;

        });
    }

    @XText("角色绑定多用户")
    @XApiPost
    public XServiceResult createSysUsersRoleRel(XContext context, CreateSysUserRoleRelInput input) throws XServiceException {
        return XTransactionHelper.begin(context, () -> {
            //查询原关系
            SysUserRoleRelMapper mapper = context.getBean(SysUserRoleRelMapper.class);
            List<SysUserRoleRelEnt> sysUserRoleRelEnts = mapper.selectList(new QueryWrapper<SysUserRoleRelEnt>()
                    .lambda()
                    .eq(SysUserRoleRelEnt::getRoleId, input.getRoleId())
                    .in(SysUserRoleRelEnt::getUserId, input.getUserIdList().stream()
                            .map(SysUserRoleRelEnt::getId).collect(Collectors.toList())));
            //创建新关系
            SysUserRoleRelViewMapper viewMapper = context.getBean(SysUserRoleRelViewMapper.class);
            List<SysUserRoleRelEnt> userIdList = input.getUserIdList();
            Date now = new Date();
            List<SysUserRoleRelView> viewList = new ArrayList<>();
            List<SysUserRoleRelEnt> entList = new ArrayList<>();
            PpsUserSession session = context.getSession(PpsUserSession.class);
            for (SysUserRoleRelEnt user : userIdList) {
                SysUserRoleRelView entity = new SysUserRoleRelView();
                entity.setId(UUID.randomUUID().toString());
                entity.setUserId(user.getId());
                entity.setRoleId(input.getRoleId());
                entity.setCreateById(session.getId());
                entity.setCreateByName(session.getUserName());
                entity.setCreateTime(now);
                viewList.add(entity);
                //同一个用户添加多次删除调旧的
                for (SysUserRoleRelEnt roleRelEnt : sysUserRoleRelEnts) {
                    if (CharSequenceUtil.equals(roleRelEnt.getUserId(), user.getId())) {
                        entList.add(roleRelEnt);
                        break;
                    }
                }
            }
            if (CollUtil.isNotEmpty(viewList)) {
                viewMapper.batchInsert(viewList);
            }
            if (CollUtil.isNotEmpty(entList)) {
                mapper.deleteBatchIds(entList.stream().map(SysUserRoleRelEnt::getId).collect(Collectors.toList()));
            }
            //返回执行成功的结果，事务提交
            return XServiceResult.OK;
        });
    }

    @XText("查询用户角色")
    @XApiGet
    public XListResult<QuerySysUserRoleRelViewOutput> querySysUserRoleRelView(XContext context, QuerySysUserRoleRelViewInput input) {
        SysUserRoleRelViewMapper mapper = context.getBean(SysUserRoleRelViewMapper.class);
        SysUserRoleRelView record = new SysUserRoleRelView();
        XCopyUtils.copyObject(input, record);
        List<SysUserRoleRelView> list = mapper.selectList(record);
        List<QuerySysUserRoleRelViewOutput> outputs = XCopyUtils.copyNewList(list, QuerySysUserRoleRelViewOutput.class);
        return XListResult.success(outputs);
    }

    @XText("根据角色查人员列表")
    @XApiPost
    public XPageResult<QuerySysUserRoleRelViewOutput> querySysUserRoleByRoleIdView(XContext context, QuerySysUserRoleRelOutput input) {
        SysUserRoleRelViewMapper mapper = context.getBean(SysUserRoleRelViewMapper.class);
        SysUserRoleRelView record = new SysUserRoleRelView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<SysUserRoleRelView> list;
        record.setEndTime(new Date());
        if (CharSequenceUtil.isNotBlank(input.getOuId())) {
            List<String> ouIds = new ArrayList<>();
            ouIds.add(input.getOuId());
            record.setOuIds(ouIds);
            list = mapper.selectListByRoleIdAndOuIds(record);
            PageInfo<SysUserRoleRelView> pageInfo = new PageInfo<>(list);
            List<QuerySysUserRoleRelViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySysUserRoleRelViewOutput.class);
            return XPageResult.success(outputs, input, pageInfo.getTotal());
        }
        PpsUserSession session = context.getSession(PpsUserSession.class);
        Integer isSuperAdmin = session.getIsSuperAdmin();
        if (BusinessConstant.ZERO.equals(isSuperAdmin)) {
            //查询当前用户所属组织机构的子级
            SysOrganizationViewMapper sysOrganizationViewMapper = context.getBean(SysOrganizationViewMapper.class);
            SysOrganizationView sysOrganizationView = new SysOrganizationView();
            sysOrganizationView.setEndTime(new Date());
            sysOrganizationView.setIsDeleted(BusinessConstant.ONE);
            sysOrganizationView.setParentOuId(session.getOuId());
            List<SysOrganizationView> viewList = sysOrganizationViewMapper.selectOuSonByParentOuId(sysOrganizationView);
            List<String> ouIds = new ArrayList<>();
            if (CollUtil.isNotEmpty(viewList)) {
                for (SysOrganizationView organizationView : viewList) {
                    ouIds.add(organizationView.getId());
                }
            }
            ouIds.add(session.getOuId());
            record.setOuIds(ouIds);
            list = mapper.selectListByRoleIdAndOuIds(record);
        } else {
            list = mapper.selectListByRoleId(record);
        }
        PageInfo<SysUserRoleRelView> pageInfo = new PageInfo<>(list);
        List<QuerySysUserRoleRelViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySysUserRoleRelViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    @XText("删除角色中的用户")
    @XApiPost
    public XServiceResult deleteSysUserRoleRel(XContext context, DeleteSysUserRoleRelOutput input) throws XServiceException {
        return XTransactionHelper.begin(context, () -> {
            SysUserRoleRelMapper mapper = context.getBean(SysUserRoleRelMapper.class);
            List<SysUserRoleRelEnt> relEnts = mapper.selectList(new QueryWrapper<SysUserRoleRelEnt>()
                    .lambda()
                    .in(SysUserRoleRelEnt::getId, input.getIds()));
            if (CollUtil.isNotEmpty(relEnts)) {
                mapper.deleteBatchIds(relEnts.stream().map(SysUserRoleRelEnt::getId).collect(Collectors.toList()));
            }
            //返回执行成功的结果，事务提交
            return XServiceResult.OK;
        });
    }

    @XText("根据人员查角色列表")
    @XApiPost
    public XListResult<QuerySysUserRoleRelViewOutput> querySysUserRoleByUserIdView(XContext context, QuerySysUserRoleRelViewInput input) {
        SysUserRoleRelViewMapper mapper = context.getBean(SysUserRoleRelViewMapper.class);
        SysUserRoleRelView record = new SysUserRoleRelView();
        XCopyUtils.copyObject(input, record);
        List<SysUserRoleRelView> list = mapper.selectListByUserId(record);
        List<QuerySysUserRoleRelViewOutput> outputs = XCopyUtils.copyNewList(list, QuerySysUserRoleRelViewOutput.class);
        return XListResult.success(outputs);
    }
}