package pps.cloud.base.service;


import pps.cloud.base.service.data.base_power_line.DynamicQueryBasePowerLineInput;
import pps.cloud.base.service.data.base_power_line.DynamicQueryBasePowerLineOutput;
import pps.cloud.base.service.data.base_power_line.GetBasePowerLineInput;
import pps.cloud.base.service.data.base_power_line.GetBasePowerLineViewOutput;
import pps.cloud.base.service.data.base_power_line_plant.DynamicQueryBasePowerLinePlantInput;
import pps.cloud.base.service.data.base_power_line_plant.DynamicQueryBasePowerLinePlantOutput;
import pps.cloud.base.service.data.base_power_line_plant.DynamicQueryBasePowerLinePlantViewOutput;
import pps.cloud.base.service.data.base_power_line_storage.DynamicQueryBasePowerLineStorageInput;
import pps.cloud.base.service.data.base_power_line_storage.DynamicQueryBasePowerLineStorageOutput;
import pps.cloud.base.service.data.base_power_line_storage.DynamicQueryBasePowerLineStorageViewOutput;
import pps.cloud.base.service.data.base_power_line_wellhead.DynamicQueryBasePowerLineWellheadInput;
import pps.cloud.base.service.data.base_power_line_wellhead.DynamicQueryBasePowerLineWellheadViewOutput;
import pps.cloud.base.service.data.base_power_line_wellhead.GetBasePowerLineWellheadViewOutput;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.data.XListResult;
import xstartup.data.XSingleResult;

/**
 * 输电线路配置Cloud模块
 *
 * @author ZWT
 * @date 2023/09/05 15:34
 */
@XService
@XText("输电线路配置Cloud模块")
public interface IBasePowerLineCloudService {

    /**
     * 输电线路配置Cloud模块--通过ID查询线路基本信息
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetBasePowerLineViewOutput}>
     */
    @XText("输电线路配置Cloud模块--通过ID查询线路基本信息")
    XSingleResult<GetBasePowerLineViewOutput> getBasePowerLineById(XContext context, GetBasePowerLineInput input);

    /**
     * 输电线路配置Cloud模块--条件查询线路基本信息列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryBasePowerLineOutput}>
     */
    @XText("输电线路配置Cloud模块--条件查询线路基本信息列表")
    XListResult<DynamicQueryBasePowerLineOutput> getBasePowerLineList(XContext context, DynamicQueryBasePowerLineInput input);

    /**
     * 输电线路配置Cloud模块--查询组织列表下所有线路
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryBasePowerLineOutput}>
     */
    @XText("输电线路配置Cloud模块--查询组织列表下所有线路")
    XListResult<DynamicQueryBasePowerLineOutput> getBasePowerLineListByOuIdList(XContext context, DynamicQueryBasePowerLineInput input);

    /**
     * 输电线路配置Cloud模块--条件查询线路关联井口信息
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryBasePowerLineWellheadViewOutput}>
     */
    @XText("输电线路配置Cloud模块--条件查询线路关联井口信息")
    XListResult<DynamicQueryBasePowerLineWellheadViewOutput> getPowerLineWellheadList(XContext context, DynamicQueryBasePowerLineWellheadInput input);

    /**
     * 输电线路配置Cloud模块--条件查询线路关联电站视图信息
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryBasePowerLinePlantViewOutput}>
     */
    @XText("输电线路配置Cloud模块--条件查询线路关联电站视图信息")
    XListResult<DynamicQueryBasePowerLinePlantViewOutput> getPowerLinePlantViewList(XContext context, DynamicQueryBasePowerLinePlantInput input);

    /**
     * 输电线路配置Cloud模块--条件查询线路关联电站信息
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryBasePowerLinePlantOutput}>
     */
    @XText("输电线路配置Cloud模块--条件查询线路关联电站信息")
    XListResult<DynamicQueryBasePowerLinePlantOutput> queryPowerLinePlantListByParam(XContext context, DynamicQueryBasePowerLinePlantInput input);

    /**
     * 输电线路配置Cloud模块--条件查询线路关联储能设备参数平均值
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryBasePowerLineStorageViewOutput}>
     */
    @XText("输电线路配置Cloud模块--条件查询线路关联储能设备参数平均值")
    XListResult<DynamicQueryBasePowerLineStorageViewOutput> queryPowerLineStorageAvgByLineIds(XContext context, DynamicQueryBasePowerLineStorageInput input);

    /**
     * 输电线路配置Cloud模块--条件查询线路关联储能设备信息
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryBasePowerLineStorageOutput}>
     */
    @XText("输电线路配置Cloud模块--条件查询线路关联储能设备信息")
    XListResult<DynamicQueryBasePowerLineStorageOutput> queryPowerLineStorageListByLineIds(XContext context, DynamicQueryBasePowerLineStorageInput input);

    /**
     * 输电线路配置Cloud模块-查询线路井口运行总功率
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetBasePowerLineWellheadViewOutput}>
     */
    @XText("输电线路配置Cloud模块--查询线路井口运行总功率")
    XSingleResult<GetBasePowerLineWellheadViewOutput> queryLineSumServiceRatingByLineId(XContext context, DynamicQueryBasePowerLineWellheadInput input);

    /**
     * 输电线路配置Cloud模块--通过电站ID查井口
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryBasePowerLineWellheadViewOutput}>
     */
    @XText("输电线路配置Cloud模块--条件查询线路关联井口信息")
    XListResult<DynamicQueryBasePowerLineWellheadViewOutput> getPowerLineWellheadListByPlant(XContext context, DynamicQueryBasePowerLineWellheadInput input);
}