package pps.core.base.mapper;

import cn.hutool.core.date.DateTime;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;
import pps.core.base.entity.ThirdWindPowerGenerationUpdateView;

import java.util.List;

/**
 * 第三方风力发电数据(每日最新数据)
 *
 * @author ZWT
 * @date 2024/09/04
 */
@Repository(value = "pps.core.base.mapper.ThirdWindPowerGenerationUpdateViewMapper")
public interface ThirdWindPowerGenerationUpdateViewMapper {

    ThirdWindPowerGenerationUpdateView selectOne(ThirdWindPowerGenerationUpdateView record);

    List<ThirdWindPowerGenerationUpdateView> selectList(ThirdWindPowerGenerationUpdateView record);

    /**
     * 电站月平均发电量
     *
     * @param list        列表
     * @param beginOfYear 年初
     * @param endOfYear   年终
     * @return {@link List }<{@link ThirdWindPowerGenerationUpdateView }>
     */
    List<ThirdWindPowerGenerationUpdateView> stationMonthAvgPowerGeneration(@Param(value = "list") List<String> list,
                                                                            @Param(value = "beginOfYear") DateTime beginOfYear,
                                                                            @Param(value = "endOfYear") DateTime endOfYear);

    /**
     * 批量插入
     *
     * @param list 列表
     * @return int
     */
    int batchInsert(@Param(value = "list") List<ThirdWindPowerGenerationUpdateView> list);

    /**
     * 批量删除
     *
     * @param deleteList 删除列表
     * @param create     创造
     * @param end        结束
     * @return int
     */
    int deleteBatch(@Param(value = "list") List<String> deleteList, @Param(value = "create") DateTime create, @Param(value = "end") DateTime end);

    /**
     * 电站发电
     *
     * @param collect  收集
     * @param dateList 日期列表
     * @return {@link ThirdWindPowerGenerationUpdateView }
     */
    ThirdWindPowerGenerationUpdateView stationPowerGeneration(@Param(value = "list") List<String> collect, @Param(value = "dateList") List<DateTime> dateList);
}
