package pps.core.base.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.text.CharSequenceUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import pps.cloud.base.service.IBaseEnergyStorageCloudService;
import pps.cloud.base.service.data.base_energy_storage_device.DynamicQueryBaseEnergyStorageDeviceInput;
import pps.cloud.base.service.data.base_energy_storage_device.DynamicQueryBaseEnergyStorageDeviceOutput;
import pps.core.base.entity.BaseEnergyStorageDeviceEnt;
import pps.core.base.mapper.BaseEnergyStorageDeviceMapper;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;

import java.util.List;

/**
 * 储能电站Cloud模块
 *
 * @author ZWT
 * @date 2024/03/25 20:54
 */
@XService
public class BaseEnergyStorageCloudServiceImpl implements IBaseEnergyStorageCloudService {

    /**
     * 条件查询储能电站列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryBaseEnergyStorageDeviceOutput}>
     */
    @Override
    public XListResult<DynamicQueryBaseEnergyStorageDeviceOutput> queryBaseStorageListByParam(XContext context, DynamicQueryBaseEnergyStorageDeviceInput input) {
        String ouId = input.getOuId();
        List<String> ouIds = input.getOuIds();
        BaseEnergyStorageDeviceMapper mapper = context.getBean(BaseEnergyStorageDeviceMapper.class);
        List<BaseEnergyStorageDeviceEnt> list = mapper.selectList(new LambdaQueryWrapper<BaseEnergyStorageDeviceEnt>()
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                .eq(CharSequenceUtil.isNotBlank(ouId), BaseEnergyStorageDeviceEnt::getOuId, ouId)
                .in(CollUtil.isNotEmpty(ouIds), BaseEnergyStorageDeviceEnt::getOuId, ouId)
        );
        return XListResult.success(XCopyUtils.copyNewList(list, DynamicQueryBaseEnergyStorageDeviceOutput.class));
    }
}
