package pps.core.base.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import pps.cloud.prediction.service.PlantTrainPowerTaskCloudService;
import pps.cloud.prediction.service.data.plant_train_power_task.GetPlantTrainPowerTaskCloudInput;
import pps.cloud.prediction.service.data.plant_train_power_task.GetPlantTrainPowerTaskCloudOutput;
import pps.cloud.system.service.SystemAreaService;
import pps.cloud.system.service.SystemDictionaryService;
import pps.cloud.system.service.SystemUserRoleRelService;
import pps.cloud.system.service.data.sys_area.GetSysAreaInput;
import pps.cloud.system.service.data.sys_area.GetSysAreaOutput;
import pps.cloud.system.service.data.sys_dictionary.CreateSysDictionaryInput;
import pps.cloud.system.service.data.sys_dictionary.QuerySysDictionaryViewOutput;
import pps.cloud.system.service.data.user_role.GetSysUserRoleRelInput;
import pps.cloud.system.service.data.user_role.GetSysUserRoleRelOutput;
import pps.core.base.constant.MapeConstant;
import pps.core.base.entity.BasePhotovoltaicPlantEnt;
import pps.core.base.entity.BasePhotovoltaicPlantMapeAdjView;
import pps.core.base.entity.BasePowerLineEnt;
import pps.core.base.entity.BasePowerLinePlantEnt;
import pps.core.base.enums.BusinessError;
import pps.core.base.mapper.BasePhotovoltaicPlantMapeAdjViewMapper;
import pps.core.base.mapper.BasePhotovoltaicPlantMapper;
import pps.core.base.mapper.BasePowerLinePlantMapper;
import pps.core.base.service.data.base_photovoltaic_plant.*;
import pps.core.base.utils.ServiceUtil;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.BaseUtils;
import pps.core.system.entity.SysOrganizationView;
import pps.core.system.mapper.SysOrganizationViewMapper;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XJsonUtils;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;
import xstartup.helper.XTransactionHelper;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;


/**
 * 光伏电站模块
 *
 * @author ZWT
 * @date 2023/09/08
 */
@XService
public class BasePhotovoltaicPlantService {

    /**
     * 光伏电站模块--新增
     * POST /base/base-photovoltaic-plant/create-base-photovoltaic-plant
     * 接口ID：105466278
     * 接口地址：https://app.apifox.com/project/3196988/apis/api-105466278
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XText("光伏电站模块--新增")
    @XApiPost
    public XServiceResult createBasePhotovoltaicPlant(XContext context, CreateBasePhotovoltaicPlantInput input) {
        BasePhotovoltaicPlantMapper mapper = context.getBean(BasePhotovoltaicPlantMapper.class);
        if (CharSequenceUtil.isNotEmpty(input.getMakerNumber()) && this.checkMakerNumberExists(mapper, input.getMakerNumber(), null)) {
            return XServiceResult.error(context, BusinessError.MakerNumberExists);
        }
        BasePhotovoltaicPlantEnt entity = XCopyUtils.copyNewObject(input, BasePhotovoltaicPlantEnt.class);
        BaseUtils.setBaseModelDefault(entity, context.getSession(PpsUserSession.class));
        return XTransactionHelper.begin(context, () -> {
            mapper.insert(entity);
            return XServiceResult.OK;
        });
    }

    /**
     * 光伏电站模块--修改
     * POST /base/base-photovoltaic-plant/update-base-photovoltaic-plant
     * 接口ID：105466279
     * 接口地址：https://app.apifox.com/project/3196988/apis/api-105466279
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XText("光伏电站模块--修改")
    @XApiPost
    public XServiceResult updateBasePhotovoltaicPlant(XContext context, UpdateBasePhotovoltaicPlantInput input) {
        BasePhotovoltaicPlantMapper mapper = context.getBean(BasePhotovoltaicPlantMapper.class);
        if (CharSequenceUtil.isNotEmpty(input.getMakerNumber()) && this.checkMakerNumberExists(mapper, input.getMakerNumber(), input.getId())) {
            return XServiceResult.error(context, BusinessError.MakerNumberExists);
        }
        BasePhotovoltaicPlantEnt entity = this.selectOneByPlantId(input.getId(), mapper);
        if (ObjectUtil.isNull(entity)) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        BaseUtils.setBaseModelDefault(entity, context.getSession(PpsUserSession.class));
        return XTransactionHelper.begin(context, () -> {
            mapper.updateById(entity);
            return XServiceResult.OK;
        });
    }

    /**
     * 光伏电站模块--删除
     * POST /base/base-photovoltaic-plant/delete-base-photovoltaic-plant
     * 接口ID：105466280
     * 接口地址：https://app.apifox.com/project/3196988/apis/api-105466280
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XText("光伏电站模块--删除")
    @XApiPost
    public XServiceResult deleteBasePhotovoltaicPlant(XContext context, DeleteBasePhotovoltaicPlantInput input) {
        String plantId = input.getId();
        if (this.checkPhotovoltaicIsReference(context, plantId)) {
            return XServiceResult.error(context, BusinessError.PhotovoltaicReference);
        }
        BasePhotovoltaicPlantMapper mapper = context.getBean(BasePhotovoltaicPlantMapper.class);
        if (ObjectUtil.isNull(this.selectOneByPlantId(plantId, mapper))) {
            return XServiceResult.error(context, XError.NotFound);
        }
        BasePhotovoltaicPlantEnt entity = new BasePhotovoltaicPlantEnt();
        entity.setId(plantId);
        entity.setIsDeleted(BusinessConstant.ZERO);
        BaseUtils.setBaseModelDefault(entity, context.getSession(PpsUserSession.class));
        return XTransactionHelper.begin(context, () -> {
            mapper.updateById(entity);
            return XServiceResult.OK;
        });
    }

    /**
     * 光伏电站模块--详情
     * GET /base/base-photovoltaic-plant/get-base-photovoltaic-plant
     * 接口ID：105466276
     * 接口地址：https://app.apifox.com/project/3196988/apis/api-105466276
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetBasePhotovoltaicPlantOutput}>
     */
    @XText("光伏电站模块--详情")
    @XApiGet
    public XSingleResult<GetBasePhotovoltaicPlantOutput> getBasePhotovoltaicPlant(XContext context, GetBasePhotovoltaicPlantInput input) {
        BasePhotovoltaicPlantMapper mapper = context.getBean(BasePhotovoltaicPlantMapper.class);
        BasePhotovoltaicPlantEnt entity = this.selectOneByPlantId(input.getId(), mapper);
        if (ObjectUtil.isNull(entity)) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBasePhotovoltaicPlantOutput output = XCopyUtils.copyNewObject(entity, GetBasePhotovoltaicPlantOutput.class);
        //根据组织机构的路径获取
        SysOrganizationViewMapper organizationViewMapper = context.getBean(SysOrganizationViewMapper.class);
        SysOrganizationView organizationView = new SysOrganizationView();
        organizationView.setId(entity.getOuId());
        organizationView = organizationViewMapper.selectOrgProvince(organizationView);
        if (ObjectUtil.isNotNull(organizationView)) {
            output.setProvince(organizationView.getProvinceName());
        }
        output.setAreaName(Optional.ofNullable(this.getSysAreaInfo(context, GetSysAreaInput.builder()
                        .id(entity.getAreaCode())
                        .build()))
                .map(GetSysAreaOutput::getName)
                .orElse(null));
        //20240321 从电站辐照度模拟表中获取精准率
        if (CharSequenceUtil.isNotEmpty(input.getType())) {
            PlantTrainPowerTaskCloudService cloudService = context.getBean(PlantTrainPowerTaskCloudService.class);
            GetPlantTrainPowerTaskCloudInput cloudInput = new GetPlantTrainPowerTaskCloudInput();
            cloudInput.setPlantId(input.getId());
            switch (input.getType()) {
                case "1":
                    cloudInput.setModelCycle(240);
                    break;
                case "2":
                    cloudInput.setModelCycle(72);
                    break;
                case "3":
                    cloudInput.setModelCycle(4);
                    break;
            }
            XSingleResult<GetPlantTrainPowerTaskCloudOutput> result = cloudService.getPlantTrainPowerTaskData(context, cloudInput);
            if (result.isSuccess() && result.getResult() != null && CharSequenceUtil.isNotEmpty(result.getResult().getMetricsScore())) {
                Map<String, Object> map = XJsonUtils.toMap(result.getResult().getMetricsScore());
                if (map.containsKey("MAPE") && CharSequenceUtil.isNotEmpty(map.get("MAPE").toString())) {
                    BigDecimal relMape = new BigDecimal(map.get("MAPE").toString());
                    BigDecimal mape;
                    //获取调整的准确率附加值
                    BasePhotovoltaicPlantMapeAdjViewMapper mapeAdjMapper = context.getBean(BasePhotovoltaicPlantMapeAdjViewMapper.class);
                    BasePhotovoltaicPlantMapeAdjView record = new BasePhotovoltaicPlantMapeAdjView();
                    record.setPlantId(input.getId());
                    record.setMapeType(input.getMapeType());
                    BasePhotovoltaicPlantMapeAdjView view = mapeAdjMapper.selectOne(record);
                    if (ObjectUtil.isNotNull(view) && ObjectUtil.isNotNull(view.getMapePlusVal())) {
                        mape = relMape.add(view.getMapePlusVal());
                        if (mape.compareTo(BusinessConstant.BIG_DECIMAL_100) > 0)
                            mape = BusinessConstant.BIG_DECIMAL_100;
                    } else {
                        mape = relMape;
                    }
                    output.setMape(mape.setScale(2, RoundingMode.UP).toPlainString() + "%");
                }
            }
        }
        return XSingleResult.success(output);
    }

    @XText("获取调整光伏功率预测准确率按钮")
    @XApiGet
    public XSingleResult<MapeAjdPowerOutput> getAjdButton(XContext context) {
        MapeAjdPowerOutput output = new MapeAjdPowerOutput();
        output.setShowAdjButton(this.checkUserMapeAdjPower(context));
        return XSingleResult.success(output);
    }

    /**
     * 光伏电站模块--分页查询
     * GET /base/base-photovoltaic-plant/query-base-photovoltaic-plant
     * 接口ID：105466277
     * 接口地址：https://app.apifox.com/project/3196988/apis/api-105466277
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XPageResult}<{@link QueryBasePhotovoltaicPlantOutput}>
     */
    @XText("光伏电站模块--分页查询")
    @XApiGet
    public XPageResult<QueryBasePhotovoltaicPlantOutput> queryBasePhotovoltaicPlant(XContext context, QueryBasePhotovoltaicPlantInput input) {
        BasePhotovoltaicPlantMapper mapper = context.getBean(BasePhotovoltaicPlantMapper.class);
        String stationName = input.getStationName();
        QueryWrapper<BasePhotovoltaicPlantEnt> queryWrapper = new QueryWrapper<>();
        boolean equals = CharSequenceUtil.equals(BusinessConstant.LINE_FLAG_ID, input.getLineId());
        if (CharSequenceUtil.isNotEmpty(input.getLineId())) {
            queryWrapper.lambda()
                    .notExists(!equals, "SELECT 1 FROM base_power_line_plant w WHERE base_photovoltaic_plant.id = w.plant_id AND w.is_deleted = 1 AND w.line_id <> {0}", input.getLineId())
                    .notExists(equals, "SELECT 1 FROM base_power_line_plant w WHERE base_photovoltaic_plant.id = w.plant_id AND w.is_deleted = 1")
                    .eq(BasePhotovoltaicPlantEnt::getOuId, input.getOuId());
        } else {
            List<String> allListByOuId = ServiceUtil.getOrgIdsByPath(context, input.getOuId());
            queryWrapper.lambda()
                    .in(CollUtil.isNotEmpty(allListByOuId), BasePhotovoltaicPlantEnt::getOuId, allListByOuId);
        }
        queryWrapper.lambda()
                .eq(BasePhotovoltaicPlantEnt::getIsDeleted, BusinessConstant.ONE)
                .like(CharSequenceUtil.isNotEmpty(stationName), BasePhotovoltaicPlantEnt::getStationName, stationName)
                .orderByAsc(BasePhotovoltaicPlantEnt::getOuName, BasePhotovoltaicPlantEnt::getStationName);
        return XMapperHelper.query(mapper, input, queryWrapper, QueryBasePhotovoltaicPlantOutput.class);
    }

    /**
     * 按母公司ou-id选择ou和光伏发电厂信息
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetSysOrganizationAndPlantViewOutput }>
     */
    @XText("根据父级id查询儿子级(含电站)")
    @XApiPost
    public XListResult<GetSysOrganizationAndPlantViewOutput> selectOuAndPhotovoltaicPlantInfoByParentOuId(XContext context, GetSysOrganizationAndPlantInput input) {
        SysOrganizationView record = new SysOrganizationView();
        XCopyUtils.copyObject(input, record);
        record.setIsDeleted(BusinessConstant.ONE);
        List<SysOrganizationView> viewList = null;
        String ouId = "00000000-0000-0000-0000-000000000000";
        String ouName = input.getOuName();
        if (CharSequenceUtil.isBlank(ouName)) {
            SysOrganizationViewMapper mapper = context.getBean(SysOrganizationViewMapper.class);
            if (CharSequenceUtil.isEmpty(input.getParentOuId())) {
                PpsUserSession session = context.getSession(PpsUserSession.class);
                Integer isSuperAdmin = session.getIsSuperAdmin();
                //如果是超级管理员查所以组织机构
                if (BusinessConstant.ONE.equals(isSuperAdmin)) {
                    record.setParentOuId("00000000-0000-0000-0000-000000000000");
                    viewList = mapper.selectOuSonByParentOuId(record);
                } else {
                    ouId = session.getOuId();
                    record.setId(session.getOuId());
                    viewList = mapper.selectOuByOuId(record);
                }
            } else {
                ouId = input.getParentOuId();
                viewList = mapper.selectOuSonByParentOuId(record);
            }
        }
        if (CollUtil.isEmpty(viewList)) {
            BasePhotovoltaicPlantMapper plantMapper = context.getBean(BasePhotovoltaicPlantMapper.class);
            List<BasePhotovoltaicPlantEnt> plantList = plantMapper.selectList(new LambdaQueryWrapper<BasePhotovoltaicPlantEnt>()
                    .eq(CharSequenceUtil.isBlank(ouName), BasePhotovoltaicPlantEnt::getOuId, ouId)
                    .like(CharSequenceUtil.isNotBlank(ouName), BasePhotovoltaicPlantEnt::getStationName, ouName)
                    .eq(BasePhotovoltaicPlantEnt::getIsDeleted, BusinessConstant.ONE)
                    .orderByAsc(BasePhotovoltaicPlantEnt::getStationName)
            );
            for (BasePhotovoltaicPlantEnt item : plantList) {
                SysOrganizationView view = new SysOrganizationView();
                view.setId(item.getId());
                view.setOuName(item.getStationName());
                view.setShortName(item.getStationName());
                view.setOuLevel("PHOTOVOLTAIC");
                viewList.add(view);
            }
        }
        List<GetSysOrganizationAndPlantViewOutput> outputs = XCopyUtils.copyNewList(viewList, GetSysOrganizationAndPlantViewOutput.class);
        return XListResult.success(outputs);
    }

    /*-----------------------------------private-----------------------------------*/

    /**
     * 检查光伏电站是否被引用
     *
     * @param context 上下文
     * @param plantId 电站ID
     * @return boolean
     */
    private boolean checkPhotovoltaicIsReference(XContext context, String plantId) {
        BasePowerLinePlantMapper mapper = context.getBean(BasePowerLinePlantMapper.class);
        Long count = mapper.selectCount(new LambdaQueryWrapper<BasePowerLinePlantEnt>()
                .eq(BasePowerLinePlantEnt::getPlantId, plantId)
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
        );
        return count > 0;
    }

    /**
     * 通过电站ID获取电站信息
     *
     * @param plantId 电站id
     * @param mapper  映射器
     * @return {@link BasePowerLineEnt}
     */
    private BasePhotovoltaicPlantEnt selectOneByPlantId(String plantId, BasePhotovoltaicPlantMapper mapper) {
        return mapper.selectOne(new LambdaQueryWrapper<BasePhotovoltaicPlantEnt>()
                .eq(BaseModel::getId, plantId));
    }

    /**
     * 获取系统区域信息
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link GetSysAreaOutput}
     */
    private GetSysAreaOutput getSysAreaInfo(XContext context, GetSysAreaInput input) {
        SystemAreaService systemAreaService = context.getBean(SystemAreaService.class);
        XSingleResult<GetSysAreaOutput> singleResult = systemAreaService.getSysAreaByParam(context, input);
        singleResult.throwIfFail();
        return singleResult.getResult();
    }

    /**
     * 检查编号是否存在
     *
     * @param mapper      映射器
     * @param makerNumber 出场编号
     * @param id          id
     * @return boolean
     */
    private boolean checkMakerNumberExists(BasePhotovoltaicPlantMapper mapper, String makerNumber, String id) {
        Long count = mapper.selectCount(new LambdaQueryWrapper<BasePhotovoltaicPlantEnt>()
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                .eq(BasePhotovoltaicPlantEnt::getMakerNumber, makerNumber)
                .ne(CharSequenceUtil.isNotEmpty(id), BaseModel::getId, id)
        );
        return count > 0;
    }


    /**
     * 校验当前用户是否有调整光伏预测准确率的权限
     *
     * @param context
     * @return
     */
    private boolean checkUserMapeAdjPower(XContext context) {
        boolean result = false;
        try {
            //获取字典配置的角色
            SystemDictionaryService dictionaryService = context.getBean(SystemDictionaryService.class);
            CreateSysDictionaryInput createSysDictionaryInput = new CreateSysDictionaryInput();
            createSysDictionaryInput.setAlias(MapeConstant.MAPE_ADJ_ROLE_DIC_ALIAS);
            XListResult<QuerySysDictionaryViewOutput> dicResult = dictionaryService.querySysDictionarysByAlias(context, createSysDictionaryInput);
            if (!dicResult.isSuccess()) {
                context.getLogger().error("=======get dic error: alias=" + MapeConstant.MAPE_ADJ_ROLE_DIC_ALIAS + " errorMsg:" + XJsonUtils.toJson(dicResult.getError()));
                return result;
            }
            List<QuerySysDictionaryViewOutput> roleDicList = dicResult.getResult();
            if (CollUtil.isEmpty(roleDicList)) {
                return result;
            }
            List<String> roleIds = roleDicList.stream()
                    .map(QuerySysDictionaryViewOutput::getDicKey)
                    .collect(Collectors.toList());
            //获取当前用户角色
            SystemUserRoleRelService roleRelService = context.getBean(SystemUserRoleRelService.class);
            GetSysUserRoleRelInput getSysUserRoleRelInput = new GetSysUserRoleRelInput();
            getSysUserRoleRelInput.setUserId(context.getSession(PpsUserSession.class).getId());
            XListResult<GetSysUserRoleRelOutput> roleResult = roleRelService.getSysRoleByUserId(context, getSysUserRoleRelInput);
            if (!roleResult.isSuccess()) {
                context.getLogger().error("=======get user role error:" + XJsonUtils.toJson(roleResult.getError()));
                return result;
            }
            for (GetSysUserRoleRelOutput item : roleResult.getResult()) {
                if (roleIds.contains(item.getRoleId())) {
                    result = true;
                    break;
                }
            }
        } catch (Exception e) {
            context.getLogger().error(e);
        }
        return result;
    }
}
