package pps.core.base.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.text.CharSequenceUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import pps.cloud.base.service.IBasePowerLineCloudService;
import pps.cloud.base.service.data.base_power_line.DynamicQueryBasePowerLineInput;
import pps.cloud.base.service.data.base_power_line.DynamicQueryBasePowerLineOutput;
import pps.cloud.base.service.data.base_power_line.GetBasePowerLineInput;
import pps.cloud.base.service.data.base_power_line.GetBasePowerLineViewOutput;
import pps.cloud.base.service.data.base_power_line_plant.DynamicQueryBasePowerLinePlantInput;
import pps.cloud.base.service.data.base_power_line_plant.DynamicQueryBasePowerLinePlantOutput;
import pps.cloud.base.service.data.base_power_line_plant.DynamicQueryBasePowerLinePlantViewOutput;
import pps.cloud.base.service.data.base_power_line_storage.DynamicQueryBasePowerLineStorageInput;
import pps.cloud.base.service.data.base_power_line_storage.DynamicQueryBasePowerLineStorageOutput;
import pps.cloud.base.service.data.base_power_line_storage.DynamicQueryBasePowerLineStorageViewOutput;
import pps.cloud.base.service.data.base_power_line_wellhead.DynamicQueryBasePowerLineWellheadInput;
import pps.cloud.base.service.data.base_power_line_wellhead.DynamicQueryBasePowerLineWellheadViewOutput;
import pps.cloud.base.service.data.base_power_line_wellhead.GetBasePowerLineWellheadViewOutput;
import pps.core.base.entity.*;
import pps.core.base.enums.BusinessError;
import pps.core.base.mapper.*;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.data.XSingleResult;

import java.math.BigDecimal;
import java.util.Collection;
import java.util.List;
import java.util.Optional;

/**
 * 输电线路配置Cloud模块
 *
 * @author ZWT
 * @date 2023/09/05 15:37
 */
@XService
public class BasePowerLineCloudServiceImpl implements IBasePowerLineCloudService {

    /**
     * 输电线路配置Cloud模块--通过ID查询线路基本信息
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetBasePowerLineViewOutput}>
     */
    @Override
    public XSingleResult<GetBasePowerLineViewOutput> getBasePowerLineById(XContext context, GetBasePowerLineInput input) {
        BasePowerLineMapper mapper = context.getBean(BasePowerLineMapper.class);
        BasePowerLineEnt entity = mapper.selectOne(new LambdaQueryWrapper<BasePowerLineEnt>()
                .eq(BaseModel::getId, input.getId())
        );
        return XSingleResult.success(XCopyUtils.copyNewObject(entity, GetBasePowerLineViewOutput.class));
    }

    /**
     * 输电线路配置Cloud模块--条件查询线路基本信息列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryBasePowerLineOutput}>
     */
    @Override
    public XListResult<DynamicQueryBasePowerLineOutput> getBasePowerLineList(XContext context, DynamicQueryBasePowerLineInput input) {
        String lineId = input.getLineId();
        List<String> lineIds = input.getLineIds();
        List<String> lineNames = input.getLineNames();
        String ouId = input.getOuId();
        String upperLineId = input.getUpperLineId();
        String lineName = input.getLineName();
        String gridTypeKey = input.getGridTypeKey();
        String lineTypeKey = input.getLineTypeKey();
        String strategyId = input.getStrategyId();
        Collection<String> ouIdList = input.getOuIdList();
        BasePowerLineMapper mapper = context.getBean(BasePowerLineMapper.class);
        List<BasePowerLineEnt> selectList = mapper.selectList(new LambdaQueryWrapper<BasePowerLineEnt>()
                .select(BaseModel::getId, BasePowerLineEnt::getLineName, BasePowerLineEnt::getStrategyId, BasePowerLineEnt::getGridTypeKey)
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                .eq(CharSequenceUtil.isNotBlank(lineId), BaseModel::getId, lineId)
                .eq(CharSequenceUtil.isNotBlank(ouId), BasePowerLineEnt::getOuId, ouId)
                .eq(CharSequenceUtil.isNotBlank(strategyId), BasePowerLineEnt::getStrategyId, strategyId)
                .eq(CharSequenceUtil.isNotBlank(upperLineId), BasePowerLineEnt::getUpperLineId, upperLineId)
                .eq(CharSequenceUtil.isNotBlank(gridTypeKey), BasePowerLineEnt::getGridTypeKey, gridTypeKey)
                .eq(CharSequenceUtil.isNotBlank(lineTypeKey), BasePowerLineEnt::getLineTypeKey, lineTypeKey)
                .in(CollUtil.isNotEmpty(lineIds), BaseModel::getId, lineIds)
                .in(CollUtil.isNotEmpty(ouIdList), BasePowerLineEnt::getOuId, ouIdList)
                .in(CollUtil.isNotEmpty(lineNames), BasePowerLineEnt::getLineName, lineNames)
                .like(CharSequenceUtil.isNotBlank(lineName), BasePowerLineEnt::getLineName, lineName)
                .orderByDesc(BaseModel::getModifyTime)
        );
        return XListResult.success(XCopyUtils.copyNewList(selectList, DynamicQueryBasePowerLineOutput.class));
    }

    /**
     * 输电线路配置Cloud模块--查询组织列表下所有线路
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryBasePowerLineOutput}>
     */
    @Override
    public XListResult<DynamicQueryBasePowerLineOutput> getBasePowerLineListByOuIdList(XContext context, DynamicQueryBasePowerLineInput input) {
        Collection<String> ouIdList = input.getOuIdList();
        if (CollUtil.isEmpty(ouIdList)) {
            return XListResult.error(context, BusinessError.Missing_Organization_ID);
        }
        BasePowerLineMapper mapper = context.getBean(BasePowerLineMapper.class);
        List<BasePowerLineEnt> selectList = mapper.selectList(new LambdaQueryWrapper<BasePowerLineEnt>()
                .select(BaseModel::getId, BasePowerLineEnt::getLineName, BasePowerLineEnt::getOuId)
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                .in(BasePowerLineEnt::getOuId, ouIdList)
        );
        return XListResult.success(XCopyUtils.copyNewList(selectList, DynamicQueryBasePowerLineOutput.class));
    }

    /**
     * 输电线路配置Cloud模块--条件查询线路关联井口信息
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryBasePowerLineWellheadViewOutput}>
     */
    @Override
    public XListResult<DynamicQueryBasePowerLineWellheadViewOutput> getPowerLineWellheadList(XContext context, DynamicQueryBasePowerLineWellheadInput input) {
        BasePowerLineWellheadViewMapper wellheadViewMapper = context.getBean(BasePowerLineWellheadViewMapper.class);
        List<BasePowerLineWellheadView> wellheadViews = wellheadViewMapper.selectWellheadListByParam(
                BasePowerLineWellheadView.builder()
                        .ouIds(input.getOuIds())
                        .lineId(input.getLineId())
                        .lineIds(input.getLineIds())
                        .lineName(input.getLineName())
                        .runTypeKey(input.getRunTypeKey())
                        .wellheadIds(input.getWellheadIds())
                        .build()
        );
        return XListResult.success(XCopyUtils.copyNewList(wellheadViews, DynamicQueryBasePowerLineWellheadViewOutput.class));
    }

    /**
     * 输电线路配置Cloud模块--条件查询线路关联电站视图信息
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryBasePowerLinePlantViewOutput}>
     */
    @Override
    public XListResult<DynamicQueryBasePowerLinePlantViewOutput> getPowerLinePlantViewList(XContext context, DynamicQueryBasePowerLinePlantInput input) {
        BasePowerLinePlantViewMapper mapper = context.getBean(BasePowerLinePlantViewMapper.class);
        List<BasePowerLinePlantView> plantList = mapper.selectPlantListByParam(
                BasePowerLinePlantView.builder()
                        .ouIds(input.getOuIds())
                        .lineId(input.getLineId())
                        .lineIds(input.getLineIds())
                        .lineName(input.getLineName())
                        .build()
        );
        return XListResult.success(XCopyUtils.copyNewList(
                plantList,
                DynamicQueryBasePowerLinePlantViewOutput.class
        ));
    }

    /**
     * 输电线路配置Cloud模块--条件查询线路关联电站信息
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryBasePowerLinePlantOutput}>
     */
    @Override
    public XListResult<DynamicQueryBasePowerLinePlantOutput> queryPowerLinePlantListByParam(XContext context, DynamicQueryBasePowerLinePlantInput input) {
        String plantId = input.getPlantId();
        Collection<String> lineIds = input.getLineIds();
        BasePowerLinePlantMapper mapper = context.getBean(BasePowerLinePlantMapper.class);
        List<BasePowerLinePlantEnt> list = mapper.selectList(
                new LambdaQueryWrapper<BasePowerLinePlantEnt>()
                        .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                        .eq(CharSequenceUtil.isNotBlank(plantId), BasePowerLinePlantEnt::getPlantId, plantId)
                        .in(CollUtil.isNotEmpty(lineIds), BasePowerLinePlantEnt::getLineId, lineIds)
        );
        return XListResult.success(XCopyUtils.copyNewList(
                list,
                DynamicQueryBasePowerLinePlantOutput.class
        ));
    }

    /**
     * 输电线路配置Cloud模块--条件查询线路关联储能设备参数平均值
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryBasePowerLineStorageViewOutput}>
     */
    @Override
    public XListResult<DynamicQueryBasePowerLineStorageViewOutput> queryPowerLineStorageAvgByLineIds(XContext context, DynamicQueryBasePowerLineStorageInput input) {
        BasePowerLineStorageViewMapper mapper = context.getBean(BasePowerLineStorageViewMapper.class);
        List<BasePowerLineStorageView> list = mapper.selectStorageAverageByLineIds(input.getLineIds());
        return XListResult.success(XCopyUtils.copyNewList(
                list,
                DynamicQueryBasePowerLineStorageViewOutput.class
        ));
    }

    /**
     * 输电线路配置Cloud模块--条件查询线路关联储能设备信息
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryBasePowerLineStorageOutput}>
     */
    @Override
    public XListResult<DynamicQueryBasePowerLineStorageOutput> queryPowerLineStorageListByLineIds(XContext context, DynamicQueryBasePowerLineStorageInput input) {
        BasePowerLineStorageMapper storageMapper = context.getBean(BasePowerLineStorageMapper.class);
        List<BasePowerLineStorageEnt> list = storageMapper.selectList(new LambdaQueryWrapper<BasePowerLineStorageEnt>()
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                .in(BasePowerLineStorageEnt::getLineId, input.getLineIds())
        );
        return XListResult.success(XCopyUtils.copyNewList(
                list,
                DynamicQueryBasePowerLineStorageOutput.class
        ));
    }

    /**
     * 输电线路配置Cloud模块-查询线路井口运行总功率
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetBasePowerLineWellheadViewOutput}>
     */
    @Override
    public XSingleResult<GetBasePowerLineWellheadViewOutput> queryLineSumServiceRatingByLineId(XContext context, DynamicQueryBasePowerLineWellheadInput input) {
        BaseWellheadMapper mapper = context.getBean(BaseWellheadMapper.class);
        BaseWellheadEnt baseWellheadEnt = mapper.selectOne(
                new QueryWrapper<BaseWellheadEnt>()
                        .select("IFNULL( SUM( service_rating ), 0 ) AS service_rating")
                        .lambda()
                        .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                        .apply("id IN ( SELECT wellhead_id FROM base_power_line_wellhead WHERE line_id = {0} AND is_deleted = 1 )", input.getLineId())
        );
        return XSingleResult.success(GetBasePowerLineWellheadViewOutput.builder()
                .serviceRating(Optional.ofNullable(baseWellheadEnt)
                        .map(BaseWellheadEnt::getServiceRating)
                        .orElse(BigDecimal.ZERO))
                .build());
    }

    /**
     * 通过电站ID查井口
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryBasePowerLineWellheadViewOutput}>
     */
    @Override
    public XListResult<DynamicQueryBasePowerLineWellheadViewOutput> getPowerLineWellheadListByPlant(XContext context, DynamicQueryBasePowerLineWellheadInput input) {
        BasePowerLineWellheadViewMapper wellheadViewMapper = context.getBean(BasePowerLineWellheadViewMapper.class);
        List<BasePowerLineWellheadView> wellheadViews = wellheadViewMapper.selectWellheadListByPlant(
                BasePowerLineWellheadView.builder()
                        .plantIds(input.getPlantIds())
                        .build()
        );
        return XListResult.success(XCopyUtils.copyNewList(wellheadViews, DynamicQueryBasePowerLineWellheadViewOutput.class));
    }
}