package pps.core.base.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.text.CharSequenceUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import pps.cloud.base.service.IBasePriceStrategyCloudService;
import pps.cloud.base.service.data.base_price_strategy_detail.GetBasePriceStrategyDetailInput;
import pps.cloud.base.service.data.base_price_strategy_detail.GetBasePriceStrategyDetailOutput;
import pps.core.base.entity.BasePriceStrategyDetailEnt;
import pps.core.base.mapper.BasePriceStrategyDetailMapper;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;

import java.util.Collections;
import java.util.List;

/**
 * 市电峰谷配置Cloud模块
 *
 * @author ZWT
 * @date 2023/09/07 14:52
 */
@XService
public class BasePriceStrategyCloudServiceImpl implements IBasePriceStrategyCloudService {

    /**
     * 市电峰谷配置Cloud模块--策略配置详情列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link GetBasePriceStrategyDetailOutput}>
     */
    @Override
    public XListResult<GetBasePriceStrategyDetailOutput> queryStrategyDetailList(XContext context, GetBasePriceStrategyDetailInput input) {
        String periodTypeKey = input.getPeriodTypeKey();
        String lineId = input.getLineId();
        //查明细
        BasePriceStrategyDetailMapper detailMapper = context.getBean(BasePriceStrategyDetailMapper.class);
        List<BasePriceStrategyDetailEnt> detailList = detailMapper.selectList(
                new QueryWrapper<BasePriceStrategyDetailEnt>()
                        .select("*",
                                "STR_TO_DATE( CONCAT( start_time, ':00' ), '%H:%i:%s' ) AS openTime",
                                "STR_TO_DATE( CONCAT( end_time, ':00' ), '%H:%i:%s' ) AS closeTime"
                        )
                        .lambda()
                        .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                        .eq(BasePriceStrategyDetailEnt::getStrategyMonth, input.getStrategyMonth())
                        .eq(CharSequenceUtil.isNotBlank(periodTypeKey), BasePriceStrategyDetailEnt::getPeriodTypeKey, periodTypeKey)
                        .apply(CharSequenceUtil.isNotBlank(lineId), "strategy_id = ( SELECT strategy_id FROM base_power_line WHERE id = {0} ) ", lineId)
                        .orderByAsc(BasePriceStrategyDetailEnt::getStartTime)
        );
        List<GetBasePriceStrategyDetailOutput> outputs;
        if (CollUtil.isEmpty(detailList)) {
            outputs = Collections.emptyList();
        } else {
            outputs = XCopyUtils.copyNewList(detailList, GetBasePriceStrategyDetailOutput.class);
        }
        return XListResult.success(outputs);
    }
}