package pps.core.base.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageInfo;
import pps.core.base.entity.BaseLineAttenuationRateEnt;
import pps.core.base.entity.BaseLineAttenuationRateView;
import pps.core.base.entity.BaseProjectInfoEnt;
import pps.core.base.entity.BaseProjectInfoPredictedDataEnt;
import pps.core.base.mapper.*;
import pps.core.base.service.data.base_line_attenuation_rate.CreateBaseLineAttenuationRateInput;
import pps.core.base.service.data.base_project_info.*;
import pps.core.common.session.PpsUserSession;
import pps.core.system.entity.SysAreaEnt;
import pps.core.system.mapper.SysAreaMapper;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XHttpUtils;
import xstartup.base.util.XJsonUtils;
import xstartup.core.base.helper.XThreadHelper;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.stream.Collectors;

@XService
public class BaseProjectInfoService {

    @XApiPost
    public XServiceResult createProjectInfo(XContext context, CreateBaseProjectInfoInput input) {
        BaseProjectInfoMapper mapper = context.getBean(BaseProjectInfoMapper.class);
        BaseProjectInfoEnt entity = new BaseProjectInfoEnt();
        XCopyUtils.copyObject(input, entity);
        Date now = new Date();
        PpsUserSession session = context.getSession(PpsUserSession.class);
        entity.setCreateById(session.getId());
        entity.setCreateByName(session.getUserName());
        entity.setCreateTime(now);
        entity.setIsDeleted(0);
        mapper.insert(entity);
        List<BaseLineAttenuationRateView> list = new ArrayList<>();
        if (CollUtil.isNotEmpty(input.getList())) {
            BaseLineAttenuationRateViewMapper rateMapper = context.getBean(BaseLineAttenuationRateViewMapper.class);
            for (CreateBaseLineAttenuationRateInput rateInput : input.getList()) {
                BaseLineAttenuationRateView ent = XCopyUtils.copyNewObject(rateInput, BaseLineAttenuationRateView.class);
                ent.setType(1);
                ent.setCreateTime(now);
                ent.setLineId(entity.getId());
                ent.setCreateById(session.getId());
                ent.setCreateByName(session.getUserName());
                list.add(ent);
            }
            rateMapper.batchInsert(list);
        }
        //开始预测
        XThreadHelper.async(() -> predictedData(context, entity, list));
        return XServiceResult.OK;
    }

    @XApiPost
    public XServiceResult updateProjectInfo(XContext context, UpdateBaseProjectInfoInput input) {
        BaseProjectInfoMapper mapper = context.getBean(BaseProjectInfoMapper.class);
        BaseProjectInfoEnt entity = mapper.selectOne(new LambdaQueryWrapper<BaseProjectInfoEnt>()
                .eq(BaseProjectInfoEnt::getId, input.getId()));
        if (ObjectUtil.isNull(entity)) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        Date now = new Date();
        PpsUserSession session = context.getSession(PpsUserSession.class);
        entity.setIsDeleted(0);
        entity.setModifyTime(now);
        entity.setModifyById(session.getId());
        entity.setModifyByName(session.getUserName());
        mapper.updateById(entity);
        BaseLineAttenuationRateMapper rateMapper = context.getBean(BaseLineAttenuationRateMapper.class);
        rateMapper.delete(new LambdaQueryWrapper<BaseLineAttenuationRateEnt>()
                .eq(BaseLineAttenuationRateEnt::getType, 1)
                .eq(BaseLineAttenuationRateEnt::getLineId, entity.getId())
        );
        List<BaseLineAttenuationRateView> list = new ArrayList<>();
        if (CollUtil.isNotEmpty(input.getList())) {
            BaseLineAttenuationRateViewMapper rateViewMapper = context.getBean(BaseLineAttenuationRateViewMapper.class);
            for (CreateBaseLineAttenuationRateInput rateInput : input.getList()) {
                BaseLineAttenuationRateView ent = XCopyUtils.copyNewObject(rateInput, BaseLineAttenuationRateView.class);
                ent.setType(1);
                ent.setCreateTime(now);
                ent.setLineId(entity.getId());
                ent.setCreateById(session.getId());
                ent.setCreateByName(session.getUserName());
                list.add(ent);
            }
            rateViewMapper.batchInsert(list);
        }
        //开始预测
        XThreadHelper.async(() -> predictedData(context, entity, list));
        return XServiceResult.OK;
    }

    @XApiPost
    public XServiceResult deleteProjectInfo(XContext context, DeleteProjectInfoInput input) {
        BaseProjectInfoMapper mapper = context.getBean(BaseProjectInfoMapper.class);
        BaseProjectInfoEnt entity = mapper.selectOne(new LambdaQueryWrapper<BaseProjectInfoEnt>()
                .eq(BaseProjectInfoEnt::getId, input.getId()));
        if (ObjectUtil.isNull(entity)) {
            return XServiceResult.error(context, XError.NotFound);
        }
        entity.setIsDeleted(1);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XApiGet
    public XSingleResult<GetBaseProjectInfoOutput> getProjectInfo(XContext context, GetProjectInfoInput input) {
        BaseProjectInfoMapper mapper = context.getBean(BaseProjectInfoMapper.class);
        BaseProjectInfoEnt entity = mapper.selectOne(new LambdaQueryWrapper<BaseProjectInfoEnt>()
                .eq(BaseProjectInfoEnt::getId, input.getId()));
        if (ObjectUtil.isNull(entity)) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseProjectInfoOutput output = new GetBaseProjectInfoOutput();
        XCopyUtils.copyObject(entity, output);
        if (CharSequenceUtil.isNotBlank(entity.getProvince())) {
            output.setProvince(Integer.valueOf(entity.getProvince()));
        }
        BaseLineAttenuationRateMapper rateMapper = context.getBean(BaseLineAttenuationRateMapper.class);
        List<BaseLineAttenuationRateEnt> list = rateMapper.selectList(new LambdaQueryWrapper<BaseLineAttenuationRateEnt>()
                .eq(BaseLineAttenuationRateEnt::getLineId, entity.getId())
                .eq(BaseLineAttenuationRateEnt::getType, 1));
        if (CollUtil.isNotEmpty(list)) {
            List<CreateBaseLineAttenuationRateInput> rateInputs = XCopyUtils.copyNewList(list, CreateBaseLineAttenuationRateInput.class);
            output.setList(rateInputs);
        }
        return XSingleResult.success(output);
    }

    @XApiGet
    public XPageResult<QueryBaseProjectInfoOutput> queryProjectInfoList(XContext context, QueryProjectInfoInput input) {
        BaseProjectInfoMapper mapper = context.getBean(BaseProjectInfoMapper.class);
        List<BaseProjectInfoEnt> list = mapper.selectList(new LambdaQueryWrapper<BaseProjectInfoEnt>()
                .like(CharSequenceUtil.isNotBlank(input.getProjectName()), BaseProjectInfoEnt::getProjectName, input.getProjectName())
                .like(CharSequenceUtil.isNotBlank(input.getProductionYear()), BaseProjectInfoEnt::getProductionYear, input.getProductionYear())
        );
        PageInfo<BaseProjectInfoEnt> pageInfo = new PageInfo<>(list);
        List<QueryBaseProjectInfoOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBaseProjectInfoOutput.class);
        SysAreaMapper areaMapper = context.getBean(SysAreaMapper.class);
        List<SysAreaEnt> areaEntList = areaMapper.selectList(new QueryWrapper<>());
        outputs.forEach(item -> {
            if (CharSequenceUtil.isNotBlank(item.getProvince())) {
                areaEntList.stream()
                        .filter(area -> CharSequenceUtil.equals(item.getProvince(), area.getId().toString()))
                        .findFirst()
                        .ifPresent(areaEnt -> item.setProvince(getParentNameStr(areaEntList, areaEnt)));
            }
        });
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    @XApiGet
    public XListResult<GetProjectInfoPredictedDataOutput> queryProjectInfoPredictedDataList(XContext context, GetProjectInfoPredictedDataInput input) {
        BaseProjectInfoPredictedDataMapper mapper = context.getBean(BaseProjectInfoPredictedDataMapper.class);
        List<BaseProjectInfoPredictedDataEnt> list = mapper.selectList(new LambdaQueryWrapper<BaseProjectInfoPredictedDataEnt>()
                .eq(ObjectUtil.isNotNull(input.getProjectId()), BaseProjectInfoPredictedDataEnt::getProjectId, input.getProjectId())
        );
        PageInfo<BaseProjectInfoPredictedDataEnt> pageInfo = new PageInfo<>(list);
        List<GetProjectInfoPredictedDataOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), GetProjectInfoPredictedDataOutput.class);
        return XListResult.success(outputs);
    }

    private void predictedData(XContext context, BaseProjectInfoEnt entity, List<BaseLineAttenuationRateView> list) {
        CalcProjectInfoPredictedDataInput input = new CalcProjectInfoPredictedDataInput();
        input.setLat(entity.getLatitude());
        input.setLon(entity.getLongitude());
        input.setAngle(entity.getArrayIncidence());
        input.setDirection(entity.getArrayToward());
        input.setCapacity(entity.getInstallCapacity());
        input.setEfficiency(new BigDecimal("0.35").setScale(2, RoundingMode.HALF_EVEN));
        input.setWeather(new BigDecimal("0.7").setScale(2, RoundingMode.HALF_EVEN));
        list.sort(Comparator.comparing(BaseLineAttenuationRateView::getSort));
        List<BigDecimal> decimals = list.stream().map(BaseLineAttenuationRateView::getAttenuationRate).collect(Collectors.toList());
        input.setDecline(decimals);
        Map<String, Object> map = getPredictedData(context, input);
        //插入数据
        BaseProjectInfoPredictedDataMapper mapper = context.getBean(BaseProjectInfoPredictedDataMapper.class);
        mapper.delete(new LambdaQueryWrapper<BaseProjectInfoPredictedDataEnt>()
                .eq(BaseProjectInfoPredictedDataEnt::getProjectId, entity.getId()));
        List<Double> powers = (List<Double>) map.get("power");
        Integer particularYear = Integer.valueOf(entity.getProductionYear());
        PpsUserSession userSession = context.getSession(PpsUserSession.class);
        Date date = new Date();
        List<BaseProjectInfoPredictedDataEnt> entList = new ArrayList<>();
        for (Double decimal : powers) {
            BaseProjectInfoPredictedDataEnt ent = new BaseProjectInfoPredictedDataEnt();
            ent.setProjectId(entity.getId());
            ent.setParticularYear(particularYear + "");
            ent.setPredictedPower(BigDecimal.valueOf(decimal));
            ent.setCreateById(userSession.getId());
            ent.setCreateByName(userSession.getUserName());
            ent.setCreateTime(date);
            particularYear++;
            entList.add(ent);
        }
        BaseProjectInfoPredictedDataViewMapper dataViewMapper = context.getBean(BaseProjectInfoPredictedDataViewMapper.class);
        dataViewMapper.batchInsert(entList);
    }

    public Map<String, Object> getPredictedData(XContext context, CalcProjectInfoPredictedDataInput calcProjectInfoPredictedDataInput) {
        Map<String, Object> returnMap;
        try {
            String jsonString = XJsonUtils.toJson(calcProjectInfoPredictedDataInput);
            context.getLogger().info(jsonString);
            String result = XHttpUtils.postAsJson("http://121.36.43.217:8098//model/yearmodel", jsonString, 10000);
            returnMap = XJsonUtils.toMap(result);
            return returnMap;
        } catch (Exception e) {
            throw new RuntimeException("调用预测接口失败");
        }
    }

    /**
     * 根据子节点获取最上层节点
     *
     * @return
     */
    private String getParentNameStr(List<SysAreaEnt> areaList, SysAreaEnt child) {
        String parentNameStr = child.getName();
        Integer parentId = child.getParentId();
        if (!parentId.equals(120)) {
            SysAreaEnt parent = areaList.stream().filter(item -> item.getId().equals(parentId)).findFirst().orElse(null);
            String nameStr = getParentNameStr(areaList, parent);
            parentNameStr = nameStr + "/" + parentNameStr;
        }
        return parentNameStr;
    }
}