package pps.core.base.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.text.CharSequenceUtil;
import com.alibaba.excel.EasyExcelFactory;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import pps.cloud.base.service.data.config_oil_field.GetConfigOilFieldOutput;
import pps.cloud.system.service.data.sys_area.QuerySysAreaOutput;
import pps.cloud.system.service.data.sys_dictionary.GetSysDictionaryViewOutput;
import pps.cloud.system.service.data.sys_organization.GetSysOrganizationViewOutput;
import pps.core.base.entity.*;
import pps.core.base.mapper.*;
import pps.core.base.service.data.base_diesel_generator.QueryBaseDieselGeneratorInput;
import pps.core.base.service.data.base_diesel_generator.QueryBaseDieselGeneratorOutput;
import pps.core.base.service.data.base_energy_storage_device.QueryBaseEnergyStorageDeviceInput;
import pps.core.base.service.data.base_energy_storage_device.QueryBaseEnergyStorageDeviceOutput;
import pps.core.base.service.data.base_excel.*;
import pps.core.base.service.data.base_photovoltaic_plant.QueryBasePhotovoltaicPlantInput;
import pps.core.base.service.data.base_photovoltaic_plant.QueryBasePhotovoltaicPlantOutput;
import pps.core.base.service.data.base_price_strategy.GetBasePriceStrategyViewInput;
import pps.core.base.service.data.base_wellhead.QueryBaseWellheadInput;
import pps.core.base.service.data.base_wellhead.QueryBaseWellheadOutput;
import pps.core.base.utils.ServiceUtil;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import pps.core.common.excel.config.CascadeWriteHandler;
import pps.core.common.excel.util.EasyExcelUtil;
import pps.core.common.excel.util.ExcelStyleTool;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.BaseUtils;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.data.XPageInput;
import xstartup.base.exception.XServiceException;
import xstartup.base.tool.XStorageTool;
import xstartup.data.XFileResult;
import xstartup.data.XFileType;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiUpload;
import xstartup.helper.XTransactionHelper;

import java.math.BigDecimal;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * excel导入导出
 *
 * @author ZWT
 * @date 2024/05/07 09:49
 */
@XText("Excel导入导出")
@XService
public class ExcelService {

    /**
     * 光伏电站配置导出
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XFileResult }
     */
    @XText("光伏电站配置导出")
    @XApiGet
    public XFileResult exportPhotovoltaicPlant(XContext context, QueryBasePhotovoltaicPlantInput input) {
        List<QuerySysAreaOutput> provinceAndCityList = this.getProvinceAndCityList(context);
        this.setPageDefault(input);
        //分页条件查询光伏电站列表
        List<ExcelPhotovoltaicPlantTemplate> list;
        List<QueryBasePhotovoltaicPlantOutput> items = context.getBean(BasePhotovoltaicPlantService.class)
                .queryBasePhotovoltaicPlant(context, input)
                .getResult()
                .getItems();
        if (CollUtil.isNotEmpty(items)) {
            Map<Integer, QuerySysAreaOutput> collect = provinceAndCityList.stream()
                    .collect(Collectors.toMap(QuerySysAreaOutput::getId, Function.identity()));
            list = new ArrayList<>(items.size());
            //封装并填充数据
            String province;
            String city;
            for (QueryBasePhotovoltaicPlantOutput item : items) {
                QuerySysAreaOutput area = collect.get(item.getAreaCode());
                if (collect.containsKey(item.getAreaCode())) {
                    province = area.getProvince();
                    city = area.getCity();
                } else {
                    province = BusinessConstant.DEFAULT_VALUE;
                    city = BusinessConstant.DEFAULT_VALUE;
                }
                list.add(
                        ExcelPhotovoltaicPlantTemplate.builder()
                                .province(province)
                                .city(city)
                                .ouName(item.getOuName())
                                .stationName(item.getStationName())
                                .makerNumber(item.getMakerNumber())
                                .photovoltaicModelName(item.getPhotovoltaicModelName())
                                .mountingAngle(item.getMountingAngle().toString())
                                .totalPower(item.getTotalPower().toString())
                                .arrayOrientation(item.getArrayOrientation().toString())
                                .longitude(item.getLongitude().toString())
                                .latitude(item.getLatitude().toString())
                                .elevation(item.getElevation().toString())
                                .build()
                );
            }
        } else {
            list = Collections.emptyList();
        }
        //查地区列表
        Map<String, List<QuerySysAreaOutput>> collect = provinceAndCityList.stream()
                .collect(Collectors.groupingBy(QuerySysAreaOutput::getProvince));
        //整理数据，放入一个Map中，key存放父地点，value 存放该地点下的子区域
        Map<String, List<String>> siteMap = new HashMap<>(collect.size());
        List<String> provNameList = new ArrayList<>();
        for (String province : collect.keySet()) {
            provNameList.add(province);
            siteMap.put(province, collect.get(province).stream()
                    .map(QuerySysAreaOutput::getCity)
                    .collect(Collectors.toList())
            );
        }
        String fileSavePath = XStorageTool.getAbsolutePath("/temp/excel/光伏电站配置模板.xlsx");
        //生成模板
        EasyExcelFactory.write(FileUtil.touch(fileSavePath))
                .registerWriteHandler(ExcelStyleTool.getStyleStrategy())
                .registerWriteHandler(new CascadeWriteHandler(provNameList, siteMap))
                .autoCloseStream(Boolean.TRUE)
                .build()
                .write(list, EasyExcelUtil.writeSelectedSheet(ExcelPhotovoltaicPlantTemplate.class, 0, "光伏电站配置"))
                .finish();
        return XFileResult.success(XFileType.APPLICATION_XLSX, fileSavePath);
    }

    /**
     * 井口配置导出
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XFileResult }
     */
    @XText("井口配置导出")
    @XApiGet
    public XFileResult exportWellhead(XContext context, QueryBaseWellheadInput input) {
        this.setPageDefault(input);
        //分页条件查询井口列表
        List<ExcelWellheadTemplate> list;
        List<QueryBaseWellheadOutput> items = context.getBean(BaseWellheadService.class)
                .queryBaseWellhead(context, input)
                .getResult()
                .getItems();
        if (CollUtil.isNotEmpty(items)) {
            list = new ArrayList<>(items.size());
            //封装并填充数据
            for (QueryBaseWellheadOutput item : items) {
                list.add(
                        ExcelWellheadTemplate.builder()
                                .ouName(item.getOuName())
                                .wellNumber(item.getWellNumber())
                                .runTypeName(item.getRunTypeName())
                                .serviceRating(item.getServiceRating().toString())
                                .ratedPower(item.getRatedPower().toString())
                                .build()
                );
            }
        } else {
            list = Collections.emptyList();
        }
        String fileSavePath = XStorageTool.getAbsolutePath("/temp/excel/井口配置模板.xlsx");
        //生成模板
        EasyExcelFactory.write(FileUtil.touch(fileSavePath))
                .registerWriteHandler(ExcelStyleTool.getStyleStrategy())
                .autoCloseStream(Boolean.TRUE)
                .build()
                .write(list, EasyExcelUtil.writeSelectedSheet(ExcelWellheadTemplate.class, 0, "井口配置"))
                .finish();
        return XFileResult.success(XFileType.APPLICATION_XLSX, fileSavePath);
    }

    /**
     * 柴发设备配置导出
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XFileResult }
     */
    @XText("柴发设备配置导出")
    @XApiGet
    public XFileResult exportDieselGenerator(XContext context, QueryBaseDieselGeneratorInput input) {
        this.setPageDefault(input);
        //分页条件查询柴发设备列表
        List<ExcelDieselGeneratorTemplate> list;
        List<QueryBaseDieselGeneratorOutput> items = context.getBean(BaseDieselGeneratorService.class)
                .queryBaseDieselGenerator(context, input)
                .getResult()
                .getItems();
        if (CollUtil.isNotEmpty(items)) {
            list = new ArrayList<>(items.size());
            //封装并填充数据
            for (QueryBaseDieselGeneratorOutput item : items) {
                list.add(
                        ExcelDieselGeneratorTemplate.builder()
                                .ouName(item.getOuName())
                                .deviceName(item.getDeviceName())
                                .makerNumber(item.getMakerNumber())
                                .unitTypeName(item.getUnitTypeName())
                                .normalPower(item.getNormalPower().toString())
                                .standbyPower(item.getStandbyPower().toString())
                                .build()
                );
            }
        } else {
            list = Collections.emptyList();
        }
        String fileSavePath = XStorageTool.getAbsolutePath("/temp/excel/柴发设备配置模板.xlsx");
        //生成模板
        EasyExcelFactory.write(FileUtil.touch(fileSavePath))
                .registerWriteHandler(ExcelStyleTool.getStyleStrategy())
                .autoCloseStream(Boolean.TRUE)
                .build()
                .write(list, EasyExcelUtil.writeSelectedSheet(ExcelDieselGeneratorTemplate.class, 0, "柴发设备配置"))
                .finish();
        return XFileResult.success(XFileType.APPLICATION_XLSX, fileSavePath);
    }

    /**
     * 储能设备配置导出
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XFileResult }
     */
    @XText("储能设备配置导出")
    @XApiGet
    public XFileResult exportEnergyStorageDevice(XContext context, QueryBaseEnergyStorageDeviceInput input) {
        this.setPageDefault(input);
        //分页条件查询储能设备列表
        List<ExcelEnergyStorageDeviceTemplate> list;
        List<QueryBaseEnergyStorageDeviceOutput> items = context.getBean(BaseEnergyStorageDeviceService.class)
                .queryBaseEnergyStorageDevice(context, input)
                .getResult()
                .getItems();
        if (CollUtil.isNotEmpty(items)) {
            list = new ArrayList<>(items.size());
            //封装并填充数据
            for (QueryBaseEnergyStorageDeviceOutput item : items) {
                list.add(
                        ExcelEnergyStorageDeviceTemplate.builder()
                                .ouName(item.getOuName())
                                .deviceName(item.getDeviceName())
                                .makerNumber(item.getMakerNumber())
                                .storageModelName(item.getStorageModelName())
                                .apparatusCapacity(item.getApparatusCapacity().toString())
                                .ratedChargingPower(item.getRatedChargingPower().toString())
                                .ratedDischargePower(item.getRatedDischargePower().toString())
                                .ratedDischargeDepth(item.getRatedDischargeDepth().toString())
                                .ratedDischargeEfficiency(item.getRatedDischargeEfficiency().toString())
                                .build()
                );
            }
        } else {
            list = Collections.emptyList();
        }
        String fileSavePath = XStorageTool.getAbsolutePath("/temp/excel/储能设备配置模板.xlsx");
        //生成模板
        EasyExcelFactory.write(FileUtil.touch(fileSavePath))
                .registerWriteHandler(ExcelStyleTool.getStyleStrategy())
                .autoCloseStream(Boolean.TRUE)
                .build()
                .write(list, EasyExcelUtil.writeSelectedSheet(ExcelEnergyStorageDeviceTemplate.class, 0, "储能设备配置"))
                .finish();
        return XFileResult.success(XFileType.APPLICATION_XLSX, fileSavePath);
    }

    /**
     * 市电峰谷配置导出
     *
     * @param context 上下文
     * @return {@link XFileResult }
     */
    @XText("市电峰谷配置导出")
    @XApiGet
    public XFileResult exportPriceStrategy(XContext context, GetBasePriceStrategyViewInput input) {
        List<ExcelPriceStrategyTemplate> list;
        String strategyId = input.getId();
        if (CharSequenceUtil.isNotBlank(strategyId)) {
            BasePriceStrategyDetailView view = new BasePriceStrategyDetailView();
            view.setStrategyId(strategyId);
            String month = input.getMonth();
            if (CharSequenceUtil.isNotBlank(month)) {
                view.setMonthList(CharSequenceUtil.splitTrim(month, ","));
            }
            BasePriceStrategyDetailViewMapper mapper = context.getBean(BasePriceStrategyDetailViewMapper.class);
            List<BasePriceStrategyDetailView> strategyList = mapper.selectDetailList(view);
            list = new ArrayList<>(strategyList.size());
            //封装数据
            if (CollUtil.isNotEmpty(strategyList)) {
                Map<String, String> collect = ServiceUtil.getDictionaryList(context, BusinessConstant.TIME_FRAME).stream()
                        .collect(Collectors.toMap(GetSysDictionaryViewOutput::getDicKey, GetSysDictionaryViewOutput::getDicName));
                for (BasePriceStrategyDetailView entity : strategyList) {
                    list.add(
                            ExcelPriceStrategyTemplate.builder()
                                    .month(entity.getStrategyMonth())
                                    .periodTypeName(collect.get(entity.getPeriodTypeKey()))
                                    .startTime(entity.getStartTime())
                                    .endTime(entity.getEndTime())
                                    .electrovalence(entity.getElectrovalence().toString())
                                    .build()
                    );
                }
            }
        } else {
            list = Collections.emptyList();
        }
        String fileSavePath = XStorageTool.getAbsolutePath("/temp/excel/市电峰谷配置模板.xlsx");
        //生成模板
        EasyExcelFactory.write(FileUtil.touch(fileSavePath))
                .registerWriteHandler(ExcelStyleTool.getStyleStrategy())
                .autoCloseStream(Boolean.TRUE)
                .build()
                .write(list, EasyExcelUtil.writeSelectedSheet(ExcelPriceStrategyTemplate.class, 0, "市电峰谷配置"))
                .finish();
        return XFileResult.success(XFileType.APPLICATION_XLSX, fileSavePath);
    }

    /*------------------------------------ 导入 ------------------------------------*/

    /**
     * 光伏电站配置导入
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult }
     */
    @XText("光伏电站配置导入")
    @XApiUpload
    public XServiceResult importPhotovoltaicPlant(XContext context, ImportExcelInput input) {
        List<ExcelPhotovoltaicPlantTemplate> list = EasyExcelUtil.getExcelDataAndCheck(input.getFile(), ExcelPhotovoltaicPlantTemplate.class,
                "电站名称", ExcelPhotovoltaicPlantTemplate::getStationName);
        if (CollUtil.isEmpty(list)) {
            return XServiceResult.OK;
        }
        //整理各种字典数据
        Map<String, String> ouMap = this.getPlantMap(context);
        Map<String, Integer> city = this.getProvinceAndCityList(context).stream()
                .collect(Collectors.toMap(QuerySysAreaOutput::getCity, QuerySysAreaOutput::getId));
        //封装并校验电站名称(校验电站名称是否重复,重复不导入),并插入数据
        Set<String> collect = context.getBean(BasePhotovoltaicPlantMapper.class)
                .selectList(new LambdaQueryWrapper<BasePhotovoltaicPlantEnt>()
                        .eq(BaseModel::getIsDeleted, 1))
                .stream()
                .map(BasePhotovoltaicPlantEnt::getStationName)
                .collect(Collectors.toSet());
        List<Integer> errorRowList = new ArrayList<>(list.size());
        List<BasePhotovoltaicPlantView> insertList = new ArrayList<>(list.size());
        int rowNum = 2;
        PpsUserSession session = context.getSession(PpsUserSession.class);
        for (ExcelPhotovoltaicPlantTemplate ent : list) {
            //检查名称是否重复
            if (collect.contains(ent.getStationName())) {
                errorRowList.add(rowNum);
            } else {
                //数据封装
                BasePhotovoltaicPlantView build = BasePhotovoltaicPlantView.builder()
                        .ouName(ent.getOuName())
                        .ouId(ouMap.get(ent.getOuName()))
                        .areaCode(city.get(ent.getCity()))
                        .stationName(ent.getStationName())
                        .makerNumber(ent.getMakerNumber())
                        .photovoltaicModelName(ent.getPhotovoltaicModelName())
                        .mountingAngle(new BigDecimal(ent.getMountingAngle()))
                        .totalPower(new BigDecimal(ent.getTotalPower()))
                        .arrayOrientation(new BigDecimal(ent.getArrayOrientation()))
                        .longitude(new BigDecimal(ent.getLongitude()))
                        .latitude(new BigDecimal(ent.getLatitude()))
                        .elevation(new BigDecimal(ent.getElevation()))
                        .build();
                BaseUtils.createBaseModelDefault(build, session);
                insertList.add(build);
            }
            rowNum++;
        }
        return XTransactionHelper.begin(context, () -> {
            if (CollUtil.isNotEmpty(insertList)) {
                BasePhotovoltaicPlantViewMapper viewMapper = context.getBean(BasePhotovoltaicPlantViewMapper.class);
                viewMapper.batchInsertList(insertList);
            }
            if (CollUtil.isNotEmpty(errorRowList)) {
                return XServiceResult.error(-1, "数据导入成功,第[" + CollUtil.join(errorRowList, ",") + "]行,电站已存在!");
            }
            return XServiceResult.OK;
        });
    }

    /**
     * 井口配置导入
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult }
     */
    @XText("井口配置导入")
    @XApiUpload
    public XServiceResult importWellhead(XContext context, ImportExcelInput input) {
        List<ExcelWellheadTemplate> list = EasyExcelUtil.getExcelDataAndCheck(input.getFile(), ExcelWellheadTemplate.class,
                "井号", ExcelWellheadTemplate::getWellNumber);
        if (CollUtil.isEmpty(list)) {
            return XServiceResult.OK;
        }
        //整理各种字典数据
        Map<String, String> ouMap = this.getPlantMap(context);
        Map<String, String> dictMap = ServiceUtil.getDictionaryList(context, BusinessConstant.RUN_TYPE).stream()
                .collect(Collectors.toMap(GetSysDictionaryViewOutput::getDicName, GetSysDictionaryViewOutput::getDicKey));
        //封装并校验井号(校验井号是否重复,重复不导入),并插入数据
        Set<String> collect = context.getBean(BaseWellheadMapper.class)
                .selectList(new LambdaQueryWrapper<BaseWellheadEnt>()
                        .eq(BaseModel::getIsDeleted, 1))
                .stream()
                .map(BaseWellheadEnt::getWellNumber)
                .collect(Collectors.toSet());
        List<Integer> errorRowList = new ArrayList<>(list.size());
        List<BaseWellheadView> insertList = new ArrayList<>(list.size());
        int rowNum = 2;
        PpsUserSession session = context.getSession(PpsUserSession.class);
        for (ExcelWellheadTemplate ent : list) {
            //检查名称是否重复
            if (collect.contains(ent.getWellNumber())) {
                errorRowList.add(rowNum);
            } else {
                //数据封装
                BaseWellheadView build = BaseWellheadView.builder()
                        .ouName(ent.getOuName())
                        .ouId(ouMap.get(ent.getOuName()))
                        .wellNumber(ent.getWellNumber())
                        .runTypeName(ent.getRunTypeName())
                        .runTypeKey(dictMap.get(ent.getRunTypeName()))
                        .serviceRating(new BigDecimal(ent.getServiceRating()))
                        .ratedPower(new BigDecimal(ent.getRatedPower()))
                        .build();
                BaseUtils.createBaseModelDefault(build, session);
                insertList.add(build);
            }
            rowNum++;
        }
        return XTransactionHelper.begin(context, () -> {
            if (CollUtil.isNotEmpty(insertList)) {
                BaseWellheadViewMapper viewMapper = context.getBean(BaseWellheadViewMapper.class);
                viewMapper.batchInsertList(insertList);
            }
            if (CollUtil.isNotEmpty(errorRowList)) {
                return XServiceResult.error(-1, "数据导入成功,第[" + CollUtil.join(errorRowList, ",") + "]行,井号已存在!");
            }
            return XServiceResult.OK;
        });
    }

    /**
     * 柴发设备配置导入
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult }
     */
    @XText("柴发设备配置导入")
    @XApiUpload
    public XServiceResult importDieselGenerator(XContext context, ImportExcelInput input) {
        List<ExcelDieselGeneratorTemplate> list = EasyExcelUtil.getExcelDataAndCheck(input.getFile(), ExcelDieselGeneratorTemplate.class,
                "设备名称", ExcelDieselGeneratorTemplate::getDeviceName);
        if (CollUtil.isEmpty(list)) {
            return XServiceResult.OK;
        }
        //整理各种字典数据
        Map<String, String> ouMap = this.getPlantMap(context);
        //封装并校验设备名称(校验设备名称是否重复,重复不导入),并插入数据
        Set<String> collect = context.getBean(BaseDieselGeneratorMapper.class)
                .selectList(new LambdaQueryWrapper<BaseDieselGeneratorEnt>()
                        .eq(BaseModel::getIsDeleted, 1))
                .stream()
                .map(BaseDieselGeneratorEnt::getDeviceName)
                .collect(Collectors.toSet());
        List<Integer> errorRowList = new ArrayList<>(list.size());
        List<BaseDieselGeneratorView> insertList = new ArrayList<>(list.size());
        int rowNum = 2;
        PpsUserSession session = context.getSession(PpsUserSession.class);
        for (ExcelDieselGeneratorTemplate ent : list) {
            //检查名称是否重复
            if (collect.contains(ent.getDeviceName())) {
                errorRowList.add(rowNum);
            } else {
                //数据封装
                BaseDieselGeneratorView build = BaseDieselGeneratorView.builder()
                        .ouName(ent.getOuName())
                        .ouId(ouMap.get(ent.getOuName()))
                        .deviceName(ent.getDeviceName())
                        .makerNumber(ent.getMakerNumber())
                        .unitTypeName(ent.getUnitTypeName())
                        .normalPower(new BigDecimal(ent.getNormalPower()))
                        .standbyPower(new BigDecimal(ent.getStandbyPower()))
                        .build();
                BaseUtils.createBaseModelDefault(build, session);
                insertList.add(build);
            }
            rowNum++;
        }
        return XTransactionHelper.begin(context, () -> {
            if (CollUtil.isNotEmpty(insertList)) {
                BaseDieselGeneratorViewMapper viewMapper = context.getBean(BaseDieselGeneratorViewMapper.class);
                viewMapper.batchInsertList(insertList);
            }
            if (CollUtil.isNotEmpty(errorRowList)) {
                return XServiceResult.error(-1, "数据导入成功,第[" + CollUtil.join(errorRowList, ",") + "]行,设备名称已存在!");
            }
            return XServiceResult.OK;
        });
    }

    /**
     * 储能设备配置导入
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult }
     */
    @XText("储能设备配置导入")
    @XApiUpload
    public XServiceResult importEnergyStorageDevice(XContext context, ImportExcelInput input) {
        List<ExcelEnergyStorageDeviceTemplate> list = EasyExcelUtil.getExcelDataAndCheck(input.getFile(), ExcelEnergyStorageDeviceTemplate.class,
                "设备名称", ExcelEnergyStorageDeviceTemplate::getDeviceName);
        if (CollUtil.isEmpty(list)) {
            return XServiceResult.OK;
        }
        //整理各种字典数据
        Map<String, String> ouMap = this.getPlantMap(context);
        //封装并校验设备名称(校验设备名称是否重复,重复不导入),并插入数据
        Set<String> collect = context.getBean(BaseEnergyStorageDeviceMapper.class)
                .selectList(new LambdaQueryWrapper<BaseEnergyStorageDeviceEnt>()
                        .eq(BaseModel::getIsDeleted, 1))
                .stream()
                .map(BaseEnergyStorageDeviceEnt::getDeviceName)
                .collect(Collectors.toSet());
        List<Integer> errorRowList = new ArrayList<>(list.size());
        List<BaseEnergyStorageDeviceView> insertList = new ArrayList<>(list.size());
        int rowNum = 2;
        PpsUserSession session = context.getSession(PpsUserSession.class);
        for (ExcelEnergyStorageDeviceTemplate ent : list) {
            //检查名称是否重复
            if (collect.contains(ent.getDeviceName())) {
                errorRowList.add(rowNum);
            } else {
                //数据封装
                BaseEnergyStorageDeviceView build = BaseEnergyStorageDeviceView.builder()
                        .ouName(ent.getOuName())
                        .ouId(ouMap.get(ent.getOuName()))
                        .deviceName(ent.getDeviceName())
                        .makerNumber(ent.getMakerNumber())
                        .storageModelName(ent.getStorageModelName())
                        .apparatusCapacity(new BigDecimal(ent.getApparatusCapacity()))
                        .ratedDischargePower(new BigDecimal(ent.getRatedDischargePower()))
                        .ratedChargingPower(new BigDecimal(ent.getRatedChargingPower()))
                        .ratedDischargeDepth(new BigDecimal(ent.getRatedDischargeDepth()))
                        .ratedDischargeEfficiency(new BigDecimal(ent.getRatedDischargeEfficiency()))
                        .build();
                BaseUtils.createBaseModelDefault(build, session);
                insertList.add(build);
            }
            rowNum++;
        }
        return XTransactionHelper.begin(context, () -> {
            if (CollUtil.isNotEmpty(insertList)) {
                BaseEnergyStorageDeviceViewMapper viewMapper = context.getBean(BaseEnergyStorageDeviceViewMapper.class);
                viewMapper.batchInsertList(insertList);
            }
            if (CollUtil.isNotEmpty(errorRowList)) {
                return XServiceResult.error(-1, "数据导入成功,第[" + CollUtil.join(errorRowList, ",") + "]行,设备名称已存在!");
            }
            return XServiceResult.OK;
        });
    }

    /**
     * 市电峰谷配置导入
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult }
     */
    @XText("市电峰谷配置导入")
    @XApiUpload
    public XServiceResult importPriceStrategy(XContext context, ImportPriceStrategyExcelInput input) {
        List<ExcelPriceStrategyTemplate> list = EasyExcelUtil.getExcelDataAndCheck(input.getFile(), ExcelPriceStrategyTemplate.class,
                null, null);
        if (CollUtil.isEmpty(list)) {
            return XServiceResult.OK;
        }
        //设置行号并转换开始/结束时间
        List<Integer> errorRowList = new ArrayList<>(list.size());
        int rowNum = 2;
        for (ExcelPriceStrategyTemplate excelPriceStrategyTemplate : list) {
            try {
                //转换开始时间和结束时间(开始时间不能为24:00)
                excelPriceStrategyTemplate.setStart(DateTime.of(excelPriceStrategyTemplate.getStartTime(), BusinessConstant.MINUTES_FORMAT));
                if (CharSequenceUtil.equals(excelPriceStrategyTemplate.getEndTime(), BusinessConstant.END_OF_DAY_TIME)) {
                    excelPriceStrategyTemplate.setEnd(BusinessConstant.DATE_FLAG);
                } else {
                    excelPriceStrategyTemplate.setEnd(DateTime.of(excelPriceStrategyTemplate.getEndTime(), BusinessConstant.MINUTES_FORMAT));
                }
            } catch (Exception e) {
                errorRowList.add(rowNum);
            }
            excelPriceStrategyTemplate.setRowNum(rowNum++);
        }
        if (CollUtil.isNotEmpty(errorRowList)) {
            throw new XServiceException("第[" + CollUtil.join(errorRowList, ",") + "]行,开始或结束时间输入错误!");
        }
        //按月份分组,按开井时间排序校验开关井时间
        list.sort(Comparator.comparing(ExcelPriceStrategyTemplate::getStartTime));
        StringBuilder stringBuilder = new StringBuilder();
        Map<String, List<ExcelPriceStrategyTemplate>> collect = list.stream()
                .collect(Collectors.groupingBy(ExcelPriceStrategyTemplate::getMonth));
        Set<String> monthSet = collect.keySet();
        for (String month : monthSet) {
            List<ExcelPriceStrategyTemplate> templates = collect.get(month);
            if (templates.size() > 1) {
                ExcelPriceStrategyTemplate start = templates.get(0);
                DateTime firstEnd = start.getEnd();
                if (start.getStart().compareTo(firstEnd) >= 0) {
                    stringBuilder.append("第[").append(start.getRowNum()).append("]行,开始时间大于等于结束时间;");
                }
                for (int i = 1; i < templates.size(); i++) {
                    ExcelPriceStrategyTemplate end = templates.get(i);
                    DateTime secondStart = end.getStart();
                    DateTime secondEnd = end.getEnd();
                    if (secondStart.compareTo(secondEnd) >= 0) {
                        stringBuilder.append("第[").append(start.getRowNum()).append("]行,开始时间大于等于结束时间;");
                    } else if (firstEnd.compareTo(secondStart) != 0) {
                        stringBuilder.append("第[").append(start.getRowNum()).append("]行,结束时间和第[").append(end.getRowNum()).append("]行开始时间不连续;");
                    }
                    start = end;
                    firstEnd = secondEnd;
                }
            }
        }
        if (stringBuilder.length() > 0) {
            throw new XServiceException(stringBuilder.toString());
        }
        //封装数据
        String strategyId = input.getStrategyId();
        PpsUserSession session = context.getSession(PpsUserSession.class);
        Map<String, String> dictMap = ServiceUtil.getDictionaryList(context, BusinessConstant.TIME_FRAME).stream()
                .collect(Collectors.toMap(GetSysDictionaryViewOutput::getDicName, GetSysDictionaryViewOutput::getDicKey));
        List<BasePriceStrategyMonthView> monthViewList = new ArrayList<>(monthSet.size());
        for (String month : monthSet) {
            BasePriceStrategyMonthView view = BasePriceStrategyMonthView.builder()
                    .strategyId(strategyId)
                    .strategyMonth(month)
                    .build();
            BaseUtils.createBaseModelDefault(view, session);
            monthViewList.add(view);
        }
        //检查是否固定电价
        BasePriceStrategyMapper strategyMapper = context.getBean(BasePriceStrategyMapper.class);
        BasePriceStrategyEnt strategyEnt = strategyMapper.selectById(strategyId);
        List<BasePriceStrategyDetailView> detailViewList = new ArrayList<>(list.size());
        for (ExcelPriceStrategyTemplate ent : list) {
            BigDecimal electrovalence;
            if (strategyEnt.getIsFixedPrice().equals(0)) {
                electrovalence = strategyEnt.getFixedElectricityPrice();
            } else {
                electrovalence = CharSequenceUtil.isBlank(ent.getElectrovalence()) ? BigDecimal.ZERO : new BigDecimal(ent.getElectrovalence());
            }
            BasePriceStrategyDetailView view = BasePriceStrategyDetailView.builder()
                    .strategyId(strategyId)
                    .strategyMonth(ent.getMonth())
                    .periodTypeKey(dictMap.get(ent.getPeriodTypeName()))
                    .startTime(ent.getStartTime())
                    .endTime(ent.getEndTime())
                    .electrovalence(electrovalence)
                    .build();
            BaseUtils.createBaseModelDefault(view, session);
            detailViewList.add(view);
        }
        //先按月份删除再新增
        return XTransactionHelper.begin(context, () -> {
            //删除
            BasePriceStrategyMonthMapper monthEntMapper = context.getBean(BasePriceStrategyMonthMapper.class);
            monthEntMapper.delete(new LambdaQueryWrapper<BasePriceStrategyMonthEnt>()
                    .eq(BasePriceStrategyMonthEnt::getStrategyId, strategyId)
                    .in(BasePriceStrategyMonthEnt::getStrategyMonth, monthSet)
            );
            BasePriceStrategyDetailMapper detailEntMapper = context.getBean(BasePriceStrategyDetailMapper.class);
            detailEntMapper.delete(new LambdaQueryWrapper<BasePriceStrategyDetailEnt>()
                    .eq(BasePriceStrategyDetailEnt::getStrategyId, strategyId)
                    .in(BasePriceStrategyDetailEnt::getStrategyMonth, monthSet)
            );
            //新增
            BasePriceStrategyMonthViewMapper monthMapper = context.getBean(BasePriceStrategyMonthViewMapper.class);
            BasePriceStrategyDetailViewMapper detailMapper = context.getBean(BasePriceStrategyDetailViewMapper.class);
            monthMapper.batchInsertList(monthViewList);
            detailMapper.batchInsertList(detailViewList);
            return XServiceResult.OK;
        });
    }

    /*------------------------------------ private ------------------------------------*/

    /**
     * 设置页面默认值
     *
     * @param input 输入
     */
    private void setPageDefault(XPageInput input) {
        input.setPage(1);
        input.setLimit(1000);
    }

    /**
     * 获取采油厂地图
     *
     * @param context 上下文
     * @return {@link Map }<{@link String }, {@link String }>
     */
    private Map<String, String> getPlantMap(XContext context) {
        return ServiceUtil.getPlantList(context).stream()
                .collect(Collectors.toMap(GetSysOrganizationViewOutput::getOuName, GetSysOrganizationViewOutput::getId));
    }

    /**
     * 获取省份和城市列表
     *
     * @param context 上下文
     * @return {@link List }<{@link QuerySysAreaOutput }>
     */
    private List<QuerySysAreaOutput> getProvinceAndCityList(XContext context) {
        //区分环境
        ConfigOilFieldCloudServiceImpl service = context.getBean(ConfigOilFieldCloudServiceImpl.class);
        XSingleResult<GetConfigOilFieldOutput> currentConfig = service.getCurrentConfig(context);
        currentConfig.throwIfFail();
        GetConfigOilFieldOutput result = currentConfig.getResult();
        String[] strings = CharSequenceUtil.splitToArray(result.getProvinceList(), ',');
        return ServiceUtil.getProvinceAndCityList(context, strings);
    }
}