package pps.core.base.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import pps.cloud.base.service.IThirdWindPowerGenerationCloudService;
import pps.core.base.entity.ThirdWindPowerGenerationUpdateView;
import pps.core.base.entity.ThirdWindPowerGenerationView;
import pps.core.base.mapper.ThirdWindPowerGenerationUpdateViewMapper;
import pps.core.base.mapper.ThirdWindPowerGenerationViewMapper;
import pps.core.base.utils.ServiceUtil;
import pps.core.common.utils.BaseUtils;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XServiceResult;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 第三方风力发电数据
 *
 * @author ZWT
 * @date 2024/09/03 16:34
 */
@XService
public class ThirdWindPowerGenerationCloudServiceImpl implements IThirdWindPowerGenerationCloudService {

    /**
     * 第三方风力发电数据定时任务
     * todo 模拟数据生成
     *
     * @param context 上下文
     * @return {@link XServiceResult }
     */
    @Override
    public XServiceResult thirdWindPowerGenerationJob(XContext context) {
        DateTime endTime = DateUtil.date();
        DateTime startTime = DateUtil.offsetMinute(endTime, -30);
        ThirdWindPowerGenerationViewMapper viewMapper = context.getBean(ThirdWindPowerGenerationViewMapper.class);
        List<ThirdWindPowerGenerationView> list = viewMapper.generateSimulatedData(ThirdWindPowerGenerationView.builder()
                .startTime(startTime)
                .endTime(endTime)
                .build());
        if (CollUtil.isNotEmpty(list)) {
            List<ThirdWindPowerGenerationView> batchList = new ArrayList<>(list.size() * 2);
            List<ThirdWindPowerGenerationUpdateView> batchUpdateList = new ArrayList<>(list.size());
            //查当前环境
            String systemSource = ServiceUtil.getOilFieldCode(context);
            ThirdWindPowerGenerationView copy;
            for (ThirdWindPowerGenerationView view : list) {
                view.setSystemSource(systemSource);
                view.setInputTime(endTime);
                copy = this.newThirdWindPowerGenerationView(view);
                batchList.add(copy);
                batchList.add(view);
                //每日最新数据
                batchUpdateList.add(XCopyUtils.copyNewObject(view, ThirdWindPowerGenerationUpdateView.class));
            }
            //批量新增
            if (batchList.size() > BaseUtils.BATCH_SIZE) {
                List<List<ThirdWindPowerGenerationView>> subList = BaseUtils.getSubList(batchList);
                subList.forEach(viewMapper::batchInsert);
            } else {
                viewMapper.batchInsert(batchList);
            }
            //生成每日最新数据
            ThirdWindPowerGenerationUpdateViewMapper mapper = context.getBean(ThirdWindPowerGenerationUpdateViewMapper.class);
            mapper.deleteBatch(list.stream()
                            .map(ThirdWindPowerGenerationView::getStationId)
                            .collect(Collectors.toList()),
                    DateUtil.beginOfDay(endTime), DateUtil.endOfDay(endTime));
            //批量新增
            if (batchUpdateList.size() > BaseUtils.BATCH_SIZE) {
                List<List<ThirdWindPowerGenerationUpdateView>> subList = BaseUtils.getSubList(batchUpdateList);
                subList.forEach(mapper::batchInsert);
            } else {
                mapper.batchInsert(batchUpdateList);
            }
        }
        return XServiceResult.OK;
    }

    /*--------------------------------- private ---------------------------------*/

    /**
     * 第三代风力发电新观点
     *
     * @param view 看法
     * @return {@link ThirdWindPowerGenerationView }
     */
    private ThirdWindPowerGenerationView newThirdWindPowerGenerationView(ThirdWindPowerGenerationView view) {
        return ThirdWindPowerGenerationView.builder()
                .stationName(view.getStationName())
                .stationId(view.getStationId())
                .collectTime(DateUtil.offsetMinute(view.getCollectTime(), -15))
                .actualWindSpeed(view.getActualWindSpeed())
                .actualPower(view.getActualPower())
                .actualGeneration(view.getActualGeneration())
                .actualWindDirection(view.getActualWindDirection())
                .inputTime(view.getInputTime())
                .systemSource(view.getSystemSource())
                .build();
    }
}