package pps.core.base.service.data.base_excel;

import com.alibaba.excel.annotation.ExcelProperty;
import com.alibaba.excel.annotation.write.style.ColumnWidth;
import jakarta.validation.constraints.Min;
import jakarta.validation.constraints.NotBlank;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import pps.core.base.excel.OilExtractionPlantImpl;
import pps.core.common.excel.config.ExcelSelected;

import java.io.Serializable;

/**
 * excel柴发设备模板
 *
 * @author ZWT
 * @date 2024/05/09
 */
@Data
@NoArgsConstructor
@AllArgsConstructor
@Builder
@ColumnWidth(value = 25)
public class ExcelDieselGeneratorTemplate implements Serializable {

    private static final long serialVersionUID = -6773651457799858074L;

    /**
     * 采油厂名称
     */
    @NotBlank(message = "缺少采油厂")
    @ExcelSelected(sourceClass = OilExtractionPlantImpl.class)
    @ExcelProperty(index = 0, value = "采油厂名称")
    private String ouName;

    /**
     * 设备名称
     */
    @NotBlank(message = "缺少设备名称")
    @ExcelProperty(index = 1, value = "设备名称")
    private String deviceName;

    /**
     * 出厂编号
     */
    @NotBlank(message = "缺少出厂编号")
    @ExcelProperty(index = 2, value = "出厂编号")
    private String makerNumber;

    /**
     * 机组型号
     */
    @NotBlank(message = "缺少机组型号")
    @ExcelProperty(index = 3, value = "机组型号")
    private String unitTypeName;

    /**
     * 常用功率(KW)
     */
    @NotBlank(message = "缺少常用功率")
    @Min(value = 0, message = "常用功率需大于0")
    @ExcelProperty(index = 4, value = "常用功率(KW)")
    private String normalPower;

    /**
     * 备用功率(KW)
     */
    @NotBlank(message = "缺少备用功率")
    @Min(value = 0, message = "备用功率需大于0")
    @ExcelProperty(index = 5, value = "备用功率(KW)")
    private String standbyPower;
}