package pps.core.base.service.data.base_excel;

import com.alibaba.excel.annotation.ExcelProperty;
import com.alibaba.excel.annotation.write.style.ColumnWidth;
import jakarta.validation.constraints.Min;
import jakarta.validation.constraints.NotBlank;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import pps.core.base.excel.OilExtractionPlantImpl;
import pps.core.common.excel.config.ExcelSelected;

import java.io.Serializable;

/**
 * excel储能设备模板
 *
 * @author ZWT
 * @date 2024/05/09
 */
@Data
@NoArgsConstructor
@AllArgsConstructor
@Builder
@ColumnWidth(value = 25)
public class ExcelEnergyStorageDeviceTemplate implements Serializable {

    private static final long serialVersionUID = -6239358694782042550L;

    /**
     * 采油厂名称
     */
    @NotBlank(message = "缺少采油厂")
    @ExcelSelected(sourceClass = OilExtractionPlantImpl.class)
    @ExcelProperty(index = 0, value = "采油厂名称")
    private String ouName;

    /**
     * 设备名称
     */
    @NotBlank(message = "缺少设备名称")
    @ExcelProperty(index = 1, value = "设备名称")
    private String deviceName;

    /**
     * 出厂编号
     */
    @NotBlank(message = "缺少出厂编号")
    @ExcelProperty(index = 2, value = "出厂编号")
    private String makerNumber;

    /**
     * 规格型号
     */
    @NotBlank(message = "缺少规格型号")
    @ExcelProperty(index = 3, value = "规格型号")
    private String storageModelName;

    /**
     * 设备容量(KWh)
     */
    @NotBlank(message = "缺少设备容量")
    @Min(value = 0, message = "设备容量需大于0")
    @ExcelProperty(index = 4, value = "设备容量(KWh)")
    private String apparatusCapacity;

    /**
     * 额定放电功率(KW)
     */
    @NotBlank(message = "缺少额定放电功率")
    @Min(value = 0, message = "额定放电功率需大于0")
    @ExcelProperty(index = 5, value = "额定放电功率(KW)")
    private String ratedDischargePower;

    /**
     * 额定充电功率(KW)
     */
    @NotBlank(message = "缺少额定充电功率")
    @Min(value = 0, message = "额定充电功率需大于0")
    @ExcelProperty(index = 6, value = "额定充电功率(KW)")
    private String ratedChargingPower;

    /**
     * 额定放电深度(%)
     */
    @NotBlank(message = "缺少额定放电深度")
    @Min(value = 0, message = "额定放电深度需大于0")
    @ExcelProperty(index = 7, value = "额定放电深度(%)")
    private String ratedDischargeDepth;

    /**
     * 额定放电效率(%)
     */
    @NotBlank(message = "缺少额定放电效率")
    @Min(value = 0, message = "额定放电效率需大于0")
    @ExcelProperty(index = 8, value = "额定放电效率(%)")
    private String ratedDischargeEfficiency;
}