package pps.core.base.service.data.base_excel;

import com.alibaba.excel.annotation.ExcelProperty;
import com.alibaba.excel.annotation.write.style.ColumnWidth;
import jakarta.validation.constraints.Min;
import jakarta.validation.constraints.NotBlank;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import pps.core.base.excel.OilExtractionPlantImpl;
import pps.core.common.excel.config.ExcelSelected;

import java.io.Serializable;

/**
 * excel光伏电站模板
 *
 * @author ZWT
 * @date 2024/05/07
 */
@Data
@NoArgsConstructor
@AllArgsConstructor
@Builder
@ColumnWidth(value = 25)
public class ExcelPhotovoltaicPlantTemplate implements Serializable {

    private static final long serialVersionUID = -9069694443646671657L;

    /**
     * 所在省
     */
    @NotBlank(message = "缺少省份")
    @ExcelProperty(index = 0, value = "所在省")
    private String province;

    /**
     * 所在市
     */
    @NotBlank(message = "缺少市")
    @ExcelProperty(index = 1, value = "所在市")
    private String city;

    /**
     * 采油厂名称
     */
    @NotBlank(message = "缺少采油厂名称")
    @ExcelSelected(sourceClass = OilExtractionPlantImpl.class)
    @ExcelProperty(index = 2, value = "采油厂名称")
    private String ouName;

    /**
     * 电站名称
     */
    @NotBlank(message = "缺少电站名称")
    @ExcelProperty(index = 3, value = "电站名称")
    private String stationName;

    /**
     * 逆变器出厂编号
     */
    @ExcelProperty(index = 4, value = "逆变器出厂编号")
    private String makerNumber;

    /**
     * 逆变器规格型号
     */
    @ExcelProperty(index = 5, value = "逆变器规格型号")
    private String photovoltaicModelName;

    /**
     * 安装倾角(°)
     */
    @NotBlank(message = "缺少安装倾角")
    @Min(value = 0, message = "安装倾角需大于0")
    @ExcelProperty(index = 6, value = "安装倾角(°)")
    private String mountingAngle;

    /**
     * 装机总量(KWP)
     */
    @NotBlank(message = "缺少装机总量")
    @Min(value = 0, message = "装机总量需大于0")
    @ExcelProperty(index = 7, value = "装机总量(KWP)")
    private String totalPower;

    /**
     * 阵列朝向
     */
    @NotBlank(message = "缺少阵列朝向")
    @Min(value = 0, message = "阵列朝向需大于0")
    @ExcelProperty(index = 8, value = "阵列朝向")
    private String arrayOrientation;

    /**
     * 经度(°)
     */
    @NotBlank(message = "缺少经度")
    @Min(value = 0, message = "经度需大于0")
    @ExcelProperty(index = 9, value = "经度(°)")
    private String longitude;

    /**
     * 纬度(°)
     */
    @NotBlank(message = "缺少纬度")
    @Min(value = 0, message = "纬度需大于0")
    @ExcelProperty(index = 10, value = "纬度(°)")
    private String latitude;

    /**
     * 高程(m)
     */
    @NotBlank(message = "缺少高程")
    @Min(value = 0, message = "高程需大于0")
    @ExcelProperty(index = 11, value = "高程(m)")
    private String elevation;
}