package pps.core.base.service.data.base_excel;

import cn.hutool.core.date.DateTime;
import com.alibaba.excel.annotation.ExcelIgnoreUnannotated;
import com.alibaba.excel.annotation.ExcelProperty;
import com.alibaba.excel.annotation.write.style.ColumnWidth;
import jakarta.validation.constraints.Min;
import jakarta.validation.constraints.NotBlank;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import pps.core.base.excel.TimeFrameImpl;
import pps.core.base.excel.TimeRangeImpl;
import pps.core.common.excel.config.ExcelSelected;

import java.io.Serializable;

/**
 * excel市电峰谷模板
 *
 * @author ZWT
 * @date 2024/05/13 13:51
 */
@Data
@NoArgsConstructor
@AllArgsConstructor
@Builder
@ExcelIgnoreUnannotated
@ColumnWidth(value = 25)
public class ExcelPriceStrategyTemplate implements Serializable {

    private static final long serialVersionUID = -2929661573381767982L;

    /**
     * 时段类型
     */
    @Min(value = 1, message = "月份需大于0")
    @NotBlank(message = "缺少月份")
    @ExcelSelected(source = {"1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12"})
    @ExcelProperty(index = 0, value = "月份")
    private String month;

    /**
     * 时段类型
     */
    @NotBlank(message = "缺少时段类型")
    @ExcelSelected(sourceClass = TimeFrameImpl.class)
    @ExcelProperty(index = 1, value = "时段类型")
    private String periodTypeName;

    /**
     * 开始时间
     */
    @NotBlank(message = "缺少开始时间")
    @ExcelSelected(sourceClass = TimeRangeImpl.class)
    @ExcelProperty(index = 2, value = "开始时间")
    private String startTime;

    /**
     * 结束时间
     */
    @NotBlank(message = "缺少结束时间")
    @ExcelSelected(sourceClass = TimeRangeImpl.class)
    @ExcelProperty(index = 3, value = "结束时间")
    private String endTime;

    /**
     * 电价
     */
    @Min(value = 0, message = "电价需大于0")
    @ExcelProperty(index = 4, value = "电价")
    private String electrovalence;

    /*------------------------------------ other ------------------------------------*/

    /**
     * 行号
     */
    private Integer rowNum;

    /**
     * 开始
     */
    private DateTime start;

    /**
     * 结束
     */
    private DateTime end;
}
