package pps.core.base.service.data.base_excel;

import com.alibaba.excel.annotation.ExcelProperty;
import com.alibaba.excel.annotation.write.style.ColumnWidth;
import jakarta.validation.constraints.Min;
import jakarta.validation.constraints.NotBlank;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import pps.core.base.excel.OilExtractionPlantImpl;
import pps.core.base.excel.RunTypeImpl;
import pps.core.common.excel.config.ExcelSelected;

import java.io.Serializable;

/**
 * excel井口模板
 *
 * @author ZWT
 * @date 2024/05/09
 */
@Data
@NoArgsConstructor
@AllArgsConstructor
@Builder
@ColumnWidth(value = 25)
public class ExcelWellheadTemplate implements Serializable {

    private static final long serialVersionUID = -6773651457799858074L;

    /**
     * 采油厂名称
     */
    @NotBlank(message = "缺少采油厂名称")
    @ExcelSelected(sourceClass = OilExtractionPlantImpl.class)
    @ExcelProperty(index = 0, value = "采油厂名称")
    private String ouName;

    /**
     * 井号
     */
    @NotBlank(message = "缺少井号")
    @ExcelProperty(index = 1, value = "井号")
    private String wellNumber;

    /**
     * 运行类型
     */
    @NotBlank(message = "缺少运行类型")
    @ExcelSelected(sourceClass = RunTypeImpl.class)
    @ExcelProperty(index = 2, value = "运行类型")
    private String runTypeName;

    /**
     * 运行功率(KW)
     */
    @NotBlank(message = "缺少运行功率")
    @Min(value = 0, message = "运行功率需大于0")
    @ExcelProperty(index = 3, value = "运行功率(KW)")
    private String serviceRating;

    /**
     * 额定功率(KW)
     */
    @NotBlank(message = "缺少额定功率")
    @Min(value = 0, message = "额定功率需大于0")
    @ExcelProperty(index = 4, value = "额定功率(KW)")
    private String ratedPower;
}