package pps.core.task.job;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import pps.cloud.base.service.IBaseWeatherCloudService;
import pps.core.common.cache.TaskLockCache;
import pps.core.task.constant.TaskConstant;
import pps.core.task.utils.ServiceUtil;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.data.XServiceResult;

/**
 * 天气数据邮件接收定时任务
 *
 * @author ZWT
 * @date 2023/11/13 17:15
 */
@XText("天气数据邮件接收定时任务")
public class WeatherReceiveJob {

    /**
     * 半小时隔5分钟执行一次
     *
     * @param xContext     x上下文
     * @param oilFieldCode 油田代码
     * @return {@link XServiceResult }
     */
    public XServiceResult execute(XContext xContext, String oilFieldCode) {
        xContext.getLogger().info("------ WeatherReceiveJob start:{}", DateUtil.date());
        String key = TaskConstant.TASK_LOCK_KEY + "WeatherReceiveJob";
        TaskLockCache exist = TaskLockCache.exist(xContext, key);
        if (ObjectUtil.isNull(exist)) {
            ServiceUtil.setCache(xContext, key);
            try {
                IBaseWeatherCloudService service = xContext.getBean(IBaseWeatherCloudService.class);
                XServiceResult result = service.weatherDataProcess(xContext, oilFieldCode);
                result.throwIfFail();
            } catch (Exception e) {
                xContext.getLogger().error("------ WeatherReceiveJob Exception: ", e);
            } finally {
                xContext.getLogger().info("------ WeatherReceiveJob end:{}", DateUtil.date());
                TaskLockCache.delete(xContext, key);
            }
        }
        return XServiceResult.OK;
    }
}