package pps.core.task.module;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.ArrayUtil;
import cn.hutool.core.util.ClassUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.cron.CronUtil;
import cn.hutool.cron.task.Task;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import pps.cloud.base.service.data.config_oil_field.GetConfigOilFieldOutput;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.utils.ManifestComponentInfoHelper;
import pps.core.task.entity.ConfigTimeTaskEnt;
import pps.core.task.mapper.ConfigTimeTaskMapper;
import pps.core.task.utils.ServiceUtil;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.XModule;

import java.util.List;

/**
 * 定时任务模块
 *
 * @author ZWT
 * @date 2024/07/04
 */
@XText("任务模块")
public class PpsCoreTaskModule extends XModule {

    protected PpsCoreTaskModule() {
        super(1010);
    }

    @Override
    protected void onStarted(XContext context) {
        ManifestComponentInfoHelper.init(this.getClass().getClassLoader());
        //获取当前配置
        GetConfigOilFieldOutput currentConfig = ServiceUtil.getCurrentConfig(context);
        if (ObjectUtil.isNull(currentConfig) || CharSequenceUtil.isBlank(currentConfig.getOilFieldCode())) {
            return;
        }
        //查定时任务配置
        ConfigTimeTaskMapper taskMapper = context.getBean(ConfigTimeTaskMapper.class);
        List<ConfigTimeTaskEnt> configList = taskMapper.selectList(new LambdaQueryWrapper<ConfigTimeTaskEnt>()
                .eq(ConfigTimeTaskEnt::getIsEnable, BusinessConstant.ZERO)
                .eq(ConfigTimeTaskEnt::getSystemSource, currentConfig.getOilFieldCode())
        );
        if (CollUtil.isEmpty(configList)) {
            return;
        }
        // 启动 Cron 线程
        CronUtil.setMatchSecond(true);
        CronUtil.start(true);
        //控制启动部分定时任务
        for (ConfigTimeTaskEnt config : configList) {
            CronUtil.schedule(config.getSchedulingPattern(), (Task) () -> {
                if (CharSequenceUtil.isNotBlank(config.getOtherParam())) {
                    Object[] params = {context};
                    params = ArrayUtil.append(params, CharSequenceUtil.splitToArray(config.getOtherParam(), ','));
                    ClassUtil.invoke(config.getClassPath(), config.getMethodName(), false, params);
                } else {
                    ClassUtil.invoke(config.getClassPath(), config.getMethodName(), false, context);
                }
            });
        }
    }
}