package pps.core.space.mapper;

import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;
import pps.core.space.entity.SpaceOptimizeDurationDTO;
import pps.core.space.entity.SpaceOptimizePeriodDTO;
import pps.core.space.entity.SpaceOptimizeWellheadDTO;

import java.util.List;

/**
 * 间开优化
 *
 * @author ZWT
 * @date 2023/09/19 09:40
 */
@Repository(value = "pps.core.space.mapper.SpaceOptimizeViewMapper")
public interface SpaceOptimizeViewMapper {

    /**
     * 中期间开优化批量新增
     *
     * @param list 列表
     * @return int
     */
    int longPeriodBatchInsertList(@Param(value = "list") List<SpaceOptimizePeriodDTO> list);

    /**
     * 中期间开优化井口批量新增
     *
     * @param list 列表
     * @return int
     */
    int longWellheadBatchInsertList(@Param(value = "list") List<SpaceOptimizeWellheadDTO> list);

    /**
     * 中期间开优化井口配置批量新增
     *
     * @param list 列表
     * @return int
     */
    int longDurationBatchInsertList(@Param(value = "list") List<SpaceOptimizeDurationDTO> list);

    /**
     * 短期间开开优化批量新增
     *
     * @param list 列表
     * @return int
     */
    int midPeriodBatchInsertList(@Param(value = "list") List<SpaceOptimizePeriodDTO> list);

    /**
     * 短期间开开优化井口批量新增
     *
     * @param list 列表
     * @return int
     */
    int midWellheadBatchInsertList(@Param(value = "list") List<SpaceOptimizeWellheadDTO> list);

    /**
     * 短期间开开优化井口配置批量新增
     *
     * @param list 列表
     * @return int
     */
    int midDurationBatchInsertList(@Param(value = "list") List<SpaceOptimizeDurationDTO> list);

    /**
     * 极短期间开优化批量新增
     *
     * @param list 列表
     * @return int
     */
    int shortPeriodBatchInsertList(@Param(value = "list") List<SpaceOptimizePeriodDTO> list);

    /**
     * 极短期间开优化井口批量新增
     *
     * @param list 列表
     * @return int
     */
    int shortWellheadBatchInsertList(@Param(value = "list") List<SpaceOptimizeWellheadDTO> list);

    /**
     * 极短期间开优化井口配置批量新增
     *
     * @param list 列表
     * @return int
     */
    int shortDurationBatchInsertList(@Param(value = "list") List<SpaceOptimizeDurationDTO> list);

    /**
     * 长期间开优化批量新增
     *
     * @param list 列表
     * @return int
     */
    int ultraPeriodBatchInsertList(@Param(value = "list") List<SpaceOptimizePeriodDTO> list);

    /**
     * 长期间开优化井口批量新增
     *
     * @param list 列表
     * @return int
     */
    int ultraWellheadBatchInsertList(@Param(value = "list") List<SpaceOptimizeWellheadDTO> list);

    /**
     * 长期间开优化井口配置批量新增
     *
     * @param list 列表
     * @return int
     */
    int ultraDurationBatchInsertList(@Param(value = "list") List<SpaceOptimizeDurationDTO> list);
}
