package pps.core.space.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import pps.cloud.space.service.ISpaceInstitutionDetailCloudService;
import pps.cloud.space.service.data.space_institution_wellhead.GetSpaceInstitutionWellheadInput;
import pps.cloud.space.service.data.space_institution_wellhead.GetSpaceInstitutionWellheadOutput;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import pps.core.space.entity.SpaceInstitutionDetailEnt;
import pps.core.space.entity.SpaceInstitutionWellheadEnt;
import pps.core.space.entity.SpaceInstitutionWellheadView;
import pps.core.space.mapper.SpaceInstitutionDetailMapper;
import pps.core.space.mapper.SpaceInstitutionWellheadMapper;
import pps.core.space.mapper.SpaceInstitutionWellheadViewMapper;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.data.XSingleResult;

import java.util.List;

/**
 * 基础间开配置Cloud模块
 *
 * @author ZWT
 * @date 2023/09/06 17:09
 */
@XService
public class SpaceInstitutionDetailCloudServiceImpl implements ISpaceInstitutionDetailCloudService {

    /**
     * 基础间开配置Cloud模块--获取线路配置制度数量
     *
     * @param context 上下文
     * @param lineId  线路id
     * @return {@link XSingleResult}<{@link Long}>
     */
    @Override
    public XSingleResult<Long> getInstitutionLineCountByLineId(XContext context, String lineId) {
        SpaceInstitutionDetailMapper mapper = context.getBean(SpaceInstitutionDetailMapper.class);
        return XSingleResult.success(mapper.selectCount(
                new LambdaQueryWrapper<SpaceInstitutionDetailEnt>()
                        .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                        .eq(SpaceInstitutionDetailEnt::getLineId, lineId)
        ));
    }

    /**
     * 基础间开配置Cloud模块--检查井口是否被引用
     *
     * @param context    上下文
     * @param wellheadId 井口id
     * @return {@link XSingleResult}<{@link Long}>
     */
    @Override
    public XSingleResult<Long> getInstitutionWellheadCountByWellheadId(XContext context, String wellheadId) {
        SpaceInstitutionWellheadMapper mapper = context.getBean(SpaceInstitutionWellheadMapper.class);
        return XSingleResult.success(mapper.selectCount(
                new LambdaQueryWrapper<SpaceInstitutionWellheadEnt>()
                        .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                        .eq(SpaceInstitutionWellheadEnt::getWellheadId, wellheadId)
        ));
    }

    /**
     * 获取间开井口列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetSpaceInstitutionWellheadOutput }>
     */
    @Override
    public XListResult<GetSpaceInstitutionWellheadOutput> getInstitutionWellheadList(XContext context, GetSpaceInstitutionWellheadInput input) {
        SpaceInstitutionWellheadViewMapper mapper = context.getBean(SpaceInstitutionWellheadViewMapper.class);
        List<SpaceInstitutionWellheadView> wellheadViewList = mapper.selectInstitutionWellheadList();
        return XListResult.success(XCopyUtils.copyNewList(wellheadViewList, GetSpaceInstitutionWellheadOutput.class));
    }
}
