package pps.core.space.service;

import cn.hutool.core.collection.CollUtil;
import com.github.pagehelper.PageInfo;
import com.github.pagehelper.page.PageMethod;
import pps.core.common.constant.BusinessConstant;
import pps.core.space.entity.SpaceOptimizeLongPeriodView;
import pps.core.space.mapper.SpaceOptimizeLongPeriodViewMapper;
import pps.core.space.service.data.space_optimize_long_period.QuerySpaceOptimizeLongPeriodViewInput;
import pps.core.space.service.data.space_optimize_long_period.QuerySpaceOptimizeLongPeriodViewOutput;
import pps.core.space.service.data.space_optimize_period.GetSpaceOptimizePeriodInput;
import pps.core.space.service.data.space_optimize_period.GetSpaceOptimizePeriodOutput;
import pps.core.space.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XSingleResult;
import xstartup.feature.api.annotation.XApiGet;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 中期间开优化模块
 *
 * @author ZWT
 * @date 2023/09/07
 */
@XService
@XText("中期间开优化模块")
public class SpaceOptimizeLongPeriodService extends SpaceOptimizeBaseService {

    /**
     * 中期间开优化--分页列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XPageResult}<{@link QuerySpaceOptimizeLongPeriodViewOutput}>
     */
    @XApiGet
    @XText("中期间开优化--分页列表")
    public XPageResult<QuerySpaceOptimizeLongPeriodViewOutput> querySpaceOptimizeLongPeriodView(XContext context, QuerySpaceOptimizeLongPeriodViewInput input) {
        SpaceOptimizeLongPeriodViewMapper mapper = context.getBean(SpaceOptimizeLongPeriodViewMapper.class);
        SpaceOptimizeLongPeriodView record = XCopyUtils.copyNewObject(input, SpaceOptimizeLongPeriodView.class);
        PageMethod.startPage(input.getPage(), input.getLimit());
        List<SpaceOptimizeLongPeriodView> list = mapper.selectList(record);
        PageInfo<SpaceOptimizeLongPeriodView> pageInfo = new PageInfo<>(list);
        List<QuerySpaceOptimizeLongPeriodViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySpaceOptimizeLongPeriodViewOutput.class);
        if (CollUtil.isNotEmpty(outputs)) {
            Map<String, String> gridTypeMap = ServiceUtil.getDictMap(context, BusinessConstant.GRID_TYPE);
            Map<String, String> ouMap = ServiceUtil.getOuMapByOuIds(context, outputs.stream()
                    .map(QuerySpaceOptimizeLongPeriodViewOutput::getOuId)
                    .distinct()
                    .collect(Collectors.toList())
            );
            String lineName = ServiceUtil.getPowerLineNameById(context, input.getLineId());
            outputs.forEach(output -> {
                output.setLineName(lineName);
                output.setOuName(ouMap.get(output.getOuId()));
                output.setGridTypeName(gridTypeMap.get(output.getGridTypeKey()));
                output.setOptimizeStateValue(output.getOptimizeState().equals(0) ? BusinessConstant.OPTIMIZED_C : BusinessConstant.NO_OPTIMIZATION_C);
            });
        }
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    /**
     * 中期间开优化--详情
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetSpaceOptimizePeriodOutput}>
     */
    @XApiGet
    @XText("中期间开优化--详情")
    public XSingleResult<GetSpaceOptimizePeriodOutput> getSpaceOptimizeLongPeriodView(XContext context, GetSpaceOptimizePeriodInput input) {
        return XSingleResult.success(super.getSpaceOptimizePeriodView(context, input, BusinessConstant.TWO, BusinessConstant.ZERO));
    }
}