package pps.core.space.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import pps.cloud.space.service.ISpaceOptimizeShortCloudService;
import pps.cloud.space.service.data.space_optimize_short_wellhead.GetMonthlyWellCountInput;
import pps.cloud.space.service.data.space_optimize_short_wellhead.GetMonthlyWellCountOutput;
import pps.cloud.space.service.data.space_optimize_short_wellhead.GetSpaceOptimizeShortWellheadInput;
import pps.cloud.space.service.data.space_optimize_short_wellhead.GetSpaceOptimizeShortWellheadOutput;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import pps.core.common.utils.BaseUtils;
import pps.core.space.entity.*;
import pps.core.space.mapper.SpaceOptimizeShortDurationMapper;
import pps.core.space.mapper.SpaceOptimizeShortWellheadMapper;
import pps.core.space.mapper.SpaceOptimizeViewMapper;
import pps.core.space.service.data.SpaceOptimizeDateDuration;
import pps.core.space.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.helper.XTransactionHelper;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 极短期间开优化Cloud模块
 *
 * @author ZWT
 * @date 2023/09/20 11:44
 */
@XService
public class SpaceOptimizeShortCloudServiceImpl extends SpaceOptimizeBaseService implements ISpaceOptimizeShortCloudService {

    /**
     * 极短期间开优化Cloud模块--定时任务
     *
     * @param context 上下文
     * @return {@link XServiceResult}
     */
    @Override
    public XServiceResult optimizeShortJob(XContext context) {
        List<SpaceOptimizePeriodDTO> periodDTOList = new ArrayList<>(32);
        List<SpaceOptimizeWellheadDTO> wellheadDTOList = new ArrayList<>(64);
        List<SpaceOptimizeDurationDTO> durationDTOList = new ArrayList<>(128);
        List<SpaceOptimizeDurationDTO> unOptimizeDurationList = new ArrayList<>(128);
        super.dynamicOptimize(context, periodDTOList, wellheadDTOList, durationDTOList, unOptimizeDurationList, BusinessConstant.ONE, BusinessConstant.THREE, 1);
        //开启事务
        return XTransactionHelper.begin(context, () -> {
            int size;
            SpaceOptimizeViewMapper optimizeViewMapper = context.getBean(SpaceOptimizeViewMapper.class);
            if (CollUtil.isNotEmpty(periodDTOList)) {
                size = periodDTOList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<SpaceOptimizePeriodDTO>> subList = BaseUtils.getSubList(periodDTOList);
                    subList.forEach(optimizeViewMapper::shortPeriodBatchInsertList);
                } else {
                    optimizeViewMapper.shortPeriodBatchInsertList(periodDTOList);
                }
            }
            if (CollUtil.isNotEmpty(wellheadDTOList)) {
                size = wellheadDTOList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<SpaceOptimizeWellheadDTO>> subList = BaseUtils.getSubList(wellheadDTOList);
                    subList.forEach(optimizeViewMapper::shortWellheadBatchInsertList);
                } else {
                    optimizeViewMapper.shortWellheadBatchInsertList(wellheadDTOList);
                }
            }
            if (CollUtil.isNotEmpty(unOptimizeDurationList)) {
                size = unOptimizeDurationList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<SpaceOptimizeDurationDTO>> subList = BaseUtils.getSubList(unOptimizeDurationList);
                    subList.forEach(optimizeViewMapper::shortDurationBatchInsertList);
                } else {
                    optimizeViewMapper.shortDurationBatchInsertList(unOptimizeDurationList);
                }
            }
            if (CollUtil.isNotEmpty(durationDTOList)) {
                size = durationDTOList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<SpaceOptimizeDurationDTO>> subList = BaseUtils.getSubList(durationDTOList);
                    subList.forEach(optimizeViewMapper::shortDurationBatchInsertList);
                } else {
                    optimizeViewMapper.shortDurationBatchInsertList(durationDTOList);
                }
            }
            return XServiceResult.OK;
        });
    }

    /**
     * 月度井口统计
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult }<{@link GetMonthlyWellCountOutput }>
     */
    @Override
    public XSingleResult<GetMonthlyWellCountOutput> monthlyWellCount(XContext context, GetMonthlyWellCountInput input) {
        SpaceOptimizeShortWellheadMapper mapper = context.getBean(SpaceOptimizeShortWellheadMapper.class);
        Long count = mapper.selectCount(new QueryWrapper<SpaceOptimizeShortWellheadEnt>()
                .select("DISTINCT well_number")
                .apply("MONTH ( optimize_date ) = {0}", input.getMonth())
        );
        return XSingleResult.success(GetMonthlyWellCountOutput.builder()
                .wellNumber(count)
                .build());
    }

    /**
     * 每日优化结果
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetSpaceOptimizeShortWellheadOutput }>
     */
    @Override
    public XListResult<GetSpaceOptimizeShortWellheadOutput> dailyOptimizationResult(XContext context, GetSpaceOptimizeShortWellheadInput input) {
        DateTime today = DateUtil.beginOfDay(DateUtil.date());
        SpaceOptimizeShortWellheadMapper wellheadMapper = context.getBean(SpaceOptimizeShortWellheadMapper.class);
        List<SpaceOptimizeShortWellheadEnt> wellheadList = wellheadMapper.selectList(new LambdaQueryWrapper<SpaceOptimizeShortWellheadEnt>()
                .in(SpaceOptimizeShortWellheadEnt::getWellheadId, input.getWellheadIds())
                .eq(SpaceOptimizeShortWellheadEnt::getOptimizeDate, today)
        );
        if (CollUtil.isEmpty(wellheadList)) {
            return XListResult.success(Collections.emptyList());
        }
        List<pps.core.space.service.data.space_optimize_short_wellhead.GetSpaceOptimizeShortWellheadOutput> wellheadOutputs = XCopyUtils.copyNewList(wellheadList, pps.core.space.service.data.space_optimize_short_wellhead.GetSpaceOptimizeShortWellheadOutput.class);
        SpaceOptimizeShortDurationMapper durationMapper = context.getBean(SpaceOptimizeShortDurationMapper.class);
        List<SpaceOptimizeShortDurationEnt> durationList = durationMapper.selectList(new LambdaQueryWrapper<SpaceOptimizeShortDurationEnt>()
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                .eq(SpaceOptimizeShortDurationEnt::getIsOptimize, BusinessConstant.ZERO)
                .in(SpaceOptimizeShortDurationEnt::getWellheadId, input.getWellheadIds())
                .eq(SpaceOptimizeShortDurationEnt::getOptimizeDate, today)
                .orderByAsc(SpaceOptimizeShortDurationEnt::getIsOptimize, SpaceOptimizeShortDurationEnt::getOpenWellTime)
        );
        List<SpaceOptimizeDateDuration> durationOutputs = XCopyUtils.copyNewList(durationList, SpaceOptimizeDateDuration.class);
        Map<String, List<SpaceOptimizeDateDuration>> collect = durationOutputs.stream()
                .collect(Collectors.groupingBy(SpaceOptimizeDateDuration::getWellheadId));
        for (pps.core.space.service.data.space_optimize_short_wellhead.GetSpaceOptimizeShortWellheadOutput wellhead : wellheadOutputs) {
            wellhead.setOptimizeList(ServiceUtil.getAllDateDuration(collect.get(wellhead.getWellheadId())));
        }
        return XListResult.success(XCopyUtils.copyNewList(wellheadOutputs, GetSpaceOptimizeShortWellheadOutput.class));
    }
}