package pps.core.space.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.github.pagehelper.PageInfo;
import com.github.pagehelper.page.PageMethod;
import pps.cloud.system.service.SysOrganizationCloudService;
import pps.cloud.system.service.data.sys_organization.GetAllOuListByOuIdInput;
import pps.cloud.system.service.data.sys_organization.GetSysOrganizationViewOutput;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import pps.core.common.session.PpsUserSession;
import pps.core.space.entity.*;
import pps.core.space.mapper.*;
import pps.core.space.service.data.SpaceOptimizeDateDuration;
import pps.core.space.service.data.space_optimize_period.GetSpaceOptimizePeriodInput;
import pps.core.space.service.data.space_optimize_period.GetSpaceOptimizePeriodOutput;
import pps.core.space.service.data.space_optimize_short_period.QuerySpaceOptimizeShortPeriodViewInput;
import pps.core.space.service.data.space_optimize_short_period.QuerySpaceOptimizeShortPeriodViewOutput;
import pps.core.space.service.data.space_optimize_short_wellhead.GetSpaceOptimizeShortWellheadInput;
import pps.core.space.service.data.space_optimize_short_wellhead.GetSpaceOptimizeShortWellheadOutput;
import pps.core.space.service.data.space_optimize_short_wellhead.QuerySpaceOptimizeShortWellheadViewInput;
import pps.core.space.service.data.space_optimize_short_wellhead.QuerySpaceOptimizeShortWellheadViewOutput;
import pps.core.space.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XSingleResult;
import xstartup.feature.api.annotation.XApiGet;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 极短期间开优化模块
 *
 * @author ZWT
 * @date 2023/09/07
 */
@XService
@XText("极短期间开优化模块")
public class SpaceOptimizeShortPeriodService extends SpaceOptimizeBaseService {

    /**
     * 极短期间开开优化--分页列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XPageResult}<{@link QuerySpaceOptimizeShortPeriodViewOutput}>
     */
    @XApiGet
    @XText("极短期间开优化--分页列表")
    public XPageResult<QuerySpaceOptimizeShortPeriodViewOutput> querySpaceOptimizeShortPeriodView(XContext context, QuerySpaceOptimizeShortPeriodViewInput input) {
        SpaceOptimizeShortPeriodViewMapper mapper = context.getBean(SpaceOptimizeShortPeriodViewMapper.class);
        SpaceOptimizeShortPeriodView record = XCopyUtils.copyNewObject(input, SpaceOptimizeShortPeriodView.class);
        PageMethod.startPage(input.getPage(), input.getLimit());
        List<SpaceOptimizeShortPeriodView> list = mapper.selectList(record);
        PageInfo<SpaceOptimizeShortPeriodView> pageInfo = new PageInfo<>(list);
        List<QuerySpaceOptimizeShortPeriodViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySpaceOptimizeShortPeriodViewOutput.class);
        if (CollUtil.isNotEmpty(outputs)) {
            Map<String, String> gridTypeMap = ServiceUtil.getDictMap(context, BusinessConstant.GRID_TYPE);
            Map<String, String> ouMap = ServiceUtil.getOuMapByOuIds(context, outputs.stream()
                    .map(QuerySpaceOptimizeShortPeriodViewOutput::getOuId)
                    .distinct()
                    .collect(Collectors.toList())
            );
            String lineName = ServiceUtil.getPowerLineNameById(context, input.getLineId());
            outputs.forEach(output -> {
                output.setLineName(lineName);
                output.setOuName(ouMap.get(output.getOuId()));
                output.setGridTypeName(gridTypeMap.get(output.getGridTypeKey()));
                output.setOptimizeStateValue(output.getOptimizeState().equals(0) ? BusinessConstant.OPTIMIZED_C : BusinessConstant.NO_OPTIMIZATION_C);
            });
        }
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    /**
     * 极短期间开开优化--详情
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetSpaceOptimizePeriodOutput}>
     */
    @XApiGet
    @XText("极短期间开优化--详情")
    public XSingleResult<GetSpaceOptimizePeriodOutput> getSpaceOptimizeShortPeriodView(XContext context, GetSpaceOptimizePeriodInput input) {
        return XSingleResult.success(super.getSpaceOptimizePeriodView(context, input, BusinessConstant.FOUR, BusinessConstant.ONE));
    }

    /**
     * 获取当日井口优化列表
     * GET /space/space-optimize-short-period/get-optimize-list
     * 接口ID：158457157
     * 接口地址：https://app.apifox.com/project/3196988/apis/api-158457157
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link GetSpaceOptimizeShortWellheadOutput}>
     */
    @XApiGet
    @XText("极短期间开优化--获取当日井口优化列表")
    public XListResult<GetSpaceOptimizeShortWellheadOutput> getOptimizeList(XContext context, GetSpaceOptimizeShortWellheadInput input) {
        DateTime now = DateUtil.beginOfDay(DateTime.now());
        String lineId = input.getLineId();
        String wellheadId = input.getWellheadId();
        SpaceOptimizeShortPeriodMapper mapper = context.getBean(SpaceOptimizeShortPeriodMapper.class);
        SpaceOptimizeShortPeriodEnt periodEnt = mapper.selectOne(new LambdaQueryWrapper<SpaceOptimizeShortPeriodEnt>()
                .eq(SpaceOptimizeShortPeriodEnt::getOptimizeDate, now)
                .eq(SpaceOptimizeShortPeriodEnt::getLineId, lineId)
                .last(BusinessConstant.LAST_LIMIT)
        );
        if (ObjectUtil.isEmpty(periodEnt)) {
            return XListResult.success(Collections.emptyList());
        }
        SpaceOptimizeShortWellheadMapper wellheadMapper = context.getBean(SpaceOptimizeShortWellheadMapper.class);
        List<SpaceOptimizeShortWellheadEnt> wellheadList = wellheadMapper.selectList(new LambdaQueryWrapper<SpaceOptimizeShortWellheadEnt>()
                .eq(SpaceOptimizeShortWellheadEnt::getShortPeriodId, periodEnt.getId())
                .eq(CharSequenceUtil.isNotBlank(wellheadId), SpaceOptimizeShortWellheadEnt::getWellheadId, wellheadId)
        );
        if (CollUtil.isEmpty(wellheadList)) {
            return XListResult.success(Collections.emptyList());
        }
        List<GetSpaceOptimizeShortWellheadOutput> wellheadOutputs = XCopyUtils.copyNewList(wellheadList, GetSpaceOptimizeShortWellheadOutput.class);
        SpaceOptimizeShortDurationMapper durationMapper = context.getBean(SpaceOptimizeShortDurationMapper.class);
        List<SpaceOptimizeShortDurationEnt> durationList = durationMapper.selectList(new LambdaQueryWrapper<SpaceOptimizeShortDurationEnt>()
                .eq(SpaceOptimizeShortDurationEnt::getShortPeriodId, periodEnt.getId())
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                .eq(SpaceOptimizeShortDurationEnt::getIsOptimize, BusinessConstant.ZERO)
                .eq(CharSequenceUtil.isNotBlank(wellheadId), SpaceOptimizeShortDurationEnt::getWellheadId, wellheadId)
                .orderByAsc(SpaceOptimizeShortDurationEnt::getIsOptimize, SpaceOptimizeShortDurationEnt::getOpenWellTime)
        );
        List<SpaceOptimizeDateDuration> durationOutputs = XCopyUtils.copyNewList(durationList, SpaceOptimizeDateDuration.class);
        Map<String, List<SpaceOptimizeDateDuration>> collect = durationOutputs.stream()
                .collect(Collectors.groupingBy(SpaceOptimizeDateDuration::getWellheadId));
        for (GetSpaceOptimizeShortWellheadOutput wellhead : wellheadOutputs) {
            wellhead.setOptimizeList(ServiceUtil.getAllDateDuration(collect.get(wellhead.getWellheadId())));
        }
        return XListResult.success(wellheadOutputs);
    }

    /**
     * 先导实验井间开制度
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XPageResult}<{@link QuerySpaceOptimizeShortWellheadViewOutput}>
     */
    @XApiGet
    @XText("极短期间开优化--先导实验井间开制度")
    public XPageResult<QuerySpaceOptimizeShortWellheadViewOutput> getDailySystemDisplay(XContext context, QuerySpaceOptimizeShortWellheadViewInput input) {
        String stationId = input.getStationId();
        if (CharSequenceUtil.isBlank(stationId)) {
            stationId = context.getSession(PpsUserSession.class).getOuId();
        }
        SysOrganizationCloudService organizationCloudService = context.getBean(SysOrganizationCloudService.class);
        XListResult<GetSysOrganizationViewOutput> result = organizationCloudService.getAllListByOuId(context, GetAllOuListByOuIdInput.builder()
                .ouId(stationId)
                .ouLevel(BusinessConstant.PLANT_FLAG)
                .build());
        result.throwIfFail();
        List<GetSysOrganizationViewOutput> plantList = result.getResult();
        if (CollUtil.isEmpty(plantList)) {
            return XPageResult.success(Collections.emptyList(), 1, 1, 0);
        }
        List<String> ouIds = plantList.stream()
                .map(GetSysOrganizationViewOutput::getId)
                .collect(Collectors.toList());
        SpaceOptimizeShortDurationViewMapper mapper = context.getBean(SpaceOptimizeShortDurationViewMapper.class);
        PageMethod.startPage(input.getPage(), input.getLimit());
        List<SpaceOptimizeShortDurationView> list = mapper.selectDailySystemDisplay(SpaceOptimizeShortDurationView.builder()
                .queryDate(DateUtil.beginOfDay(DateUtil.date()))
                .ouIds(ouIds)
                .build());
        PageInfo<SpaceOptimizeShortDurationView> pageInfo = new PageInfo<>(list);
        return XPageResult.success(XCopyUtils.copyNewList(pageInfo.getList(), QuerySpaceOptimizeShortWellheadViewOutput.class),
                input, pageInfo.getTotal());
    }
}