package pps.core.space.service;

import cn.hutool.core.collection.CollUtil;
import pps.cloud.space.service.ISpaceOptimizeUltraCloudService;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.utils.BaseUtils;
import pps.core.space.entity.SpaceOptimizeDurationDTO;
import pps.core.space.entity.SpaceOptimizePeriodDTO;
import pps.core.space.entity.SpaceOptimizeWellheadDTO;
import pps.core.space.mapper.SpaceOptimizeViewMapper;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.data.XServiceResult;
import xstartup.helper.XTransactionHelper;

import java.util.ArrayList;
import java.util.List;

/**
 * 长期间开优化Cloud模块
 *
 * @author ZWT
 * @date 2024/03/22
 */
@XService
public class SpaceOptimizeUltraCloudServiceImpl extends SpaceOptimizeBaseService implements ISpaceOptimizeUltraCloudService {

    /**
     * 长期间开优化Cloud模块--定时任务
     *
     * @param context 上下文
     * @return {@link XServiceResult}
     */
    @Override
    public XServiceResult optimizeUltraJob(XContext context) {
        List<SpaceOptimizePeriodDTO> periodDTOList = new ArrayList<>(32);
        List<SpaceOptimizeWellheadDTO> wellheadDTOList = new ArrayList<>(64);
        List<SpaceOptimizeDurationDTO> durationDTOList = new ArrayList<>(128);
        List<SpaceOptimizeDurationDTO> unOptimizeDurationList = new ArrayList<>(128);
        super.doOptimize(context, periodDTOList, wellheadDTOList, durationDTOList, unOptimizeDurationList, BusinessConstant.ZERO, BusinessConstant.TWO, 15);
        //开启事务
        return XTransactionHelper.begin(context, () -> {
            int size;
            SpaceOptimizeViewMapper optimizeViewMapper = context.getBean(SpaceOptimizeViewMapper.class);
            if (CollUtil.isNotEmpty(periodDTOList)) {
                size = periodDTOList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<SpaceOptimizePeriodDTO>> subList = BaseUtils.getSubList(periodDTOList);
                    subList.forEach(optimizeViewMapper::ultraPeriodBatchInsertList);
                } else {
                    optimizeViewMapper.ultraPeriodBatchInsertList(periodDTOList);
                }
            }
            if (CollUtil.isNotEmpty(wellheadDTOList)) {
                size = wellheadDTOList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<SpaceOptimizeWellheadDTO>> subList = BaseUtils.getSubList(wellheadDTOList);
                    subList.forEach(optimizeViewMapper::ultraWellheadBatchInsertList);
                } else {
                    optimizeViewMapper.ultraWellheadBatchInsertList(wellheadDTOList);
                }
            }
            if (CollUtil.isNotEmpty(unOptimizeDurationList)) {
                size = unOptimizeDurationList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<SpaceOptimizeDurationDTO>> subList = BaseUtils.getSubList(unOptimizeDurationList);
                    subList.forEach(optimizeViewMapper::ultraDurationBatchInsertList);
                } else {
                    optimizeViewMapper.ultraDurationBatchInsertList(unOptimizeDurationList);
                }
            }
            if (CollUtil.isNotEmpty(durationDTOList)) {
                size = durationDTOList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<SpaceOptimizeDurationDTO>> subList = BaseUtils.getSubList(durationDTOList);
                    subList.forEach(optimizeViewMapper::ultraDurationBatchInsertList);
                } else {
                    optimizeViewMapper.ultraDurationBatchInsertList(durationDTOList);
                }
            }
            return XServiceResult.OK;
        });
    }
}