package pps.core.prediction.mapper;

import org.springframework.stereotype.Repository;
import pps.core.prediction.entity.PredictedPowerView;

import java.util.List;

/**
 * 功率预测
 *
 * @author ZWT
 * @date 2024/03/21 17:30
 */
@Repository(value = "pps.core.prediction.mapper.PredictedPowerViewMapper")
public interface PredictedPowerViewMapper {

    /**
     * 获取中期预测数据
     *
     * @param view 看法
     * @return {@link List}<{@link PredictedPowerView}>
     */
    List<PredictedPowerView> selectMidPowerList(PredictedPowerView view);

    /**
     * 获取短期预测数据
     *
     * @param view 看法
     * @return {@link List}<{@link PredictedPowerView}>
     */
    List<PredictedPowerView> selectShortPowerList(PredictedPowerView view);

    /*-------------------------------------------------------------------------*/

    /**
     * 获取中期预测数据(小时)
     *
     * @param view 看法
     * @return {@link List}<{@link PredictedPowerView}>
     */
    List<PredictedPowerView> selectMidPowerHourList(PredictedPowerView view);

    /**
     * 获取短期预测数据(小时)
     *
     * @param view 看法
     * @return {@link List}<{@link PredictedPowerView}>
     */
    List<PredictedPowerView> selectShortPowerHourList(PredictedPowerView view);

    /*-------------------------------------------------------------------------*/

    /**
     * 获取中期半小时预测数据
     *
     * @param view 看法
     * @return {@link List}<{@link PredictedPowerView}>
     */
    List<PredictedPowerView> selectMidPowerHalfHourList(PredictedPowerView view);

    /**
     * 获取短期半小时预测数据
     *
     * @param view 看法
     * @return {@link List}<{@link PredictedPowerView}>
     */
    List<PredictedPowerView> selectShortPowerHalfHourList(PredictedPowerView view);
}
