package pps.core.prediction.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.excel.EasyExcelFactory;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.excel.util.EasyExcelUtil;
import pps.core.common.excel.util.ExcelStyleTool;
import pps.core.prediction.entity.PlantPowerDataEnt;
import pps.core.prediction.entity.PlantPowerDataView;
import pps.core.prediction.enums.ImportFieldDic;
import pps.core.prediction.mapper.PlantPowerDataMapper;
import pps.core.prediction.mapper.PlantPowerDataViewMapper;
import pps.core.prediction.service.data.plant_power_data.ExcelPlantPowerTemplate;
import pps.core.prediction.service.data.plant_power_data.ImportFileInput;
import pps.core.prediction.service.data.plant_power_data.QueryPlantPowerDataInput;
import pps.core.prediction.service.data.plant_power_data.QueryPlantPowerDataOutput;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.tool.XStorageTool;
import xstartup.base.util.XHttpUtils;
import xstartup.data.XFileResult;
import xstartup.data.XFileType;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiUpload;
import xstartup.feature.mybatis.helper.XMapperHelper;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

@XService
@XText("电站实际发电数据")
public class PlantPowerDataService {

    @XApiGet
    public XPageResult<QueryPlantPowerDataOutput> queryPlantPowerDataList(XContext context, QueryPlantPowerDataInput input) {
        PlantPowerDataMapper mapper = context.getBean(PlantPowerDataMapper.class);
        QueryWrapper<PlantPowerDataEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(PlantPowerDataEnt::getPlantId, input.getPlantId());
        if (CharSequenceUtil.isNotBlank(input.getStartTime()))
            queryWrapper.lambda().ge(PlantPowerDataEnt::getDataDate, input.getStartTime());
        if (CharSequenceUtil.isNotBlank(input.getEndTime()))
            queryWrapper.lambda().ge(PlantPowerDataEnt::getDataDate, input.getEndTime());
        queryWrapper.lambda().orderByDesc(PlantPowerDataEnt::getDataDate);
        return XMapperHelper.query(mapper, input, queryWrapper, QueryPlantPowerDataOutput.class);
    }

    /**
     * 下载导入模板
     *
     * @param context 上下文
     * @return {@link XFileResult }
     */
    @XText("下载电站实际发电数据导入模板")
    @XApiGet
    public XFileResult downloadImportTemplate(XContext context) {
        DateTime date = DateUtil.date();
        String fileSavePath = XStorageTool.getAbsolutePath("/temp/excel/导入模板_" + DateUtil.formatDate(date) + ".xlsx");
        //生成模板
        EasyExcelFactory.write(FileUtil.touch(fileSavePath))
                .registerWriteHandler(ExcelStyleTool.getStyleStrategy())
                .autoCloseStream(Boolean.TRUE)
                .build()
                .write(CollUtil.list(false, ExcelPlantPowerTemplate.builder()
                        .dataDate(date)
                        .windSpeed(BusinessConstant.BIG_DECIMAL_6_67)
                        .windDirection(BigDecimal.ZERO)
                        .temperature(BusinessConstant.BIG_DECIMAL_4)
                        .pressure(BigDecimal.ONE)
                        .humidity(BusinessConstant.BIG_DECIMAL_3)
                        .planeIrradiance(BusinessConstant.BIG_DECIMAL_2)
                        .horizontalIrradiance(BusinessConstant.BIG_DECIMAL_4)
                        .power(BusinessConstant.BIG_DECIMAL_60)
                        .build()), EasyExcelUtil.writeSelectedSheet(ExcelPlantPowerTemplate.class, 0, "导入模板"))
                .finish();
        return XFileResult.success(XFileType.APPLICATION_XLSX, fileSavePath);
    }

    /**
     * 导入工厂电力数据
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult }
     */
    @XText("导入电站实际发电数据")
    @XApiUpload
    public XServiceResult importPlantPowerData(XContext context, ImportFileInput input) {
        List<ExcelPlantPowerTemplate> list = EasyExcelUtil.getExcelDataAndCheck(input.getFile(), ExcelPlantPowerTemplate.class,
                null, null);
        if (CollUtil.isNotEmpty(list)) {
            DateTime dateTime;
            DateTime minDate = null;
            DateTime maxDate = null;
            List<PlantPowerDataView> dataList = new ArrayList<>(list.size());
            for (ExcelPlantPowerTemplate template : list) {
                if (ObjectUtil.isNotNull(template.getDataDate())) {
                    dateTime = DateUtil.date(template.getDataDate());
                    if (ObjectUtil.isNull(minDate)) {
                        minDate = dateTime;
                        maxDate = dateTime;
                    }
                    if (minDate.compareTo(dateTime) > 0) {
                        minDate = dateTime;
                    } else if (dateTime.compareTo(maxDate) > 0) {
                        maxDate = dateTime;
                    }
                    dataList.add(
                            PlantPowerDataView.builder()
                                    .dataDate(dateTime)
                                    .windSpeed(BusinessConstant.BIG_DECIMAL_6_67)
                                    .windDirection(BigDecimal.ZERO)
                                    .temperature(BusinessConstant.BIG_DECIMAL_4)
                                    .pressure(BigDecimal.ONE)
                                    .humidity(BusinessConstant.BIG_DECIMAL_3)
                                    .planeIrradiance(BusinessConstant.BIG_DECIMAL_2)
                                    .horizontalIrradiance(BusinessConstant.BIG_DECIMAL_4)
                                    .power(BusinessConstant.BIG_DECIMAL_60)
                                    .build()
                    );
                }
            }
            PlantPowerDataMapper mapper = context.getBean(PlantPowerDataMapper.class);
            mapper.delete(new QueryWrapper<PlantPowerDataEnt>()
                    .lambda()
                    .eq(PlantPowerDataEnt::getPlantId, input.getPlantId())
                    .ge(PlantPowerDataEnt::getDataDate, minDate)
                    .le(PlantPowerDataEnt::getDataDate, maxDate));
            PlantPowerDataViewMapper viewMapper = context.getBean(PlantPowerDataViewMapper.class);
            viewMapper.insertBatch(dataList);
            //进行训练
            doTrainData(context, input.getPlantId());
        }
        return XServiceResult.OK;
    }

    private List<String> getExcelHeaderList() {
        List<Map<String, Object>> fieldList = new ArrayList<>();
        for (ImportFieldDic value : ImportFieldDic.values()) {
            Map<String, Object> map = new HashMap<>();
            map.put("col", value.getCol());
            map.put("sort", value.getSort());
            fieldList.add(map);
        }
        fieldList.sort(Comparator.comparing(o -> ((Integer) o.get("sort"))));
        return fieldList.stream().map(x -> ((String) x.get("col"))).collect(Collectors.toList());
    }

    public void doTrainData(XContext context, String plantId) {
        try {
            XHttpUtils.postAsJson("http://127.0.0.1:10098/aiprediction/xgbtrain?plantId=" + plantId, 10000);
        } catch (Exception e) {
            context.getLogger().error(e.getMessage());
        }
    }
}