package pps.core.prediction.service;

import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import pps.cloud.base.service.data.base_photovoltaic_plant.GetBasePhotovoltaicPlantCloudInput;
import pps.cloud.base.service.data.base_photovoltaic_plant.GetBasePhotovoltaicPlantCloudOutput;
import pps.cloud.prediction.service.PlantPredictedPowerLongTermDataCloudService;
import pps.core.prediction.entity.PlantPowerDataEnt;
import pps.core.prediction.entity.PlantPredictedPowerLongTermDataEnt;
import pps.core.prediction.mapper.PlantPowerDataMapper;
import pps.core.prediction.mapper.PlantPredictedPowerLongTermDataMapper;
import pps.core.prediction.mapper.PlantPredictedPowerLongTermDataViewMapper;
import pps.core.prediction.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XServiceResult;
import xstartup.feature.api.annotation.XApiGet;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

/**
 * 光伏预测Cloud模块
 *
 * @author ZWT
 * @date 2023/09/13 19:05
 */
@XService
public class PlantPredictedPowerLongTermDataCloudServiceImpl implements PlantPredictedPowerLongTermDataCloudService {

    @Override
    @XApiGet
    @XText("定时运行预测长期数据")
    public XServiceResult runPlantPredictedPowerLongTermData(XContext context) {
        //获取所有的电站
        List<GetBasePhotovoltaicPlantCloudOutput> list = ServiceUtil.getBasePhotovoltaicPlantList(context, new GetBasePhotovoltaicPlantCloudInput());
        PlantPowerDataMapper powerDataMapper = context.getBean(PlantPowerDataMapper.class);
        Date currentDate = new Date();
        for (GetBasePhotovoltaicPlantCloudOutput output : list) {
            //获取最近七天的实际数据
            QueryWrapper<PlantPowerDataEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.select("IFNULL(avg(power ) , 0 ) power")
                    .lambda()
                    .eq(PlantPowerDataEnt::getPlantId, output.getId())
                    .ge(PlantPowerDataEnt::getDataDate, DateUtil.offsetDay(currentDate, -7));
            PlantPowerDataEnt currentEnt = powerDataMapper.selectOne(queryWrapper);
            //获取同期七天的实际数据
            queryWrapper.clear();
            queryWrapper.select("IFNULL(avg(power ) , 0 ) power")
                    .lambda()
                    .eq(PlantPowerDataEnt::getPlantId, output.getId())
                    .ge(PlantPowerDataEnt::getDataDate, DateUtil.offset(currentDate, DateField.YEAR, -1));
            PlantPowerDataEnt lastEnt = powerDataMapper.selectOne(queryWrapper);
            //获取同比 = 当前/同期
            BigDecimal compare = new BigDecimal(1);
            if (lastEnt.getPower().compareTo(BigDecimal.ZERO) > 0 && currentEnt.getPower().compareTo(BigDecimal.ZERO) != 0) {
                compare = currentEnt.getPower().divide(lastEnt.getPower(), 6, RoundingMode.HALF_UP);
            }
            //获取同期30天数据
            queryWrapper.clear();
            queryWrapper.select("plant_id, data_date, temperature, humidity, wind_speed, wind_direction, pressure, plane_irradiance, horizontal_irradiance, power");
            queryWrapper.lambda()
                    .eq(PlantPowerDataEnt::getPlantId, output.getId())
                    .ge(PlantPowerDataEnt::getDataDate, DateUtil.offset(currentDate, DateField.YEAR, -1))
                    .le(PlantPowerDataEnt::getDataDate, DateUtil.offsetMonth(currentDate, -11));
            List<PlantPowerDataEnt> lastList = powerDataMapper.selectList(queryWrapper);
            //未来30天=同期30*同比
            List<PlantPredictedPowerLongTermDataEnt> insertList = new ArrayList<>();
            for (PlantPowerDataEnt item : lastList) {
                item.setPower(item.getPower().multiply(compare));
                item.setDataDate(DateUtil.formatDate(DateUtil.offset(DateUtil.parseDateTime(item.getDataDate()), DateField.YEAR, 1)));
                PlantPredictedPowerLongTermDataEnt insertData = XCopyUtils.copyNewObject(item, PlantPredictedPowerLongTermDataEnt.class);
                insertData.setCreateTime(currentDate);
                insertList.add(insertData);
            }
            //插入数据前，先删除数据
            PlantPredictedPowerLongTermDataMapper longTermDataMapper = context.getBean(PlantPredictedPowerLongTermDataMapper.class);
            longTermDataMapper.delete(new QueryWrapper<PlantPredictedPowerLongTermDataEnt>().lambda()
                    .eq(PlantPredictedPowerLongTermDataEnt::getPlantId, output.getId())
                    .ge(PlantPredictedPowerLongTermDataEnt::getDataDate, currentDate)
                    .le(PlantPredictedPowerLongTermDataEnt::getDataDate, DateUtil.offsetMonth(currentDate, 1)));
            PlantPredictedPowerLongTermDataViewMapper longTermDataViewMapper = context.getBean(PlantPredictedPowerLongTermDataViewMapper.class);
            List<PlantPredictedPowerLongTermDataEnt> batchList = new ArrayList<>();
            List<String> timeList = Arrays.asList("15", "30", "45");
            for (int i = 0; i < insertList.size(); i++) {
                PlantPredictedPowerLongTermDataEnt item = insertList.get(i);
                DateTime dateTime = DateUtil.parseDateTime(item.getDataDate());
                item.setYearTime(dateTime.year() + "");
                item.setMonthTime(String.format("%02d", dateTime.monthBaseOne()));
                item.setDayTime(String.format("%02d", dateTime.dayOfMonth()));
                item.setHourTime(String.format("%02d", dateTime.hour(true)));
                item.setMinTime(String.format("%02d", dateTime.minute() + ":00"));
                batchList.add(item);
                if (i < insertList.size() - 1) {
                    for (int y = 0; y < timeList.size(); y++) {
                        PlantPredictedPowerLongTermDataEnt dao = XCopyUtils.copyNewObject(item);
                        dao.setMinTime(timeList.get(y) + ":00");
                        dao.setDataDate(dao.getYearTime() + "-" + dao.getMonthTime() + "-" + dao.getDayTime() + " " + dao.getHourTime() + ":" + dao.getMinTime());
                        int I_i = i + 1;
                        BigDecimal temperature_1 = insertList.get(i).getTemperature() == null ? new BigDecimal(0) : insertList.get(i).getTemperature();
                        BigDecimal temperature_2 = insertList.get(I_i).getTemperature() == null ? new BigDecimal(0) : insertList.get(I_i).getTemperature();
                        dao.setTemperature(temperature_1.add(temperature_2.subtract(temperature_1).multiply(BigDecimal.valueOf(0.3 * (y + 1)))));
                        BigDecimal humidity_1 = insertList.get(i).getHumidity() == null ? new BigDecimal(0) : insertList.get(i).getHumidity();
                        BigDecimal humidity_2 = insertList.get(I_i).getHumidity() == null ? new BigDecimal(0) : insertList.get(I_i).getHumidity();
                        dao.setHumidity(humidity_1.add(humidity_2.subtract(humidity_1).multiply(BigDecimal.valueOf(0.3 * (y + 1)))));
                        BigDecimal windSpeed_1 = insertList.get(i).getWindSpeed() == null ? new BigDecimal(0) : insertList.get(i).getWindSpeed();
                        BigDecimal windSpeed_2 = insertList.get(I_i).getWindSpeed() == null ? new BigDecimal(0) : insertList.get(I_i).getWindSpeed();
                        dao.setWindSpeed(windSpeed_1.add(windSpeed_2.subtract(windSpeed_1).multiply(BigDecimal.valueOf(0.3 * (y + 1)))));
                        BigDecimal pressure_1 = insertList.get(i).getPressure() == null ? new BigDecimal(0) : insertList.get(i).getPressure();
                        BigDecimal pressure_2 = insertList.get(I_i).getPressure() == null ? new BigDecimal(0) : insertList.get(I_i).getPressure();
                        dao.setPressure(pressure_1.add(pressure_2.subtract(pressure_1).multiply(BigDecimal.valueOf(0.3 * (y + 1)))));
                        BigDecimal planeIrradiance_1 = insertList.get(i).getPlaneIrradiance() == null ? new BigDecimal(0) : insertList.get(i).getPlaneIrradiance();
                        BigDecimal planeIrradiance_2 = insertList.get(I_i).getPlaneIrradiance() == null ? new BigDecimal(0) : insertList.get(I_i).getPlaneIrradiance();
                        dao.setPlaneIrradiance(planeIrradiance_1.add(planeIrradiance_2.subtract(planeIrradiance_1).multiply(BigDecimal.valueOf(0.3 * (y + 1)))));
                        BigDecimal horizontalIrradiance_1 = insertList.get(i).getHorizontalIrradiance() == null ? new BigDecimal(0) : insertList.get(i).getHorizontalIrradiance();
                        BigDecimal horizontalIrradiance_2 = insertList.get(I_i).getHorizontalIrradiance() == null ? new BigDecimal(0) : insertList.get(I_i).getHorizontalIrradiance();
                        dao.setHorizontalIrradiance(horizontalIrradiance_1.add(horizontalIrradiance_2.subtract(horizontalIrradiance_1).multiply(BigDecimal.valueOf(0.3 * (y + 1)))));
                        BigDecimal power_1 = insertList.get(i).getPower() == null ? new BigDecimal(0) : insertList.get(i).getPower();
                        BigDecimal power_2 = insertList.get(I_i).getPower() == null ? new BigDecimal(0) : insertList.get(I_i).getPower();
                        dao.setPower(power_1.add(power_2.subtract(power_1).multiply(BigDecimal.valueOf(0.3 * (y + 1)))));
                        batchList.add(dao);
                    }
                }
                if (batchList.size() > 500) {
                    longTermDataViewMapper.insertBatch(batchList);
                    batchList = new ArrayList<>();
                }
            }
            if (!batchList.isEmpty())
                longTermDataViewMapper.insertBatch(batchList);
        }
        return XServiceResult.OK;
    }
}