package pps.core.prediction.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.text.CharSequenceUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import pps.cloud.base.service.data.base_power_line_wellhead.DynamicQueryBasePowerLineWellheadInput;
import pps.cloud.base.service.data.base_power_line_wellhead.DynamicQueryBasePowerLineWellheadViewOutput;
import pps.core.common.constant.BusinessConstant;
import pps.core.prediction.entity.ThirdCurrentWellConditionEnt;
import pps.core.prediction.mapper.ThirdCurrentWellConditionMapper;
import pps.core.prediction.service.data.third_current_well_condition.QueryThirdCurrentWellConditionInput;
import pps.core.prediction.service.data.third_current_well_condition.QueryThirdCurrentWellConditionOutput;
import pps.core.prediction.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.data.XListResult;
import xstartup.feature.api.annotation.XApiPost;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 第三方当日油井状态
 *
 * @author ZWT
 * @date 2024/03/19
 */
@XText("第三方当日油井状态")
@XService
public class ThirdCurrentWellConditionService {

    /**
     * 油井状态
     * POST /prediction/third-current-well-condition/query-well-real-pv-data
     * 接口ID：156558765
     * 接口地址：https://app.apifox.com/project/3196988/apis/api-156558765
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link QueryThirdCurrentWellConditionOutput}>
     */
    @XText("井口生产情况--油井状态")
    @XApiPost
    public XListResult<QueryThirdCurrentWellConditionOutput> queryWellRealPvData(XContext context, QueryThirdCurrentWellConditionInput input) {
        List<DynamicQueryBasePowerLineWellheadViewOutput> list = ServiceUtil.getPowerLineWellheadList(context, DynamicQueryBasePowerLineWellheadInput.builder()
                .lineId(input.getLineId())
                .build());
        if (CollUtil.isEmpty(list)) {
            return XListResult.success(Collections.emptyList());
        }
        String oilFieldCode = ServiceUtil.getOilFieldCode(context);
        ThirdCurrentWellConditionMapper mapper = context.getBean(ThirdCurrentWellConditionMapper.class);
        List<ThirdCurrentWellConditionEnt> selectList = mapper.selectList(new LambdaQueryWrapper<ThirdCurrentWellConditionEnt>()
                .in(ThirdCurrentWellConditionEnt::getWellNumber, list.stream()
                        .map(DynamicQueryBasePowerLineWellheadViewOutput::getWellNumber)
                        .collect(Collectors.toList()))
                .eq(ThirdCurrentWellConditionEnt::getSystemSource, oilFieldCode)
        );
        List<QueryThirdCurrentWellConditionOutput> outputList = new ArrayList<>(selectList.size());
        for (ThirdCurrentWellConditionEnt ent : selectList) {
            outputList.add(
                    QueryThirdCurrentWellConditionOutput.builder()
                            .wellNumber(ent.getWellNumber())
                            .nextTime(CharSequenceUtil.equals(ent.getWellStatus(), BusinessConstant.OPEN_WELL) ? ent.getNextTime() : null)
                            .wellStatus(ent.getWellStatus())
                            .cumulativeProduction(ent.getCumulativeProduction())
                            .runningTime(ent.getRunningTime())
                            .build()
            );
        }
        return XListResult.success(outputList);
    }
}
