package pps.core.prediction.service;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import pps.cloud.base.service.data.base_power_line.DynamicQueryBasePowerLineOutput;
import pps.cloud.base.service.data.base_power_line_wellhead.DynamicQueryBasePowerLineWellheadInput;
import pps.cloud.base.service.data.base_power_line_wellhead.DynamicQueryBasePowerLineWellheadViewOutput;
import pps.core.common.utils.BaseUtils;
import pps.core.prediction.entity.ThirdPhotovoltaicPowerEnt;
import pps.core.prediction.entity.WellheadDailyProductionSituationEnt;
import pps.core.prediction.mapper.ThirdPhotovoltaicPowerMapper;
import pps.core.prediction.mapper.WellheadDailyProductionSituationMapper;
import pps.core.prediction.service.data.third_photovoltaic_power.GetThirdPhotovoltaicPowerInput;
import pps.core.prediction.service.data.third_photovoltaic_power.GetThirdPhotovoltaicPowerOutput;
import pps.core.prediction.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.data.XSingleResult;
import xstartup.feature.api.annotation.XApiPost;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 第三方光伏日发电量
 *
 * @author ZWT
 * @date 2024/03/19
 */
@XText("第三方光伏日发电量")
@XService
public class ThirdPhotovoltaicPowerService {

    /**
     * 获取光伏发电概述
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetThirdPhotovoltaicPowerOutput}>
     */
    @XText("能耗分析--能耗概览")
    @XApiPost
    public XSingleResult<GetThirdPhotovoltaicPowerOutput> getPhotovoltaicPowerOverview(XContext context, GetThirdPhotovoltaicPowerInput input) {
        GetThirdPhotovoltaicPowerOutput output = new GetThirdPhotovoltaicPowerOutput();
        List<DynamicQueryBasePowerLineOutput> lineList = ServiceUtil.getBasePowerLineListByOuIdList(context, ServiceUtil.getOrgIdsByPath(context, input.getOuId()));
        if (CollUtil.isEmpty(lineList)) {
            output.setPowerGeneration(BigDecimal.ZERO);
            output.setPowerConsumption(BigDecimal.ZERO);
            output.setAbsorptionRate(BigDecimal.ZERO);
        } else {
            List<String> lineNameList = new ArrayList<>(lineList.size());
            List<String> lineIdList = new ArrayList<>(lineList.size());
            lineList.forEach(line -> {
                lineNameList.add(line.getLineName());
                lineIdList.add(line.getId());
            });
            input.getBetweenDate();
            BigDecimal powerConsumption;
            //查井口
            List<DynamicQueryBasePowerLineWellheadViewOutput> wellheadList = ServiceUtil.getPowerLineWellheadList(context, DynamicQueryBasePowerLineWellheadInput.builder()
                    .lineIds(lineIdList)
                    .build());
            if (CollUtil.isEmpty(lineList)) {
                powerConsumption = BigDecimal.ONE;
            } else {
                WellheadDailyProductionSituationMapper wellheadMapper = context.getBean(WellheadDailyProductionSituationMapper.class);
                WellheadDailyProductionSituationEnt wellheadSituationEnt = wellheadMapper.selectOne(new QueryWrapper<WellheadDailyProductionSituationEnt>()
                        .select("IFNULL( SUM( daily_electricity_consumption ), 0 ) AS daily_electricity_consumption")
                        .lambda()
                        .in(WellheadDailyProductionSituationEnt::getWellheadId, wellheadList.stream()
                                .map(DynamicQueryBasePowerLineWellheadViewOutput::getWellheadId)
                                .collect(Collectors.toSet()))
                        .between(WellheadDailyProductionSituationEnt::getDataDate, input.getStartTime(), input.getEndTime())
                );
                powerConsumption = wellheadSituationEnt.getDailyElectricityConsumption();
            }
            //查光伏发电
            ThirdPhotovoltaicPowerMapper powerMapper = context.getBean(ThirdPhotovoltaicPowerMapper.class);
            ThirdPhotovoltaicPowerEnt photovoltaicPowerEnt = powerMapper.selectOne(new QueryWrapper<ThirdPhotovoltaicPowerEnt>()
                    .select("IFNULL( SUM( photovoltaic_power ), 0 ) AS photovoltaic_power")
                    .lambda()
                    .in(ThirdPhotovoltaicPowerEnt::getStationName, lineNameList)
                    .between(ThirdPhotovoltaicPowerEnt::getCreateDate, input.getStartTime(), input.getEndTime())
            );
            output.setPowerGeneration(photovoltaicPowerEnt.getPhotovoltaicPower());
            output.setPowerConsumption(powerConsumption);
            //消纳率:光伏发电/累计用电*100%
            output.setAbsorptionRate(BaseUtils.getRate(photovoltaicPowerEnt.getPhotovoltaicPower(), powerConsumption));
        }
        return XSingleResult.success(output);
    }
}
