package pps.core.prediction.service;

import cn.hutool.core.text.CharSequenceUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import pps.cloud.prediction.service.IThirdPowerCloudService;
import pps.cloud.prediction.service.data.third_active_power.GetThirdActivePowerInput;
import pps.cloud.prediction.service.data.third_active_power.GetThirdActivePowerOutput;
import pps.cloud.prediction.service.data.third_daily_accumulation_update.GetThirdDailyAccumulationUpdateInput;
import pps.cloud.prediction.service.data.third_daily_accumulation_update.GetThirdDailyAccumulationUpdateOutput;
import pps.cloud.prediction.service.data.wellhead_daily_production_situation.GetWellheadDailyProductionSituationInput;
import pps.cloud.prediction.service.data.wellhead_daily_production_situation.GetWellheadDailyProductionSituationOutput;
import pps.core.prediction.entity.ThirdActivePowerView;
import pps.core.prediction.entity.ThirdDailyAccumulationUpdateEnt;
import pps.core.prediction.entity.WellheadDailyProductionSituationEnt;
import pps.core.prediction.mapper.ThirdActivePowerViewMapper;
import pps.core.prediction.mapper.ThirdDailyAccumulationUpdateMapper;
import pps.core.prediction.mapper.WellheadDailyProductionSituationMapper;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;

import java.util.List;

/**
 * 第三方有功功率Cloud模块
 *
 * @author ZWT
 * @date 2024/05/23 14:18
 */
@XText("第三方有功功率Cloud模块")
@XService
public class ThirdPowerCloudServiceImpl implements IThirdPowerCloudService {

    /**
     * 获得15分钟平均功率
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetThirdActivePowerOutput }>
     */
    @Override
    public XListResult<GetThirdActivePowerOutput> queryAvgPhotovoltaicPower(XContext context, GetThirdActivePowerInput input) {
        ThirdActivePowerViewMapper mapper = context.getBean(ThirdActivePowerViewMapper.class);
        List<ThirdActivePowerView> list = mapper.selectAvgPhotovoltaicPower(ThirdActivePowerView.builder()
                .systemSource(input.getSystemSource())
                .createDate(input.getCreateDate())
                .build());
        return XListResult.success(XCopyUtils.copyNewList(list, GetThirdActivePowerOutput.class));
    }

    /**
     * 查询每日井场日累计数据
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetThirdDailyAccumulationUpdateOutput }>
     */
    @Override
    public XListResult<GetThirdDailyAccumulationUpdateOutput> queryDailyUpdatePower(XContext context, GetThirdDailyAccumulationUpdateInput input) {
        ThirdDailyAccumulationUpdateMapper mapper = context.getBean(ThirdDailyAccumulationUpdateMapper.class);
        List<ThirdDailyAccumulationUpdateEnt> list = mapper.selectList(new QueryWrapper<ThirdDailyAccumulationUpdateEnt>()
                .select("station_name",
                        "IFNULL( AVG( photovoltaic_power ), 0 ) AS photovoltaic_power",
                        "IFNULL( AVG( daily_electricity_consumption ), 0 ) AS daily_electricity_consumption")
                .lambda()
                .gt(ThirdDailyAccumulationUpdateEnt::getPhotovoltaicPower, 0)
                .eq(CharSequenceUtil.isNotBlank(input.getSystemSource()), ThirdDailyAccumulationUpdateEnt::getSystemSource, input.getSystemSource())
                .apply("DATE( create_date ) = {0}", input.getCreateDate())
                .groupBy(ThirdDailyAccumulationUpdateEnt::getStationName)
        );
        return XListResult.success(XCopyUtils.copyNewList(list, GetThirdDailyAccumulationUpdateOutput.class));
    }

    /**
     * 查询每日井口生产情况
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetWellheadDailyProductionSituationOutput }>
     */
    @Override
    public XListResult<GetWellheadDailyProductionSituationOutput> queryWellProductionList(XContext context, GetWellheadDailyProductionSituationInput input) {
        WellheadDailyProductionSituationMapper mapper = context.getBean(WellheadDailyProductionSituationMapper.class);
        List<WellheadDailyProductionSituationEnt> list = mapper.selectList(new QueryWrapper<WellheadDailyProductionSituationEnt>()
                .select("well_number",
                        "IFNULL( production_time, 0 ) AS production_time",
                        "IFNULL( daily_electricity_consumption, 0 ) AS daily_electricity_consumption",
                        "IFNULL( daily_liquid_production, 0 ) AS daily_liquid_production")
                .lambda()
                .eq(WellheadDailyProductionSituationEnt::getDataDate, input.getCreateDate())
                .eq(CharSequenceUtil.isNotBlank(input.getSystemSource()), WellheadDailyProductionSituationEnt::getSystemSource, input.getSystemSource())
        );
        return XListResult.success(XCopyUtils.copyNewList(list, GetWellheadDailyProductionSituationOutput.class));
    }
}