package pps.core.prediction.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.text.CharSequenceUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.github.pagehelper.PageInfo;
import com.github.pagehelper.page.PageMethod;
import pps.core.auth.HttpRequestClient;
import pps.core.common.constant.BusinessConstant;
import pps.core.prediction.constant.ThirdPartyApiConstant;
import pps.core.prediction.entity.WellheadDailyProductionSituationEnt;
import pps.core.prediction.excel.listener.ThirdActivePowerDataListener;
import pps.core.prediction.mapper.WellheadDailyProductionSituationMapper;
import pps.core.prediction.service.data.third_active_power.ThirdActivePowerExcelData;
import pps.core.prediction.service.data.wellhead_daily_production_situation.GetWellheadDailyProductionSituationInput;
import pps.core.prediction.service.data.wellhead_daily_production_situation.GetWellheadDailyProductionSituationOutput;
import pps.core.prediction.service.data.wellhead_daily_production_situation.QueryWellheadDailyProductionSituationInput;
import pps.core.prediction.service.data.wellhead_daily_production_situation.QueryWellheadDailyProductionSituationOutput;
import pps.core.prediction.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.XStartup;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 井口生产情况模块
 *
 * @author ZWT
 * @date 2023/09/27
 */
@XText("井口生产情况模块")
@XService
public class WellheadDailyProductionSituationService {

    /**
     * 井口生产情况--分页列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XPageResult}<{@link QueryWellheadDailyProductionSituationOutput}>
     */
    @XText("井口生产情况--分页列表")
    @XApiPost
    public XPageResult<QueryWellheadDailyProductionSituationOutput> queryWellheadDailyProductionSituationView(XContext context, QueryWellheadDailyProductionSituationInput input) {
        WellheadDailyProductionSituationMapper mapper = context.getBean(WellheadDailyProductionSituationMapper.class);
        PageMethod.startPage(input.getPage(), input.getLimit());
        LambdaQueryWrapper<WellheadDailyProductionSituationEnt> queryWrapper = new LambdaQueryWrapper<WellheadDailyProductionSituationEnt>()
                .eq(WellheadDailyProductionSituationEnt::getWellheadId, input.getWellheadId())
                .eq(WellheadDailyProductionSituationEnt::getWellNumber, input.getWellNumber())
                .orderByDesc(WellheadDailyProductionSituationEnt::getDataDate);
        String executionCycle = input.getExecutionCycle();
        if (CharSequenceUtil.isNotEmpty(executionCycle)) {
            String[] split = CharSequenceUtil.splitToArray(executionCycle, "-");
            queryWrapper.ge(WellheadDailyProductionSituationEnt::getDataDate, DateUtil.parse(split[0], BusinessConstant.DATE_FORMAT_DAY_C))
                    .le(WellheadDailyProductionSituationEnt::getDataDate, DateUtil.parse(split[1], BusinessConstant.DATE_FORMAT_DAY_C));
        }
        List<WellheadDailyProductionSituationEnt> list = mapper.selectList(queryWrapper);
        PageInfo<WellheadDailyProductionSituationEnt> pageInfo = new PageInfo<>(list);
        List<QueryWellheadDailyProductionSituationOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryWellheadDailyProductionSituationOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    /**
     * 甘特图
     * POST /prediction/wellhead-daily-production-situation/query-rpt-system-start-stop
     * 接口ID：156546795
     * 接口地址：https://app.apifox.com/project/3196988/apis/api-156546795
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link GetWellheadDailyProductionSituationOutput}>
     */
    @XText("井口生产情况--甘特图")
    @XApiPost
    public XListResult<GetWellheadDailyProductionSituationOutput> queryRptSystemStartStop(XContext context, GetWellheadDailyProductionSituationInput input) {
        String token = ServiceUtil.getToken(context);
        HttpRequestClient client = new HttpRequestClient(token);
        Map<String, Object> param = new HashMap<>(4);
        param.put("Org_Name", input.getStationName());
        param.put("WELL_NAME", input.getWellNumber());
        String url = XStartup.getCurrent().getProperty("third-party.chang-qing.url");
        url += ThirdPartyApiConstant.CQ_RPT_SYSTEM_START_STOP;
        String resultStr = client.doPostForm(url, param);
        JSONObject jsonObject = JSON.parseObject(resultStr);
        String data = jsonObject.getString("data");
        List<GetWellheadDailyProductionSituationOutput> outputs = JSON.parseArray(data, GetWellheadDailyProductionSituationOutput.class);
        if (CollUtil.isEmpty(outputs)) {
            outputs = Collections.emptyList();
        }
        return XListResult.success(outputs);
    }

    @XText("有功功率历史数据导入")
    @XApiGet
    public XServiceResult importHistory(XContext context) {
        String fileName = "D:\\home\\历史数据汇总.xlsx";
        EasyExcel.read(fileName, ThirdActivePowerExcelData.class, new ThirdActivePowerDataListener(context)).sheet().doRead();
        return XServiceResult.OK;
    }
}