package pps.core.prediction.utils;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.alibaba.fastjson2.TypeReference;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import pps.cloud.base.service.BasePhotovoltaicPlantCloudService;
import pps.cloud.base.service.IBasePowerLineCloudService;
import pps.cloud.base.service.IBaseWellheadCloudService;
import pps.cloud.base.service.IConfigOilFieldCloudService;
import pps.cloud.base.service.data.base_photovoltaic_plant.GetBasePhotovoltaicPlantCloudInput;
import pps.cloud.base.service.data.base_photovoltaic_plant.GetBasePhotovoltaicPlantCloudOutput;
import pps.cloud.base.service.data.base_power_line.DynamicQueryBasePowerLineInput;
import pps.cloud.base.service.data.base_power_line.DynamicQueryBasePowerLineOutput;
import pps.cloud.base.service.data.base_power_line_plant.DynamicQueryBasePowerLinePlantInput;
import pps.cloud.base.service.data.base_power_line_plant.DynamicQueryBasePowerLinePlantViewOutput;
import pps.cloud.base.service.data.base_power_line_wellhead.DynamicQueryBasePowerLineWellheadInput;
import pps.cloud.base.service.data.base_power_line_wellhead.DynamicQueryBasePowerLineWellheadViewOutput;
import pps.cloud.base.service.data.base_wellhead.DynamicQueryBaseWellheadInput;
import pps.cloud.base.service.data.base_wellhead.DynamicQueryBaseWellheadOutput;
import pps.cloud.base.service.data.config_oil_field.GetConfigOilFieldOutput;
import pps.cloud.system.service.SysOrganizationCloudService;
import pps.cloud.system.service.data.sys_organization.GetAllOuListByOuIdInput;
import pps.cloud.system.service.data.sys_organization.GetSysOrganizationViewOutput;
import pps.core.auth.HttpRequestClient;
import pps.core.common.cache.ThirdPartyConfigCache;
import pps.core.common.constant.BusinessConstant;
import pps.core.prediction.constant.ThirdPartyApiConstant;
import pps.core.prediction.entity.PlantPredictedPowerMidTermEnt;
import pps.core.prediction.entity.PlantPredictedPowerShortTermEnt;
import pps.core.prediction.entity.PlantPredictedPowerUltraTermEnt;
import pps.core.prediction.entity.ThirdActivePowerView;
import pps.core.prediction.mapper.PlantPredictedPowerMidTermMapper;
import pps.core.prediction.mapper.PlantPredictedPowerShortTermMapper;
import pps.core.prediction.mapper.PlantPredictedPowerUltraTermMapper;
import pps.core.prediction.mapper.ThirdActivePowerViewMapper;
import pps.core.prediction.service.data.plant_predicted_power_data.GetPredictedPowerOutput;
import xstartup.base.XContext;
import xstartup.base.XStartup;
import xstartup.base.exception.XServiceException;
import xstartup.base.util.XHttpUtils;
import xstartup.data.XListResult;
import xstartup.data.XSingleResult;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 常用服务调用工具类
 *
 * @author ZWT
 * @date 2024/03/18 15:07
 */
public class ServiceUtil {

    /**
     * 调用接口(长庆)
     *
     * @param context 上下文
     * @param api     应用程序编程接口
     * @param param   参数
     * @return {@link String}
     */
    public static String doPostFormCq(XContext context, String api, Map<String, Object> param) {
        String url = XStartup.getCurrent().getProperty("third-party.chang-qing.url");
        String token = ServiceUtil.getToken(context);
        HttpRequestClient client = new HttpRequestClient(token);
        String resultStr = client.doPostForm(url + api, param);
        JSONObject jsonObject = JSON.parseObject(resultStr);
        context.getLogger().info("**** doPostFormCq api: {}", api);
        context.getLogger().info("**** doPostFormCq resultStr: {}", resultStr);
        return jsonObject.getString("data");
    }

    /**
     * 获取长庆TOKEN
     *
     * @param context 上下文
     * @return {@link String}
     */
    public static String getToken(XContext context) {
        String token;
        ThirdPartyConfigCache exist = ThirdPartyConfigCache.exist(context, ThirdPartyApiConstant.CQ_TOKEN_CACHE_KEY);
        if (ObjectUtil.isNull(exist)) {
            token = ServiceUtil.getCqToken(context);
        } else {
            long duration = (long) exist.getDuration();
            if (duration - DateUtil.between(exist.getCurrentDate(), DateUtil.date(), DateUnit.SECOND) < ThirdPartyApiConstant.MAX_SECOND) {
                token = ServiceUtil.getCqToken(context);
            } else {
                token = exist.getCode();
            }
        }
        return token;
    }

    /**
     * 获得预测功率输出
     *
     * @param context      上下文
     * @param inputTimeStr 输入时间str
     * @param stationId    工作站id
     * @param offsetDay    偏移日
     * @param cycle        周期
     * @return {@link List }<{@link GetPredictedPowerOutput }>
     */
    public static List<GetPredictedPowerOutput> getPredictedPowerOutput(XContext context, String inputTimeStr, String stationId, int offsetDay, int cycle) {
        //判断是否需要启用演示配置
        DateTime date = DateUtil.date();
        DateTime inputTime;
        if (CharSequenceUtil.isBlank(inputTimeStr)) {
            inputTime = DateUtil.beginOfDay(date);
        } else {
            inputTime = DateUtil.parse(inputTimeStr, BusinessConstant.DATE_FORMAT_DAY);
        }
        //时间出力截至时间
        Date showTime = date;
        DateTime day = inputTime;
        int betweenDay = 0;
        //输入时间等于当日时间时,进行演示开关校验
        if (0 == DateUtil.compare(DateUtil.beginOfDay(date), inputTime)) {
            GetConfigOilFieldOutput currentConfig = ServiceUtil.getCurrentConfig(context);
            if (ObjectUtil.isNotNull(currentConfig.getDemoDate())) {
                day = currentConfig.getIsDemo().equals(0) ? DateUtil.date(currentConfig.getDemoDate()) : inputTime;
                showTime = DateUtil.parse(day.toString(BusinessConstant.DATE_FORMAT_DAY) + " " + date.toString(BusinessConstant.TIME_FORMAT));
                //计算偏移
                betweenDay = (int) DateUtil.betweenDay(inputTime, day, false);
                if (DateUtil.compare(day, inputTime) > 0) {
                    betweenDay = -betweenDay;
                }
            }
        }
        DateTime beginTime = DateUtil.offsetMinute(day, -1);
        DateTime endTime = DateUtil.offsetDay(day, offsetDay);
        showTime = DateUtil.offsetDay(showTime, offsetDay);
        //查预测功率
        Map<Date, BigDecimal> predictedMap = getPredictedPowerMap15(context, cycle, stationId, beginTime, endTime);
        //查实际功率
        Map<Date, BigDecimal> powerMap = getActivePowerMap15(context, stationId, beginTime, showTime);
        return getPowerOutput(day, endTime, betweenDay, date, predictedMap, powerMap);
    }

    /**
     * 查15分钟电站预测功率
     *
     * @param context   上下文
     * @param cycle     周期
     * @param stationId 工作站id
     * @param beginTime 开始时间
     * @param endTime   结束时间
     * @return {@link Map }<{@link DateTime }, {@link BigDecimal }>
     */
    public static Map<Date, BigDecimal> getPredictedPowerMap15(XContext context, int cycle, String stationId, Date beginTime, Date endTime) {
        //查预测功率
        Map<Date, BigDecimal> predictedMap;
        switch (cycle) {
            case 1:
                //查短期预测
                PlantPredictedPowerShortTermMapper shortMapper = context.getBean(PlantPredictedPowerShortTermMapper.class);
                List<PlantPredictedPowerShortTermEnt> predictedList = shortMapper.selectList(new LambdaQueryWrapper<PlantPredictedPowerShortTermEnt>()
                        .eq(PlantPredictedPowerShortTermEnt::getPlantId, stationId)
                        .between(PlantPredictedPowerShortTermEnt::getDataDate, beginTime, endTime)
                        .orderByAsc(PlantPredictedPowerShortTermEnt::getDataDate)
                );
                if (CollUtil.isNotEmpty(predictedList)) {
                    predictedMap = predictedList.stream()
                            .collect(Collectors.toMap(x -> DateUtil.parse(x.getDataDate()), PlantPredictedPowerShortTermEnt::getPredictPower));
                } else {
                    predictedMap = Collections.emptyMap();
                }
                break;
            case 2:
                //查中期预测
                PlantPredictedPowerMidTermMapper midMapper = context.getBean(PlantPredictedPowerMidTermMapper.class);
                List<PlantPredictedPowerMidTermEnt> midList = midMapper.selectList(new LambdaQueryWrapper<PlantPredictedPowerMidTermEnt>()
                        .eq(PlantPredictedPowerMidTermEnt::getPlantId, stationId)
                        .between(PlantPredictedPowerMidTermEnt::getDataDate, beginTime, endTime)
                        .orderByAsc(PlantPredictedPowerMidTermEnt::getDataDate)
                );
                if (CollUtil.isNotEmpty(midList)) {
                    predictedMap = midList.stream()
                            .collect(Collectors.toMap(x -> DateUtil.parse(x.getDataDate()), PlantPredictedPowerMidTermEnt::getPredictPower));
                } else {
                    predictedMap = Collections.emptyMap();
                }
                break;
            case 3:
                //4小时功率预测
                PlantPredictedPowerUltraTermMapper ultraMapper = context.getBean(PlantPredictedPowerUltraTermMapper.class);
                List<PlantPredictedPowerUltraTermEnt> ultraList = ultraMapper.selectList(new LambdaQueryWrapper<PlantPredictedPowerUltraTermEnt>()
                        .eq(PlantPredictedPowerUltraTermEnt::getPlantId, stationId)
                        .between(PlantPredictedPowerUltraTermEnt::getDataDate, beginTime, endTime)
                        .orderByAsc(PlantPredictedPowerUltraTermEnt::getDataDate)
                );
                if (CollUtil.isNotEmpty(ultraList)) {
                    predictedMap = ultraList.stream()
                            .collect(Collectors.toMap(x -> DateUtil.parse(x.getDataDate()), PlantPredictedPowerUltraTermEnt::getPredictPower));
                } else {
                    predictedMap = Collections.emptyMap();
                }
                break;
            default:
                predictedMap = Collections.emptyMap();
        }
        return predictedMap;
    }

    /**
     * 查15分钟电站实际功率
     *
     * @param context   上下文
     * @param stationId 工作站id
     * @param beginTime 开始时间
     * @param showTime  演出时间
     * @return {@link Map }<{@link Date }, {@link BigDecimal }>
     */
    public static Map<Date, BigDecimal> getActivePowerMap15(XContext context, String stationId, Date beginTime, Date showTime) {
        String oilFieldCode = ServiceUtil.getOilFieldCode(context);
        //查实际功率
        ThirdActivePowerViewMapper powerMapper = context.getBean(ThirdActivePowerViewMapper.class);
        List<ThirdActivePowerView> powerList = powerMapper.selectAvgPower15(ThirdActivePowerView.builder()
                .stationId(stationId)
                .startTime(beginTime)
                .endTime(showTime)
                .systemSource(oilFieldCode)
                .build());
        Map<Date, BigDecimal> powerMap;
        if (CollUtil.isNotEmpty(powerList)) {
            powerMap = powerList.stream()
                    .collect(Collectors.toMap(ThirdActivePowerView::getInputTime, ThirdActivePowerView::getPhotovoltaicPower));
        } else {
            powerMap = Collections.emptyMap();
        }
        return powerMap;
    }

    /**
     * 获得预测功率输出
     *
     * @param beginTime    开始时间
     * @param endTime      结束时间
     * @param betweenDay   天之间
     * @param now          现在
     * @param predictedMap 预测地图
     * @param powerMap     功率图
     * @return {@link List }<{@link GetPredictedPowerOutput }>
     */
    public static List<GetPredictedPowerOutput> getPowerOutput(DateTime beginTime, DateTime endTime, int betweenDay, Date now,
                                                               Map<Date, BigDecimal> predictedMap, Map<Date, BigDecimal> powerMap) {
        //取时间范围
        List<DateTime> rangeToList = DateUtil.rangeToList(beginTime, endTime, DateField.MINUTE, 15);
        GetPredictedPowerOutput output;
        List<GetPredictedPowerOutput> outputs = new ArrayList<>(rangeToList.size());
        Date dataDate;
        for (DateTime dateTime : rangeToList) {
            output = new GetPredictedPowerOutput();
            //时间偏移
            if (betweenDay != 0) {
                dataDate = DateUtil.offsetDay(dateTime, betweenDay);
            } else {
                dataDate = dateTime;
            }
            output.setDataDate(dataDate);
            //匹配预测发电量
            output.setPredictPower(predictedMap.getOrDefault(dateTime, BigDecimal.ZERO));
            //匹配实际发电量
            if (DateUtil.compare(dataDate, now) > 0) {
                output.setPower(null);
            } else {
                output.setPower(powerMap.getOrDefault(dateTime, null));
            }
            outputs.add(output);
        }
        return outputs;
    }

    /**
     * 获取当前配置
     *
     * @param context 上下文
     * @return {@link GetConfigOilFieldOutput }
     */
    public static GetConfigOilFieldOutput getCurrentConfig(XContext context) {
        IConfigOilFieldCloudService bean = context.getBean(IConfigOilFieldCloudService.class);
        XSingleResult<GetConfigOilFieldOutput> currentConfig = bean.getCurrentConfig(context);
        currentConfig.throwIfFail();
        return currentConfig.getResult();
    }

    /**
     * 获取油田代码
     *
     * @param context 上下文
     * @return {@link String }
     */
    public static String getOilFieldCode(XContext context) {
        //判断部署环境
        GetConfigOilFieldOutput currentConfig = getCurrentConfig(context);
        return ObjectUtil.defaultIfNull(currentConfig.getOilFieldCode(), BusinessConstant.DEFAULT_VALUE);
    }

    /**
     * 获取所有path包含组织ID的组织
     *
     * @param context 上下文
     * @param ouId    ou id
     * @return {@link List}<{@link String}>
     */
    public static List<String> getOrgIdsByPath(XContext context, String ouId) {
        SysOrganizationCloudService organizationCloudService = context.getBean(SysOrganizationCloudService.class);
        GetAllOuListByOuIdInput ouIdInput = new GetAllOuListByOuIdInput();
        ouIdInput.setOuId(ouId);
        XListResult<GetSysOrganizationViewOutput> allListByOuId = organizationCloudService.getAllListByOuId(context, ouIdInput);
        allListByOuId.throwIfFail();
        return allListByOuId.getResult().stream()
                .map(GetSysOrganizationViewOutput::getId)
                .collect(Collectors.toList());
    }

    /**
     * 获取所有path包含组织ID的组织
     *
     * @param context   上下文
     * @param ouIdInput ou id输入
     * @return {@link List }<{@link GetSysOrganizationViewOutput }>
     */
    public static List<GetSysOrganizationViewOutput> getOrgIdsByPath(XContext context, GetAllOuListByOuIdInput ouIdInput) {
        SysOrganizationCloudService organizationCloudService = context.getBean(SysOrganizationCloudService.class);
        XListResult<GetSysOrganizationViewOutput> allListByOuId = organizationCloudService.getAllListByOuId(context, ouIdInput);
        allListByOuId.throwIfFail();
        return allListByOuId.getResult();
    }

    /**
     * 获取电源线列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link List }<{@link DynamicQueryBasePowerLineOutput }>
     */
    public static List<DynamicQueryBasePowerLineOutput> getPowerLineList(XContext context, DynamicQueryBasePowerLineInput input) {
        IBasePowerLineCloudService lineService = context.getBean(IBasePowerLineCloudService.class);
        XListResult<DynamicQueryBasePowerLineOutput> result = lineService.getBasePowerLineList(context, input);
        result.throwIfFail();
        return result.getResult();
    }

    /**
     * 查询组织列表下所有线路
     *
     * @param context  上下文
     * @param ouIdList ou id列表
     * @return {@link List }<{@link DynamicQueryBasePowerLineOutput }>
     */
    public static List<DynamicQueryBasePowerLineOutput> getBasePowerLineListByOuIdList(XContext context, Collection<String> ouIdList) {
        IBasePowerLineCloudService basePowerLineCloudService = context.getBean(IBasePowerLineCloudService.class);
        XListResult<DynamicQueryBasePowerLineOutput> lineList = basePowerLineCloudService.getBasePowerLineListByOuIdList(context, DynamicQueryBasePowerLineInput.builder()
                .ouIdList(ouIdList)
                .build());
        lineList.throwIfFail();
        return lineList.getResult();
    }

    /**
     * 查询线路列表下所有井口
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link List }<{@link DynamicQueryBasePowerLineWellheadViewOutput }>
     */
    public static List<DynamicQueryBasePowerLineWellheadViewOutput> getPowerLineWellheadList(XContext context, DynamicQueryBasePowerLineWellheadInput input) {
        IBasePowerLineCloudService service = context.getBean(IBasePowerLineCloudService.class);
        XListResult<DynamicQueryBasePowerLineWellheadViewOutput> result = service.getPowerLineWellheadList(context, input);
        result.throwIfFail();
        return result.getResult();
    }

    /**
     * 获取光伏电站列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link List }<{@link GetBasePhotovoltaicPlantCloudOutput }>
     */
    public static List<GetBasePhotovoltaicPlantCloudOutput> getBasePhotovoltaicPlantList(XContext context, GetBasePhotovoltaicPlantCloudInput input) {
        BasePhotovoltaicPlantCloudService service = context.getBean(BasePhotovoltaicPlantCloudService.class);
        XListResult<GetBasePhotovoltaicPlantCloudOutput> result = service.getBasePhotovoltaicPlantList(context, input);
        result.throwIfFail();
        return result.getResult();
    }

    /**
     * 获取井口列表
     *
     * @param context 上下文
     * @return {@link List }<{@link DynamicQueryBaseWellheadOutput }>
     */
    public static List<DynamicQueryBaseWellheadOutput> getBaseWellheadList(XContext context, DynamicQueryBaseWellheadInput input) {
        IBaseWellheadCloudService service = context.getBean(IBaseWellheadCloudService.class);
        XListResult<DynamicQueryBaseWellheadOutput> result = service.queryBaseWellheadListByParam(context, input);
        result.throwIfFail();
        return result.getResult();
    }

    /**
     * 获取输电线电站列表
     *
     * @param context 上下文
     * @param input   入参
     * @return {@link List}<{@link DynamicQueryBasePowerLinePlantViewOutput}>
     */
    public static List<DynamicQueryBasePowerLinePlantViewOutput> getPowerLinePlantList(XContext context, DynamicQueryBasePowerLinePlantInput input) {
        IBasePowerLineCloudService service = context.getBean(IBasePowerLineCloudService.class);
        XListResult<DynamicQueryBasePowerLinePlantViewOutput> result = service.getPowerLinePlantViewList(context, input);
        result.throwIfFail();
        return result.getResult();
    }

    /**
     * 计算累计减碳量
     *
     * @param photovoltaicPower 光伏发电
     * @return {@link BigDecimal }
     */
    public static BigDecimal calculateCarbonReduction(BigDecimal photovoltaicPower) {
        BigDecimal carbonReduction;
        if (photovoltaicPower.compareTo(BigDecimal.ZERO) > 0) {
            carbonReduction = photovoltaicPower
                    .multiply(BusinessConstant.BIG_DECIMAL_6_67)
                    .divide(BusinessConstant.BIG_DECIMAL_10000, 2, RoundingMode.HALF_UP);
        } else {
            carbonReduction = BigDecimal.ZERO;
        }
        return carbonReduction;
    }

    /*----------------------------------- 私有方法 -----------------------------------*/

    /**
     * 获取长庆接口Token
     *
     * @return {@link String}
     */
    private static String getCqToken(XContext context) {
        GetConfigOilFieldOutput currentConfig = getCurrentConfig(context);
        if (ObjectUtil.isNull(currentConfig) || CharSequenceUtil.isBlank(currentConfig.getThirdPartyAuthConfig())) {
            throw new XServiceException("缺少认证配置");
        }
        Map<String, String> param = JSON.parseObject(currentConfig.getThirdPartyAuthConfig(),
                new TypeReference<Map<String, String>>() {
                }
        );
        String url = XStartup.getCurrent().getProperty("third-party.chang-qing.url");
        url += ThirdPartyApiConstant.CQ_TOKEN;
        String result = XHttpUtils.postAsForm(url, param);
        JSONObject jsonObject = JSON.parseObject(result);
        String token = jsonObject.getString("token");
        if (CharSequenceUtil.isBlank(token)) {
            throw new XServiceException("获取长庆Token失败");
        }
        ServiceUtil.setThirdPartyConfigCache(context, token);
        return token;
    }

    /**
     * 设置缓存
     *
     * @param context 上下文
     * @param token   代币
     */
    private static void setThirdPartyConfigCache(XContext context, String token) {
        ThirdPartyConfigCache cache = new ThirdPartyConfigCache();
        cache.setCodeKey(ThirdPartyApiConstant.CQ_TOKEN_CACHE_KEY);
        cache.setCode(token);
        cache.setValidity(90);
        cache.setCurrentDate(DateUtil.date());
        ThirdPartyConfigCache.set(context, cache);
    }
}
